@@ -19,9 +19,8 @@
 
 #include "GeoMath.h"
 #include "canvas/IDrawContext.h"
-#include <math.h>
-#include <stdlib.h>
 #include <proj_api.h>
+#include <stdlib.h>
 
 #include <QtGui>
 #include <QtWidgets>
@@ -31,7 +30,6 @@
 
 void GPS_Math_DegMinSec_To_Deg(bool sign, const int32_t d, const int32_t m, const int32_t s, qreal &deg)
 {
-
     deg = qAbs(d) + qreal(m) / 60.0 + qreal(s) / 3600;
     if(sign)
     {
@@ -46,7 +44,7 @@ bool GPS_Math_Deg_To_DegMin(qreal v, qint32 *d, qreal *m)
 {
     bool sign = v < 0;
     qint32 deg = qAbs(v);
-    qreal  min = (qAbs(v) - deg) * 60.0;
+    qreal min = (qAbs(v) - deg) * 60.0;
 
     *d = deg;
     *m = min;
@@ -57,9 +55,9 @@ bool GPS_Math_Deg_To_DegMin(qreal v, qint32 *d, qreal *m)
 
 void GPS_Math_DegMin_To_Deg(bool sign, const qint32 d, const qreal m, qreal& deg)
 {
-
     deg = qAbs(d) + m / 60.0;
-    if(sign) {
+    if(sign)
+    {
         deg = -deg;
     }
 
@@ -82,25 +80,91 @@ void GPS_Math_Deg_To_Str(const qreal& x, const qreal& y, QString& str)
 }
 
 
+static QRegExp reCoord1("^\\s*([N|S]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s+([E|W|O]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s*$");
+
+static QRegExp reCoord2("^\\s*([N|S]){1}\\s*([0-9]+\\.[0-9]+)\\W*\\s+([E|W|O]){1}\\s*([0-9]+\\.[0-9]+)\\W*\\s*$");
+
+static QRegExp reCoord3("^\\s*([-0-9]+\\.[0-9]+)\\s+([-0-9]+\\.[0-9]+)\\s*$");
+
+static QRegExp reCoord4("^\\s*([N|S]){1}\\s*([0-9]+)\\W+([0-9]+)\\W+([0-9]+)\\W*([E|W|O]){1}\\W*([0-9]+)\\W+([0-9]+)\\W+([0-9]+)\\W*\\s*$");
+
+static QRegExp reCoord5("^\\s*([-0-9]+\\.[0-9]+)([N|S])\\s+([-0-9]+\\.[0-9]+)([W|E])\\s*$");
+
+
 bool GPS_Math_Str_To_Deg(const QString& str, qreal& lon, qreal& lat)
 {
-    QRegExp re("^\\s*([N|S]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s+([E|W]){1}\\W*([0-9]+)\\W*([0-9]+\\.[0-9]+)\\s*$");
-    if(!re.exactMatch(str)) {
-        QMessageBox::warning(0,QObject::tr("Error"),QObject::tr("Bad position format. Must be: [N|S] ddd mm.sss [W|E] ddd mm.sss"),QMessageBox::Abort,QMessageBox::Abort);
+    if(reCoord2.exactMatch(str))
+    {
+        bool signLat    = reCoord2.cap(1) == "S";
+        float absLat    = reCoord2.cap(2).toDouble();
+        lat = signLat ? -absLat : absLat;
+
+        bool signLon    = reCoord2.cap(3) == "W";
+        float absLon    = reCoord2.cap(4).toDouble();
+        lon = signLon ? -absLon : absLon;
+    }
+    else if(reCoord1.exactMatch(str))
+    {
+        bool signLat    = reCoord1.cap(1) == "S";
+        int degLat      = reCoord1.cap(2).toInt();
+        float minLat    = reCoord1.cap(3).toDouble();
+
+        GPS_Math_DegMin_To_Deg(signLat, degLat, minLat, lat);
+
+        bool signLon    = reCoord1.cap(4) == "W";
+        int degLon      = reCoord1.cap(5).toInt();
+        float minLon    = reCoord1.cap(6).toDouble();
+
+        GPS_Math_DegMin_To_Deg(signLon, degLon, minLon, lon);
+    }
+    else if(reCoord3.exactMatch(str))
+    {
+        lat             = reCoord3.cap(1).toDouble();
+        lon             = reCoord3.cap(2).toDouble();
+    }
+    else if(reCoord4.exactMatch(str))
+    {
+        bool signLat    = reCoord4.cap(1) == "S";
+        int degLat    = reCoord4.cap(2).toInt();
+        int minLat    = reCoord4.cap(3).toInt();
+        int secLat    = reCoord4.cap(4).toInt();
+
+        GPS_Math_DegMinSec_To_Deg(signLat, degLat, minLat, secLat, lat);
+
+        bool signLon    = reCoord4.cap(5) == "W";
+        int degLon    = reCoord4.cap(6).toInt();
+        int minLon    = reCoord4.cap(7).toInt();
+        int secLon    = reCoord4.cap(8).toInt();
+
+        GPS_Math_DegMinSec_To_Deg(signLon, degLon, minLon, secLon, lon);
+    }
+    else if(reCoord5.exactMatch(str))
+    {
+        bool signLon    = reCoord4.cap(4) == "W";
+        bool signLat    = reCoord4.cap(2) == "S";
+        lat             = reCoord5.cap(1).toDouble();
+        lon             = reCoord5.cap(3).toDouble();
+
+        if(signLon)
+        {
+            lon = -lon;
+        }
+        if(signLat)
+        {
+            lat = -lat;
+        }
+    }
+    else
+    {
+        QMessageBox::warning(0,QObject::tr("Error"),QObject::tr("Bad position format. Must be: \"[N|S] ddd mm.sss [W|E] ddd mm.sss\" or \"[N|S] ddd.ddd [W|E] ddd.ddd\""),QMessageBox::Ok,QMessageBox::NoButton);
         return false;
     }
 
-    bool signLat    = re.cap(1) == "S";
-    qint32 degLat      = re.cap(2).toInt();
-    qreal minLat   = re.cap(3).toDouble();
-
-    GPS_Math_DegMin_To_Deg(signLat, degLat, minLat, lat);
-
-    bool signLon    = re.cap(4) == "W";
-    qint32 degLon      = re.cap(5).toInt();
-    qreal minLon   = re.cap(6).toDouble();
-
-    GPS_Math_DegMin_To_Deg(signLon, degLon, minLon, lon);
+    if(fabs(lon) > 180.0 || fabs(lat) > 90.0)
+    {
+        QMessageBox::warning(0,QObject::tr("Error"),QObject::tr("Position values out of bounds. "),QMessageBox::Ok,QMessageBox::NoButton);
+        return false;
+    }
 
     return true;
 }
@@ -120,29 +184,32 @@ qreal GPS_Math_Distance(const qreal u1, const qreal v1, const qreal u2, const qr
     qreal a = 6378137.0, b = 6356752.3142,  f = 1.0/298.257223563;  // WGS-84 ellipsiod
     qreal L = u2 - u1;
 
-    qreal U1 = atan((1-f) * tan(v1));
-    qreal U2 = atan((1-f) * tan(v2));
-    qreal sinU1 = sin(U1), cosU1 = cos(U1);
-    qreal sinU2 = sin(U2), cosU2 = cos(U2);
+    qreal U1 = qAtan((1-f) * qTan(v1));
+    qreal U2 = qAtan((1-f) * qTan(v2));
+    qreal sinU1 = qSin(U1), cosU1 = qCos(U1);
+    qreal sinU2 = qSin(U2), cosU2 = qCos(U2);
     qreal lambda = L, lambdaP = 2*PI;
     unsigned iterLimit = 20;
 
-    while ((qAbs(lambda - lambdaP) > 1e-12) && (--iterLimit > 0)) {
-        sinLambda = sin(lambda);
-        cosLambda = cos(lambda);
-        sinSigma = sqrt((cosU2*sinLambda) * (cosU2*sinLambda) + (cosU1*sinU2-sinU1*cosU2*cosLambda) * (cosU1*sinU2-sinU1*cosU2*cosLambda));
+    while ((qAbs(lambda - lambdaP) > 1e-12) && (--iterLimit > 0))
+    {
+        sinLambda = qSin(lambda);
+        cosLambda = qCos(lambda);
+        sinSigma = qSqrt((cosU2*sinLambda) * (cosU2*sinLambda) + (cosU1*sinU2-sinU1*cosU2*cosLambda) * (cosU1*sinU2-sinU1*cosU2*cosLambda));
 
-        if (sinSigma==0){
-            return 0;  // co-incident points
+        if (sinSigma==0)
+        {
+            return 0;   // co-incident points
         }
 
         cosSigma = sinU1 * sinU2 + cosU1 * cosU2 * cosLambda;
-        sigma = atan2(sinSigma, cosSigma);
+        sigma = qAtan2(sinSigma, cosSigma);
         sinAlpha = cosU1 * cosU2 * sinLambda / sinSigma;
         cosSqAlpha = 1 - sinAlpha * sinAlpha;
         cos2SigmaM = cosSigma - 2 * sinU1 * sinU2 / cosSqAlpha;
 
-        if (isnan(cos2SigmaM)){
+        if (qIsNaN(cos2SigmaM))
+        {
             cos2SigmaM = 0;  // equatorial line: cosSqAlpha=0 (§6)
         }
 
@@ -150,16 +217,18 @@ qreal GPS_Math_Distance(const qreal u1, const qreal v1, const qreal u2, const qr
         lambdaP = lambda;
         lambda = L + (1-C) * f * sinAlpha * (sigma + C*sinSigma*(cos2SigmaM + C * cosSigma * (-1 + 2 * cos2SigmaM * cos2SigmaM)));
     }
-    if (iterLimit==0) return FP_NAN;  // formula failed to converge
-
+    if (iterLimit==0)
+    {
+        return FP_NAN;                // formula failed to converge
+    }
     qreal uSq = cosSqAlpha * (a*a - b*b) / (b*b);
     qreal A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
     qreal B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
     qreal deltaSigma = B*sinSigma*(cos2SigmaM+B/4*(cosSigma*(-1+2*cos2SigmaM*cos2SigmaM)-B/6*cos2SigmaM*(-3+4*sinSigma*sinSigma)*(-3+4*cos2SigmaM*cos2SigmaM)));
     qreal s = b*A*(sigma-deltaSigma);
 
-    a1 = atan2(cosU2 * sinLambda, cosU1 * sinU2 - sinU1 * cosU2 * cosLambda) * 360 / TWOPI;
-    a2 = atan2(cosU1 * sinLambda, -sinU1 * cosU2 + cosU1 * sinU2 * cosLambda) * 360 / TWOPI;
+    a1 = qAtan2(cosU2 * sinLambda, cosU1 * sinU2 - sinU1 * cosU2 * cosLambda) * 360 / TWOPI;
+    a2 = qAtan2(cosU1 * sinLambda, -sinU1 * cosU2 + cosU1 * sinU2 * cosLambda) * 360 / TWOPI;
     return s;
 }
 
@@ -177,29 +246,32 @@ qreal GPS_Math_Distance(const qreal u1, const qreal v1, const qreal u2, const qr
     qreal a = 6378137.0, b = 6356752.3142,  f = 1.0/298.257223563;  // WGS-84 ellipsiod
     qreal L = u2 - u1;
 
-    qreal U1 = atan((1-f) * tan(v1));
-    qreal U2 = atan((1-f) * tan(v2));
-    qreal sinU1 = sin(U1), cosU1 = cos(U1);
-    qreal sinU2 = sin(U2), cosU2 = cos(U2);
+    qreal U1 = qAtan((1-f) * qTan(v1));
+    qreal U2 = qAtan((1-f) * qTan(v2));
+    qreal sinU1 = qSin(U1), cosU1 = qCos(U1);
+    qreal sinU2 = qSin(U2), cosU2 = qCos(U2);
     qreal lambda = L, lambdaP = 2*PI;
     unsigned iterLimit = 20;
 
-    while ((qAbs(lambda - lambdaP) > 1e-12) && (--iterLimit > 0)) {
-        sinLambda = sin(lambda);
-        cosLambda = cos(lambda);
-        sinSigma = sqrt((cosU2*sinLambda) * (cosU2*sinLambda) + (cosU1*sinU2-sinU1*cosU2*cosLambda) * (cosU1*sinU2-sinU1*cosU2*cosLambda));
+    while ((qAbs(lambda - lambdaP) > 1e-12) && (--iterLimit > 0))
+    {
+        sinLambda = qSin(lambda);
+        cosLambda = qCos(lambda);
+        sinSigma = qSqrt((cosU2*sinLambda) * (cosU2*sinLambda) + (cosU1*sinU2-sinU1*cosU2*cosLambda) * (cosU1*sinU2-sinU1*cosU2*cosLambda));
 
-        if (sinSigma==0){
-            return 0;  // co-incident points
+        if (sinSigma==0)
+        {
+            return 0;   // co-incident points
         }
 
         cosSigma = sinU1 * sinU2 + cosU1 * cosU2 * cosLambda;
-        sigma = atan2(sinSigma, cosSigma);
+        sigma = qAtan2(sinSigma, cosSigma);
         sinAlpha = cosU1 * cosU2 * sinLambda / sinSigma;
         cosSqAlpha = 1 - sinAlpha * sinAlpha;
         cos2SigmaM = cosSigma - 2 * sinU1 * sinU2 / cosSqAlpha;
 
-        if (isnan(cos2SigmaM)){
+        if (qIsNaN(cos2SigmaM))
+        {
             cos2SigmaM = 0;  // equatorial line: cosSqAlpha=0 (§6)
         }
 
@@ -207,8 +279,10 @@ qreal GPS_Math_Distance(const qreal u1, const qreal v1, const qreal u2, const qr
         lambdaP = lambda;
         lambda = L + (1-C) * f * sinAlpha * (sigma + C*sinSigma*(cos2SigmaM + C * cosSigma * (-1 + 2 * cos2SigmaM * cos2SigmaM)));
     }
-    if (iterLimit==0) return FP_NAN;  // formula failed to converge
-
+    if (iterLimit==0)
+    {
+        return FP_NAN;                // formula failed to converge
+    }
     qreal uSq = cosSqAlpha * (a*a - b*b) / (b*b);
     qreal A = 1 + uSq/16384*(4096+uSq*(-768+uSq*(320-175*uSq)));
     qreal B = uSq/1024 * (256+uSq*(-128+uSq*(74-47*uSq)));
@@ -223,7 +297,7 @@ qreal GPS_Math_DistanceQuick(const qreal u1, const qreal v1, const qreal u2, con
     qreal dU = u2 - u1; // lambda
     qreal dV = v2 - v1; // roh
 
-    qreal d = 2*asin(sqrt(sin(dV/2) * sin(dV/2) + cos(v1) * cos(v2) * sin(dU/2) * sin(dU/2)));
+    qreal d = 2*qAsin(qSqrt(qSin(dV/2) * qSin(dV/2) + qCos(v1) * qCos(v2) * qSin(dU/2) * qSin(dU/2)));
 
     return 6371010 * d;
 }
@@ -231,13 +305,12 @@ qreal GPS_Math_DistanceQuick(const qreal u1, const qreal v1, const qreal u2, con
 void GPS_Math_Wpt_Projection(const qreal lon1, const qreal lat1, const qreal distance, const qreal bearing, qreal& lon2, qreal& lat2)
 {
     qreal d    = distance / 6378130.0;
-    lat2 = asin(sin(lat1) * cos(d) + cos(lat1) * sin(d) * cos(-bearing));
-    lon2 = cos(lat1) == 0 ? lon1 : fmod(lon1 - asin(sin(-bearing) * sin(d) / cos(lat1)) + PI, TWOPI) - PI;
+    lat2 = qAsin(qSin(lat1) * qCos(d) + qCos(lat1) * qSin(d) * qCos(-bearing));
+    lon2 = qCos(lat1) == 0 ? lon1 : fmod(lon1 - qAsin(qSin(-bearing) * qSin(d) / qCos(lat1)) + PI, TWOPI) - PI;
 }
 
 qreal GPS_Math_distPointLine3D(point3D& x1, point3D& x2, point3D& x0)
 {
-
     point3D v1, v2, v3, v1x2;
 
     qreal a1x2, a3;
@@ -267,28 +340,35 @@ qreal GPS_Math_distPointLine3D(point3D& x1, point3D& x2, point3D& x0)
     // |(x2 - x1)|
     a3      = v3.x*v3.x + v3.y*v3.y + v3.z*v3.z;
 
-    return sqrt(a1x2/a3);
+    return qSqrt(a1x2/a3);
 }
 
 
 struct segment
 {
-    segment(): idx1(0), idx2(0){}
-    segment(qint32 idx1, quint32 idx2) : idx1(idx1), idx2(idx2) {}
+    segment() : idx1(0), idx2(0)
+    {
+    }
+    segment(qint32 idx1, quint32 idx2) : idx1(idx1), idx2(idx2)
+    {
+    }
     qint32 idx1;
     qint32 idx2;
 };
 
 void GPS_Math_DouglasPeucker(QVector<pointDP> &line, qreal d)
 {
-    if(line.count() < 3) return;
+    if(line.count() < 3)
+    {
+        return;
+    }
 
     QStack<segment> stack;
     stack << segment(0, line.size() - 1);
 
     while(!stack.isEmpty())
     {
-        qint32 idx = -1;
+        qint32 idx = NOIDX;
         segment seg = stack.pop();
 
         pointDP& x1 = line[seg.idx1];
@@ -327,8 +407,8 @@ QPointF GPS_Math_Wpt_Projection(const QPointF& pt1, qreal distance, qreal bearin
     qreal lon1 = pt1.x();
     qreal lat1 = pt1.y();
 
-    qreal lat2 = asin(sin(lat1) * cos(d) + cos(lat1) * sin(d) * cos(-bearing));
-    qreal lon2 = cos(lat1) == 0 ? lon1 : fmod(lon1 - asin(sin(-bearing) * sin(d) / cos(lat1)) + M_PI, (2*M_PI)) - M_PI;
+    qreal lat2 = qAsin(qSin(lat1) * qCos(d) + qCos(lat1) * qSin(d) * qCos(-bearing));
+    qreal lon2 = qCos(lat1) == 0 ? lon1 : fmod(lon1 - qAsin(qSin(-bearing) * qSin(d) / qCos(lat1)) + M_PI, (2*M_PI)) - M_PI;
 
     pt2.rx() = lon2;
     pt2.ry() = lat2;
@@ -338,7 +418,6 @@ QPointF GPS_Math_Wpt_Projection(const QPointF& pt1, qreal distance, qreal bearin
 
 bool GPS_Math_LineCrossesRect(const QPointF &p1, const QPointF &p2, const QRectF &rect)
 {
-
     // the trival case
     if(rect.contains(p1) || rect.contains(p2))
     {
@@ -374,7 +453,7 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
     qreal distance;             // the distance to the polyline
     qreal shortest1 = threshold;
     qreal shortest2 = threshold;
-    qint32 idx11 = -1, idx21 = -1, idx12 = -1;
+    qint32 idx11 = NOIDX, idx21 = NOIDX, idx12 = NOIDX;
 
     QPointF pt11;
     QPointF pt21;
@@ -391,7 +470,7 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
 
         dx = p2.u - p1.u;
         dy = p2.v - p1.v;
-        d_p1_p2 = sqrt(dx * dx + dy * dy);
+        d_p1_p2 = qSqrt(dx * dx + dy * dy);
 
         // find point on line closest to pt1
         u = ((pt1.x() - p1.u) * dx + (pt1.y() - p1.v) * dy) / (d_p1_p2 * d_p1_p2);
@@ -401,7 +480,7 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
             x = p1.u + u * dx;
             y = p1.v + u * dy;
 
-            distance = sqrt((x - pt1.x())*(x - pt1.x()) + (y - pt1.y())*(y - pt1.y()));
+            distance = qSqrt((x - pt1.x())*(x - pt1.x()) + (y - pt1.y())*(y - pt1.y()));
 
             if(distance < shortest1)
             {
@@ -418,11 +497,10 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
 
         if(u >= 0.0 && u <= 1.0)
         {
-
             x = p1.u + u * dx;
             y = p1.v + u * dy;
 
-            distance = sqrt((x - pt2.x())*(x - pt2.x()) + (y - pt2.y())*(y - pt2.y()));
+            distance = qSqrt((x - pt2.x())*(x - pt2.x()) + (y - pt2.y())*(y - pt2.y()));
 
             if(distance < shortest2)
             {
@@ -435,10 +513,10 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
     }
 
     // if 1st point can't be found test for distance to both ends
-    if(idx11 == -1)
+    if(idx11 == NOIDX)
     {
         QPointF px = pixel.first();
-        distance = sqrt((qreal)((px.x() - pt1.x())*(px.x() - pt1.x()) + (px.y() - pt1.y())*(px.y() - pt1.y())));
+        distance = qSqrt((qreal)((px.x() - pt1.x())*(px.x() - pt1.x()) + (px.y() - pt1.y())*(px.y() - pt1.y())));
         if(distance < (threshold<<1))
         {
             idx11 = 0;
@@ -448,7 +526,7 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
         else
         {
             px = pixel.last();
-            distance = sqrt((qreal)((px.x() - pt1.x())*(px.x() - pt1.x()) + (px.y() - pt1.y())*(px.y() - pt1.y())));
+            distance = qSqrt((qreal)((px.x() - pt1.x())*(px.x() - pt1.x()) + (px.y() - pt1.y())*(px.y() - pt1.y())));
             if(distance < (threshold<<1))
             {
                 idx11 = pixel.size() - 2;
@@ -459,10 +537,10 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
     }
 
     // if 2nd point can't be found test for distance to both ends
-    if(idx21 == -1)
+    if(idx21 == NOIDX)
     {
         QPointF px = pixel.first();
-        distance = sqrt((qreal)((px.x() - pt2.x())*(px.x() - pt2.x()) + (px.y() - pt2.y())*(px.y() - pt2.y())));
+        distance = qSqrt((qreal)((px.x() - pt2.x())*(px.x() - pt2.x()) + (px.y() - pt2.y())*(px.y() - pt2.y())));
 
         if(distance < (threshold<<1))
         {
@@ -472,7 +550,7 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
         else
         {
             px = pixel.last();
-            distance = sqrt((qreal)((px.x() - pt2.x())*(px.x() - pt2.x()) + (px.y() - pt2.y())*(px.y() - pt2.y())));
+            distance = qSqrt((qreal)((px.x() - pt2.x())*(px.x() - pt2.x()) + (px.y() - pt2.y())*(px.y() - pt2.y())));
             if(distance < (threshold<<1))
             {
                 idx21 = pixel.size() - 2;
@@ -492,58 +570,55 @@ void GPS_Math_SubPolyline(const QPointF& pt1, const QPointF& pt2, qint32 thresho
 
 void segment_t::apply(const QPolygonF& coords, const QPolygonF& pixel, QPolygonF& segCoord, QPolygonF& segPixel, IDrawContext * context)
 {
-
     QPointF pt1 = px1;
     QPointF pt2 = px2;
 
     context->convertPx2Rad(pt1);
     context->convertPx2Rad(pt2);
 
-    if(idx11 != -1 && idx21 != -1)
+    if(idx11 != NOIDX && idx21 != NOIDX)
     {
         if(idx11 == idx21)
         {
-            segPixel.push_back(px1);
-            segPixel.push_back(px2);
-
-            segCoord.push_back(pt1);
-            segCoord.push_back(pt2);
+//            segPixel.push_back(px1);
+//            segPixel.push_back(px2);
 
+//            segCoord.push_back(pt1);
+//            segCoord.push_back(pt2);
         }
         else if(idx12 == idx21)
         {
-            segPixel.push_back(px1);
+//            segPixel.push_back(px1);
             segPixel.push_back(pixel[idx12]);
-            segPixel.push_back(px2);
+//            segPixel.push_back(px2);
 
-            segCoord.push_back(pt1);
+//            segCoord.push_back(pt1);
             segCoord.push_back(coords[idx12]);
-            segCoord.push_back(pt2);
-
+//            segCoord.push_back(pt2);
         }
         else if(idx11 < idx21)
         {
-            segPixel.push_back(px1);
-            segCoord.push_back(pt1);
+//            segPixel.push_back(px1);
+//            segCoord.push_back(pt1);
             for(int i = idx12; i <= idx21; i++)
             {
                 segPixel.push_back(pixel[i]);
                 segCoord.push_back(coords[i]);
             }
-            segPixel.push_back(px2);
-            segCoord.push_back(pt2);
+//            segPixel.push_back(px2);
+//            segCoord.push_back(pt2);
         }
         else if(idx11 > idx21)
         {
-            segPixel.push_back(px1);
-            segCoord.push_back(pt1);
+//            segPixel.push_back(px1);
+//            segCoord.push_back(pt1);
             for(int i = idx11; i > idx21; i--)
             {
                 segPixel.push_back(pixel[i]);
                 segCoord.push_back(coords[i]);
             }
-            segPixel.push_back(px2);
-            segCoord.push_back(pt2);
+//            segPixel.push_back(px2);
+//            segCoord.push_back(pt2);
         }
     }
 }
