@@ -19,17 +19,22 @@
 
 #ifndef GEOMATH_H
 #define GEOMATH_H
-#include <QString>
 #include <QPointF>
 #include <QRectF>
+#include <QString>
 
 #include <stdint.h>
 
+#include "units/IUnit.h"
+
 class QPolygonF;
 class IDrawContext;
 
 struct point3D
 {
+    point3D() : x(0), y(0), z(0)
+    {
+    }
     qreal x;
     qreal y;
     qreal z;
@@ -37,13 +42,18 @@ struct point3D
 
 struct pointDP : public point3D
 {
-    pointDP():used(true){}
+    pointDP() : used(true), idx(NOIDX)
+    {
+    }
     bool used;
+    qint32 idx;
 };
 
 struct segment_t
 {
-    segment_t() : idx11(-1), idx12(-1), idx21(-1){}
+    segment_t() : idx11(NOIDX), idx12(NOIDX), idx21(NOIDX)
+    {
+    }
 
     void apply(const QPolygonF& coords, const QPolygonF& pixel, QPolygonF& segCoord, QPolygonF& segPixel, IDrawContext * context);
 
