@@ -17,24 +17,27 @@
 **********************************************************************************************/
 
 #include "CMainWindow.h"
+#include "GeoMath.h"
 #include "canvas/CCanvas.h"
 #include "canvas/CCanvasSetup.h"
-#include "GeoMath.h"
-#include "map/CMapDraw.h"
 #include "dem/CDemDraw.h"
+#include "gis/CGisDraw.h"
+#include "gis/CGisWidget.h"
+#include "gis/ovl/CGisItemOvlArea.h"
+#include "gis/trk/CGisItemTrk.h"
 #include "grid/CGrid.h"
 #include "grid/CGridSetup.h"
-#include "units/IUnit.h"
-#include "mouse/CMouseNormal.h"
-#include "mouse/CMouseMoveWpt.h"
-#include "mouse/CMouseEditTrk.h"
+#include "helpers/CSettings.h"
+#include "map/CMapDraw.h"
 #include "mouse/CMouseEditArea.h"
+#include "mouse/CMouseEditRte.h"
+#include "mouse/CMouseEditTrk.h"
+#include "mouse/CMouseMoveWpt.h"
+#include "mouse/CMouseNormal.h"
 #include "mouse/CMouseRangeTrk.h"
-#include "gis/CGisWidget.h"
-#include "gis/CGisDraw.h"
-#include "gis/trk/CGisItemTrk.h"
-#include "gis/ovl/CGisItemOvlArea.h"
+#include "mouse/CMouseWptBubble.h"
 #include "plot/CPlotProfile.h"
+#include "units/IUnit.h"
 
 
 #include <QtWidgets>
@@ -46,23 +49,30 @@ QBrush CCanvas::brushBackWhite(QColor(255,255,255,255));
 QBrush CCanvas::brushBackYellow(QColor(0xff, 0xff, 0xcc, 0xE0));
 
 
-CCanvas::CCanvas(QWidget *parent)
+CCanvas::CCanvas(QWidget *parent, const QString &name)
     : QWidget(parent)
     , posFocus(12.00 * DEG_TO_RAD, 49.00 * DEG_TO_RAD)
     , plotTrackProfile(0)
 {
     setFocusPolicy(Qt::WheelFocus);
 
-    int count = 1;
-    while(1)
+    if(name.isEmpty())
     {
-        QString name = tr("Workspace %1").arg(count);
-        if(CMainWindow::self().findChild<CCanvas*>(name) == 0)
+        int count = 1;
+        while(1)
         {
-            setObjectName(name);
-            break;
+            QString name = tr("View %1").arg(count);
+            if(CMainWindow::self().findChild<CCanvas*>(name) == 0)
+            {
+                setObjectName(name);
+                break;
+            }
+            count++;
         }
-        count++;
+    }
+    else
+    {
+        setObjectName(name);
     }
 
     setMouseTracking(true);
@@ -96,6 +106,9 @@ CCanvas::CCanvas(QWidget *parent)
     demLoadIndicator->show();
 
     labelStatusMessages = new QLabel(this);
+    labelStatusMessages->setWordWrap(true);
+    labelStatusMessages->setMinimumWidth(300);
+    labelStatusMessages->setAlignment(Qt::AlignJustify);
     labelStatusMessages->hide();
 
     connect(map, SIGNAL(sigStartThread()), mapLoadIndicator, SLOT(show()));
@@ -113,7 +126,13 @@ CCanvas::CCanvas(QWidget *parent)
 
 CCanvas::~CCanvas()
 {
-
+    /*
+        Some mouse objects call methods from their canvas on destruction.
+        So they are better deleted now explicitely befor any other object
+        in CCanvas is destroyed.
+     */
+    delete mouse;
+    saveSizeTrackProfile();
 }
 
 void CCanvas::saveConfig(QSettings& cfg)
@@ -123,15 +142,18 @@ void CCanvas::saveConfig(QSettings& cfg)
     grid->saveConfig(cfg);
     cfg.setValue("posFocus", posFocus);
     cfg.setValue("proj", map->getProjection());
+    cfg.setValue("scales", map->getScalesType());
 }
 
 void CCanvas::loadConfig(QSettings& cfg)
 {
-    map->loadConfig(cfg);    
-    dem->loadConfig(cfg);
-    grid->loadConfig(cfg);
     posFocus = cfg.value("posFocus", posFocus).toPointF();
     setProjection(cfg.value("proj", map->getProjection()).toString());
+    setScales((CCanvas::scales_type_e)cfg.value("scales",  map->getScalesType()).toInt());
+
+    map->loadConfig(cfg);
+    dem->loadConfig(cfg);
+    grid->loadConfig(cfg);
 
     dem->zoom(map->zoom());
     gis->zoom(map->zoom());
@@ -161,7 +183,6 @@ void CCanvas::setMouseMoveWpt(CGisItemWpt& wpt)
 
 void CCanvas::setMouseEditTrk(const QPointF &pt)
 {
-
     mouse->deleteLater();
     mouse = new CMouseEditTrk(pt, gis, this);
     if(underMouse())
@@ -171,6 +192,17 @@ void CCanvas::setMouseEditTrk(const QPointF &pt)
     }
 }
 
+void CCanvas::setMouseEditRte(const QPointF &pt)
+{
+    mouse->deleteLater();
+    mouse = new CMouseEditRte(pt, gis, this);
+    if(underMouse())
+    {
+        QApplication::restoreOverrideCursor();
+        QApplication::setOverrideCursor(*mouse);
+    }
+}
+
 void CCanvas::setMouseEditTrk(CGisItemTrk& trk)
 {
     mouse->deleteLater();
@@ -215,7 +247,27 @@ void CCanvas::setMouseEditArea(CGisItemOvlArea& area)
     }
 }
 
+void CCanvas::setMouseEditRte(CGisItemRte& rte)
+{
+    mouse->deleteLater();
+    mouse = new CMouseEditRte(rte, gis, this);
+    if(underMouse())
+    {
+        QApplication::restoreOverrideCursor();
+        QApplication::setOverrideCursor(*mouse);
+    }
+}
 
+void CCanvas::setMouseWptBubble(const IGisItem::key_t& key)
+{
+    mouse->deleteLater();
+    mouse = new CMouseWptBubble(key, gis, this);
+    if(underMouse())
+    {
+        QApplication::restoreOverrideCursor();
+        QApplication::setOverrideCursor(*mouse);
+    }
+}
 
 void CCanvas::reportStatus(const QString& key, const QString& msg)
 {
@@ -231,7 +283,7 @@ void CCanvas::reportStatus(const QString& key, const QString& msg)
     QString report;
     QStringList keys = statusMessages.keys();
     keys.sort();
-    foreach(const QString& key, keys)
+    foreach(const QString &key, keys)
     {
         report += statusMessages[key] + "\n";
     }
@@ -244,7 +296,7 @@ void CCanvas::reportStatus(const QString& key, const QString& msg)
     {
         labelStatusMessages->show();
         labelStatusMessages->setText(report);
-        labelStatusMessages->adjustSize();        
+        labelStatusMessages->adjustSize();
     }
     update();
 }
@@ -254,9 +306,18 @@ void CCanvas::resizeEvent(QResizeEvent * e)
     needsRedraw = eRedrawAll;
 
     QSize s = e->size();
-    if(map) map->resize(s);
-    if(dem) dem->resize(s);
-    if(gis) gis->resize(s);
+    if(map)
+    {
+        map->resize(s);
+    }
+    if(dem)
+    {
+        dem->resize(s);
+    }
+    if(gis)
+    {
+        gis->resize(s);
+    }
 
     QWidget::resizeEvent(e);
 
@@ -269,19 +330,7 @@ void CCanvas::resizeEvent(QResizeEvent * e)
 
     labelStatusMessages->move(20,50);
 
-    if(plotTrackProfile)
-    {
-        if(s.height() < 700)
-        {
-            plotTrackProfile->resize(200,80);
-        }
-        else
-        {
-            plotTrackProfile->resize(300,120);
-        }
-
-        plotTrackProfile->move(20, height() - plotTrackProfile->height() - 20);
-    }
+    setSizeTrackProfile();
 }
 
 void CCanvas::paintEvent(QPaintEvent * e)
@@ -293,8 +342,6 @@ void CCanvas::paintEvent(QPaintEvent * e)
         return;
     }
 
-    const QRectF& r = e->rect();
-
     QPainter p;
     p.begin(this);
     USE_ANTI_ALIASING(p,true);
@@ -306,24 +353,27 @@ void CCanvas::paintEvent(QPaintEvent * e)
     // move coordinate system to center of the screen
     p.translate(width() >> 1, height() >> 1);
 
-    map->draw(p, needsRedraw, posFocus, r);
-    dem->draw(p, needsRedraw, posFocus, r);
-    gis->draw(p, needsRedraw, posFocus, r);   
+    map->draw(p, needsRedraw, posFocus);
+    dem->draw(p, needsRedraw, posFocus);
+    gis->draw(p, needsRedraw, posFocus);
 
     // restore coordinate system to default
     p.resetTransform();
     // ----- start to draw fast content -----
 
     grid->draw(p, rect());
+    if(map->isFinished() && dem->isFinished() && gis->isFinished())
+    {
+        gis->draw(p, rect());
+    }
     mouse->draw(p, needsRedraw, rect());
-    gis->draw(p, rect());
+
 
     drawStatusMessages(p);
     drawScale(p);
 
     p.end();
     needsRedraw = eRedrawNone;
-
 }
 
 void CCanvas::mousePressEvent(QMouseEvent * e)
@@ -350,6 +400,12 @@ void CCanvas::mouseReleaseEvent(QMouseEvent *e)
     QWidget::mouseReleaseEvent(e);
 }
 
+void CCanvas::mouseDoubleClickEvent(QMouseEvent * e)
+{
+    mouse->mouseDoubleClickEvent(e);
+    QWidget::mouseDoubleClickEvent(e);
+}
+
 void CCanvas::wheelEvent(QWheelEvent * e)
 {
     mouse->wheelEvent(e);
@@ -444,9 +500,8 @@ void CCanvas::drawText(const QString& str, QPainter& p, const QPoint& center, co
 
 void CCanvas::drawText(const QString& str, QPainter& p, const QPoint& center, const QColor& color, const QFont& font)
 {
-
-    QFontMetrics    fm(font);
-    QRect           r = fm.boundingRect(str);
+    QFontMetrics fm(font);
+    QRect r = fm.boundingRect(str);
 
     r.moveCenter(center);
 
@@ -466,13 +521,11 @@ void CCanvas::drawText(const QString& str, QPainter& p, const QPoint& center, co
 
     p.setPen(color);
     p.drawText(r.topLeft(),str);
-
 }
 
 
 void CCanvas::drawText(const QString& str, QPainter& p, const QRect& r, const QColor& color)
 {
-
     p.setPen(Qt::white);
     p.setFont(CMainWindow::self().getMapFont());
 
@@ -489,7 +542,6 @@ void CCanvas::drawText(const QString& str, QPainter& p, const QRect& r, const QC
 
     p.setPen(color);
     p.drawText(r,Qt::AlignCenter,str);
-
 }
 
 void CCanvas::drawStatusMessages(QPainter& p)
@@ -507,7 +559,6 @@ void CCanvas::drawStatusMessages(QPainter& p)
 
 void CCanvas::drawScale(QPainter& p)
 {
-
     if(!CMainWindow::self().isScaleVisible())
     {
         return;
@@ -530,15 +581,15 @@ void CCanvas::drawScale(QPainter& p)
 
     if(0 <= b && b < log10(3.0f))
     {
-        d = 1 * pow(10,a);
+        d = 1 * qPow(10,a);
     }
     else if(log10(3.0f) < b && b < log10(5.0f))
     {
-        d = 3 * pow(10,a);
+        d = 3 * qPow(10,a);
     }
     else
     {
-        d = 5 * pow(10,a);
+        d = 5 * qPow(10,a);
     }
 
     // step III: convert the scale length from [m] into [px]
@@ -592,11 +643,12 @@ void CCanvas::slotToolTip()
 
 void CCanvas::slotCheckTrackOnFocus()
 {
-    const QString& key = CGisItemTrk::getKeyUserFocus();
+    const IGisItem::key_t& key = CGisItemTrk::getKeyUserFocus();
 
     // any changes?
     if(key != keyTrackOnFocus)
     {
+        saveSizeTrackProfile();
         // get access to current track object
         delete plotTrackProfile;
         keyTrackOnFocus.clear();
@@ -609,17 +661,12 @@ void CCanvas::slotCheckTrackOnFocus()
         }
 
         // create new profile plot, the plot will register itself at the track
-        plotTrackProfile = new CPlotProfile(trk2, IPlot::eModeIcon, this);
-        if(height() < 700)
+        plotTrackProfile = new CPlotProfile(trk2, CMainWindow::self().profileIsWindow() ? IPlot::eModeWindow : IPlot::eModeIcon, this);
+        setSizeTrackProfile();
+        if(isVisible())
         {
-            plotTrackProfile->resize(200,80);
+            plotTrackProfile->show();
         }
-        else
-        {
-            plotTrackProfile->resize(300,120);
-        }
-        plotTrackProfile->move(20, height() - plotTrackProfile->height() - 20);
-        plotTrackProfile->show();
 
         // finally store the new key as track on focus
         keyTrackOnFocus = key;
@@ -685,6 +732,18 @@ void CCanvas::setProjection(const QString& proj)
     dem->setProjection(proj);
     gis->setProjection(proj);
 }
+void CCanvas::setScales(const scales_type_e type)
+{
+    map->setScales(type);
+    dem->setScales(type);
+    gis->setScales(type);
+}
+
+CCanvas::scales_type_e CCanvas::getScalesType()
+{
+    return map->getScalesType();
+}
+
 
 qreal CCanvas::getElevationAt(const QPointF& pos)
 {
@@ -696,6 +755,11 @@ void CCanvas::getElevationAt(const QPolygonF& pos, QPolygonF& ele)
     return dem->getElevationAt(pos, ele);
 }
 
+void CCanvas::getElevationAt(SGisLine& line)
+{
+    return dem->getElevationAt(line);
+}
+
 void CCanvas::setZoom(bool in, redraw_e& needsRedraw)
 {
     map->zoom(in, needsRedraw);
@@ -703,7 +767,100 @@ void CCanvas::setZoom(bool in, redraw_e& needsRedraw)
     gis->zoom(map->zoom());
 }
 
-bool CCanvas::findPolylineCloseBy(QPointF& pt1, QPointF& pt2, qint32 threshold, QPolygonF& polyline)
+bool CCanvas::findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qint32 threshold, QPolygonF& polyline)
 {
     return map->findPolylineCloseBy(pt1, pt2, threshold, polyline);
 }
+
+void CCanvas::saveSizeTrackProfile()
+{
+    if(plotTrackProfile == 0)
+    {
+        return;
+    }
+
+    if(plotTrackProfile->windowFlags() & Qt::Window)
+    {
+        SETTINGS;
+        cfg.beginGroup("Canvas");
+        cfg.beginGroup("Profile");
+        cfg.beginGroup(objectName());
+
+        cfg.setValue("geometry", plotTrackProfile->saveGeometry());
+
+        cfg.endGroup(); // objectName()
+        cfg.endGroup(); // Profile
+        cfg.endGroup(); // Canvas
+    }
+}
+
+void CCanvas::setSizeTrackProfile()
+{
+    if(plotTrackProfile == 0)
+    {
+        return;
+    }
+
+    if(plotTrackProfile->windowFlags() & Qt::Window)
+    {
+        SETTINGS;
+        cfg.beginGroup("Canvas");
+        cfg.beginGroup("Profile");
+        cfg.beginGroup(objectName());
+
+        if(cfg.contains("geometry"))
+        {
+            plotTrackProfile->restoreGeometry(cfg.value("geometry").toByteArray());
+        }
+        else
+        {
+            plotTrackProfile->resize(300,200);
+            plotTrackProfile->move(100,100);
+        }
+
+        cfg.endGroup(); // objectName()
+        cfg.endGroup(); // Profile
+        cfg.endGroup(); // Canvas
+    }
+    else
+    {
+        if(size().height() < 700)
+        {
+            plotTrackProfile->resize(200,80);
+        }
+        else
+        {
+            plotTrackProfile->resize(300,120);
+        }
+
+        plotTrackProfile->move(20, height() - plotTrackProfile->height() - 20);
+    }
+}
+
+void CCanvas::showProfileAsWindow(bool yes)
+{
+    if(plotTrackProfile)
+    {
+        const IGisItem::key_t key = CGisItemTrk::getKeyUserFocus();
+
+        delete plotTrackProfile;
+        keyTrackOnFocus.clear();
+
+        CGisWidget::self().focusTrkByKey(true, key);
+    }
+}
+
+void CCanvas::showProfile(bool yes)
+{
+    if(plotTrackProfile)
+    {
+        if(yes)
+        {
+            plotTrackProfile->show();
+        }
+        else
+        {
+            plotTrackProfile->hide();
+        }
+    }
+}
