@@ -16,15 +16,16 @@
 
 **********************************************************************************************/
 
-#include "dem/CDemVRT.h"
-#include "dem/CDemDraw.h"
-#include "units/IUnit.h"
+#include "CMainWindow.h"
 #include "GeoMath.h"
 #include "canvas/CCanvas.h"
+#include "dem/CDemDraw.h"
+#include "dem/CDemVRT.h"
+#include "units/IUnit.h"
 
+#include <QtWidgets>
 #include <gdal_priv.h>
 #include <ogr_spatialref.h>
-#include <QtWidgets>
 
 #define TILELIMIT 30000
 #define TILESIZEX 64
@@ -41,14 +42,14 @@ CDemVRT::CDemVRT(const QString &filename, CDemDraw *parent)
     dataset = (GDALDataset*)GDALOpen(filename.toUtf8(),GA_ReadOnly);
     if(dataset == 0)
     {
-        QMessageBox::warning(0, tr("Error..."), tr("Failed to load file: %1").arg(filename));
+        QMessageBox::warning(&CMainWindow::self(), tr("Error..."), tr("Failed to load file: %1").arg(filename));
         return;
     }
 
     if(dataset->GetRasterCount() != 1)
     {
         delete dataset; dataset = 0;
-        QMessageBox::warning(0, tr("Error..."), tr("DEM must have one band with 16bit or 32bit data."));
+        QMessageBox::warning(&CMainWindow::self(), tr("Error..."), tr("DEM must have one band with 16bit or 32bit data."));
         return;
     }
 
@@ -57,13 +58,16 @@ CDemVRT::CDemVRT(const QString &filename, CDemDraw *parent)
     if(pBand == 0)
     {
         delete dataset; dataset = 0;
-        QMessageBox::warning(0, tr("Error..."), tr("DEM must have one band with 16bit or 32bit data."));
+        QMessageBox::warning(&CMainWindow::self(), tr("Error..."), tr("DEM must have one band with 16bit or 32bit data."));
         return;
     }
 
     hasOverviews = pBand->GetOverviewCount() != 0;
     qDebug() << "has overviews" << hasOverviews;
 
+    noData = pBand->GetNoDataValue(&hasNoData);
+    qDebug() << "no data:" << hasNoData << noData;
+
     // ------- setup projection ---------------
     char str[1024] = {0};
     if(dataset->GetProjectionRef())
@@ -78,6 +82,7 @@ CDemVRT::CDemVRT(const QString &filename, CDemDraw *parent)
     qDebug() << ptr;
 
     pjsrc = pj_init_plus(ptr);
+    free(ptr);
     if(pjsrc == 0)
     {
         delete dataset; dataset = 0;
@@ -99,14 +104,13 @@ CDemVRT::CDemVRT(const QString &filename, CDemDraw *parent)
 
     if(adfGeoTransform[4] != 0.0)
     {
-        trFwd.rotate(atan(adfGeoTransform[2]/adfGeoTransform[4]));
+        trFwd.rotate(qAtan(adfGeoTransform[2]/adfGeoTransform[4]));
     }
 
     if(pj_is_latlong(pjsrc))
     {
-
-        xscale *= 111120 / 2;
-        yscale *= 111120 / 2;
+        xscale *= 111120;
+        yscale *= 111120;
         // convert to RAD to match internal notations
         trFwd = trFwd * DEG_TO_RAD;
     }
@@ -134,7 +138,10 @@ CDemVRT::~CDemVRT()
 
 qreal CDemVRT::getElevationAt(const QPointF& pos)
 {
-    if(pjsrc == 0) return NOFLOAT;
+    if(pjsrc == 0)
+    {
+        return NOFLOAT;
+    }
 
     qint16 e[4];
     QPointF pt = pos;
@@ -148,17 +155,22 @@ qreal CDemVRT::getElevationAt(const QPointF& pos)
 
     pt = trInv.map(pt);
 
-    qreal x    = pt.x() - floor(pt.x());
-    qreal y    = pt.y() - floor(pt.y());
+    qreal x    = pt.x() - qFloor(pt.x());
+    qreal y    = pt.y() - qFloor(pt.y());
 
     mutex.lock();
-    CPLErr err = dataset->RasterIO(GF_Read, floor(pt.x()), floor(pt.y()), 2, 2, &e, 2, 2, GDT_Int16, 1, 0, 0, 0, 0);
+    CPLErr err = dataset->RasterIO(GF_Read, qFloor(pt.x()), qFloor(pt.y()), 2, 2, &e, 2, 2, GDT_Int16, 1, 0, 0, 0, 0);
     mutex.unlock();
     if(err == CE_Failure)
     {
         return NOFLOAT;
     }
 
+    if(hasNoData && e[0] == noData)
+    {
+        return NOFLOAT;
+    }
+
     qreal b1 = e[0];
     qreal b2 = e[1] - e[0];
     qreal b3 = e[2] - e[0];
@@ -177,7 +189,7 @@ void CDemVRT::draw(IDrawContext::buffer_t& buf)
         return;
     }
 
-    if(!doHillshading())
+    if(!doHillshading() && !doSlopeColor())
     {
         QThread::msleep(100);
         return;
@@ -211,17 +223,41 @@ void CDemVRT::draw(IDrawContext::buffer_t& buf)
     top      = qRound(pt1.y() < pt2.y() ? pt1.y() : pt2.y());
     bottom   = qRound(pt4.y() > pt3.y() ? pt4.y() : pt3.y());
 
-    if(left <= 0) left = 1;
-    if(left >= xsize_px) left = xsize_px - 1;
+    if(left <= 0)
+    {
+        left = 1;
+    }
+    if(left >= xsize_px)
+    {
+        left = xsize_px - 1;
+    }
 
-    if(top <= 0) top  = 1;
-    if(top >= ysize_px) top = ysize_px - 1;
+    if(top <= 0)
+    {
+        top  = 1;
+    }
+    if(top >= ysize_px)
+    {
+        top = ysize_px - 1;
+    }
 
-    if(right >= xsize_px) right = xsize_px - 1;
-    if(right <= 0) right = 1;
+    if(right >= xsize_px)
+    {
+        right = xsize_px - 1;
+    }
+    if(right <= 0)
+    {
+        right = 1;
+    }
 
-    if(bottom >= ysize_px) bottom = ysize_px - 1;
-    if(bottom <= 0) bottom = 1;
+    if(bottom >= ysize_px)
+    {
+        bottom = ysize_px - 1;
+    }
+    if(bottom <= 0)
+    {
+        bottom = 1;
+    }
 
     qreal imgw = TILESIZEX;
     qreal imgh = TILESIZEY;
@@ -231,16 +267,19 @@ void CDemVRT::draw(IDrawContext::buffer_t& buf)
     /*
         As the 3x3 window will create a border of one pixel
         more data is read than displayed to compensate.
-    */
+     */
     int wp2 = w + 2;
     int hp2 = h + 2;
 
     // start to draw the map
     QPainter p(&buf.image);
     USE_ANTI_ALIASING(p,true);
-    p.setOpacity(getOpacity()/100.0);
     p.translate(-pp);
 
+    qreal o1 = getOpacity()/100.0;
+    qreal o2 = ((o1 + 0.4) >= 1.0) ? o1 : (o1 + 0.4);
+    p.setOpacity(o1);
+
     qreal nTiles = ((right - left) * (bottom - top) / (w * h));
     qDebug() << "DEM> tiles:" << nTiles;
     if(!isOutOfScale(bufferScale) && (nTiles < TILELIMIT))
@@ -310,12 +349,28 @@ void CDemVRT::draw(IDrawContext::buffer_t& buf)
 
                 if(doHillshading())
                 {
+                    QPolygonF r = l;
+
                     QImage img(w_used,h_used,QImage::Format_Indexed8);
                     img.setColorTable(graytable);
 
                     hillshading(data, w_used, h_used, img);
 
-                    drawTile(img, l, p);
+                    drawTile(img, r, p);
+                }
+
+                if(doSlopeColor())
+                {
+                    QPolygonF r = l;
+
+                    QImage img(w_used,h_used,QImage::Format_Indexed8);
+                    img.setColorTable(slopetable);
+
+                    slopecolor(data, w_used, h_used, img);
+
+                    p.setOpacity(o2);
+                    drawTile(img, r, p);
+                    p.setOpacity(o1);
                 }
             }
         }
