@@ -19,17 +19,49 @@
 #ifndef IGISLINE_H
 #define IGISLINE_H
 
+#include <QPointF>
+#include <QVector>
+
 class QPolygonF;
+class CGisDraw;
+class CDemDraw;
+struct SGisLine;
 
 class IGisLine
 {
-    public:
-        IGisLine();
-        virtual ~IGisLine();
+public:
+    IGisLine();
+    virtual ~IGisLine();
 
-        virtual void setData(const QPolygonF& line) = 0;
-        virtual void getData(QPolygonF& line) = 0;
+    struct subpt_t
+    {
+        subpt_t();
+        subpt_t(const QPointF& pt);
+        QPointF coord;
+        QPointF pixel;
+        qint32 ele;
+    };
+
+    struct point_t : public subpt_t
+    {
+        point_t()
+        {
+        }
+        point_t(const QPointF &pt);
+        void resetElevation();
+        QVector<subpt_t> subpts;
+    };
+
+    virtual void setDataFromPolyline(const SGisLine& line) = 0;
+    virtual void getPolylineFromData(SGisLine& line) = 0;
 };
 
+struct SGisLine : public QVector<IGisLine::point_t>
+{
+    void updateElevation(CDemDraw * dem);
+    void updatePixel(CGisDraw * gis);
+};
+
+
 #endif //IGISLINE_H
 
