@@ -16,10 +16,11 @@
 
 **********************************************************************************************/
 
+#include "GeoMath.h"
 #include "gis/ovl/CDetailsOvlArea.h"
 #include "gis/ovl/CGisItemOvlArea.h"
+#include "helpers/CLinksDialog.h"
 #include "helpers/CTextEditWidget.h"
-#include "GeoMath.h"
 
 
 #include <QtWidgets>
@@ -27,6 +28,7 @@
 CDetailsOvlArea::CDetailsOvlArea(CGisItemOvlArea &area, QWidget * parent)
     : QDialog(parent)
     , area(area)
+    , originator(false)
 {
     setupUi(this);
 
@@ -56,6 +58,12 @@ CDetailsOvlArea::CDetailsOvlArea(CGisItemOvlArea &area, QWidget * parent)
 
     setupGui();
 
+    if(area.isOnDevice())
+    {
+        toolLock->setDisabled(true);
+    }
+
+
     connect(comboColor, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSetColor(int)));
     connect(comboBorderWidth, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSetWidth(int)));
     connect(comboStyle, SIGNAL(currentIndexChanged(int)), this, SLOT(slotSetStyle(int)));
@@ -63,17 +71,18 @@ CDetailsOvlArea::CDetailsOvlArea(CGisItemOvlArea &area, QWidget * parent)
     connect(toolLock, SIGNAL(toggled(bool)), this, SLOT(slotChangeReadOnlyMode(bool)));
     connect(textCmtDesc, SIGNAL(anchorClicked(QUrl)), this, SLOT(slotLinkActivated(QUrl)));
     connect(labelName, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
+
+    connect(listHistory, SIGNAL(sigChanged()), this, SLOT(setupGui()));
 }
 
 CDetailsOvlArea::~CDetailsOvlArea()
 {
-
 }
 
 
 void CDetailsOvlArea::slotSetColor(int idx)
 {
-    if(area.isReadOnly())
+    if(area.isReadOnly() || originator)
     {
         return;
     }
@@ -84,7 +93,7 @@ void CDetailsOvlArea::slotSetColor(int idx)
 
 void CDetailsOvlArea::slotSetWidth(int idx)
 {
-    if(area.isReadOnly())
+    if(area.isReadOnly() || originator)
     {
         return;
     }
@@ -94,7 +103,7 @@ void CDetailsOvlArea::slotSetWidth(int idx)
 
 void CDetailsOvlArea::slotSetStyle(int idx)
 {
-    if(area.isReadOnly())
+    if(area.isReadOnly() || originator)
     {
         return;
     }
@@ -105,7 +114,7 @@ void CDetailsOvlArea::slotSetStyle(int idx)
 
 void CDetailsOvlArea::slotOpyacity(bool yes)
 {
-    if(area.isReadOnly())
+    if(area.isReadOnly() || originator)
     {
         return;
     }
@@ -124,7 +133,7 @@ void CDetailsOvlArea::slotLinkActivated(const QString& link)
 {
     if(link == "name")
     {
-        QString name = QInputDialog::getText(0, tr("Edit name..."), tr("Enter new waypoint name."), QLineEdit::Normal, area.getName());
+        QString name = QInputDialog::getText(this, tr("Edit name..."), tr("Enter new area name."), QLineEdit::Normal, area.getName());
         if(name.isEmpty())
         {
             return;
@@ -143,10 +152,9 @@ void CDetailsOvlArea::slotLinkActivated(const QUrl& url)
         dlg.setHtml(area.getComment());
         if(dlg.exec() == QDialog::Accepted)
         {
-            area.setComment(dlg.getHtml());            
+            area.setComment(dlg.getHtml());
         }
         setupGui();
-
     }
     else if(url.toString() == "description")
     {
@@ -154,7 +162,17 @@ void CDetailsOvlArea::slotLinkActivated(const QUrl& url)
         dlg.setHtml(area.getDescription());
         if(dlg.exec() == QDialog::Accepted)
         {
-            area.setDescription(dlg.getHtml());            
+            area.setDescription(dlg.getHtml());
+        }
+        setupGui();
+    }
+    else if(url.toString() == "links")
+    {
+        QList<IGisItem::link_t> links = area.getLinks();
+        CLinksDialog dlg(links, this);
+        if(dlg.exec() == QDialog::Accepted)
+        {
+            area.setLinks(links);
         }
         setupGui();
     }
@@ -165,18 +183,14 @@ void CDetailsOvlArea::slotLinkActivated(const QUrl& url)
 }
 
 
-QString CDetailsOvlArea::toLink(bool isReadOnly, const QString& href, const QString& str)
-{
-    if(isReadOnly)
-    {
-        return QString("%1").arg(str);
-    }
-
-    return QString("<a href='%1'>%2</a>").arg(href).arg(str);
-}
-
 void CDetailsOvlArea::setupGui()
 {
+    if(originator)
+    {
+        return;
+    }
+    originator = true;
+
     bool isReadOnly = area.isReadOnly();
     setWindowTitle(area.getName());
 
@@ -189,18 +203,8 @@ void CDetailsOvlArea::setupGui()
         labelTainted->hide();
     }
 
-    textHistory->clear();
-    if(!area.getHistory().isEmpty())
-    {
-        foreach(const QString& entry, area.getHistory())
-        {
-            textHistory->append(entry);
-        }
-    }
 
-    toolLock->setChecked(isReadOnly);
-
-    labelName->setText(toLink(isReadOnly, "name", area.getName()));
+    labelName->setText(IGisItem::toLink(isReadOnly, "name", area.getName(), ""));
 
     comboColor->setCurrentIndex(area.getColorIdx());
     comboColor->setEnabled(!isReadOnly);
@@ -212,35 +216,9 @@ void CDetailsOvlArea::setupGui()
     checkOpacity->setEnabled(!isReadOnly);
 
     textCmtDesc->document()->clear();
-
-    foreach(const IGisItem::link_t& link, area.getLinks())
-    {
-        QString str = QString("<p><a href='%1'>%2</a></p>").arg(link.uri.toString()).arg(link.text);
-        textCmtDesc->append(str);
-    }
-
-    textCmtDesc->append(toLink(isReadOnly, "comment", tr("<h4>Comment:</h4>")));
-    if(IGisItem::removeHtml(area.getComment()).simplified().isEmpty())
-    {
-        textCmtDesc->append(tr("<p>--- no comment ---</p>"));
-    }
-    else
-    {
-        textCmtDesc->append(area.getComment());
-    }
-
-    textCmtDesc->append(toLink(isReadOnly, "description", tr("<h4>Description:</h4>")));
-    if(IGisItem::removeHtml(area.getDescription()).simplified().isEmpty())
-    {
-        textCmtDesc->append(tr("<p>--- no description ---</p>"));
-    }
-    else
-    {
-        textCmtDesc->append(area.getDescription());
-    }
-    textCmtDesc->moveCursor (QTextCursor::Start) ;
-    textCmtDesc->ensureCursorVisible() ;
-
+    textCmtDesc->append(IGisItem::createText(isReadOnly, area.getComment(), area.getDescription(), area.getLinks()));
+    textCmtDesc->moveCursor (QTextCursor::Start);
+    textCmtDesc->ensureCursorVisible();
 
     int idx = 0;
     QList<QTreeWidgetItem*> items;
@@ -255,15 +233,17 @@ void CDetailsOvlArea::setupGui()
         // position
         GPS_Math_Deg_To_Str(pt.lon, pt.lat, str);
         item->setText(eColPosition,str);
-
         items << item;
-
     }
 
     treeWidget->clear();
     treeWidget->addTopLevelItems(items);
     treeWidget->header()->resizeSections(QHeaderView::ResizeToContents);
 
+    toolLock->setChecked(isReadOnly);
 
+    listHistory->setupHistory(area);
+
+    originator = false;
 }
 
