@@ -16,11 +16,13 @@
 
 **********************************************************************************************/
 
+#include "GeoMath.h"
+#include "gis/CGisDraw.h"
+#include "gis/CGisListWks.h"
+#include "gis/ovl/CDetailsOvlArea.h"
 #include "gis/ovl/CGisItemOvlArea.h"
 #include "gis/ovl/CScrOptOvlArea.h"
-#include "gis/ovl/CDetailsOvlArea.h"
-#include "gis/IGisProject.h"
-#include "gis/CGisDraw.h"
+#include "gis/prj/IGisProject.h"
 
 #include <QtWidgets>
 #include <proj_api.h>
@@ -31,7 +33,7 @@
 
 const QColor CGisItemOvlArea::lineColors[OVL_N_COLORS] =
 {
-     Qt::black                    // 0
+    Qt::black                     // 0
     ,Qt::darkRed                 // 1
     ,Qt::darkGreen               // 2
     ,Qt::darkYellow              // 3
@@ -52,38 +54,37 @@ const QColor CGisItemOvlArea::lineColors[OVL_N_COLORS] =
 
 const QString CGisItemOvlArea::bulletColors[OVL_N_COLORS] =
 {
-
-                                 // 0
+    // 0
     QString("://icons/8x8/bullet_black.png")
-                                 // 1
+    // 1
     ,QString("://icons/8x8/bullet_dark_red.png")
-                                 // 2
+    // 2
     ,QString("://icons/8x8/bullet_dark_green.png")
-                                 // 3
+    // 3
     ,QString("://icons/8x8/bullet_dark_yellow.png")
-                                 // 4
+    // 4
     ,QString("://icons/8x8/bullet_dark_blue.png")
-                                 // 5
+    // 5
     ,QString("://icons/8x8/bullet_dark_magenta.png")
-                                 // 6
+    // 6
     ,QString("://icons/8x8/bullet_dark_cyan.png")
-                                 // 7
+    // 7
     ,QString("://icons/8x8/bullet_gray.png")
-                                 // 8
+    // 8
     ,QString("://icons/8x8/bullet_dark_gray.png")
-                                 // 9
+    // 9
     ,QString("://icons/8x8/bullet_red.png")
-                                 // 10
+    // 10
     ,QString("://icons/8x8/bullet_green.png")
-                                 // 11
+    // 11
     ,QString("://icons/8x8/bullet_yellow.png")
-                                 // 12
+    // 12
     ,QString("://icons/8x8/bullet_blue.png")
-                                 // 13
+    // 13
     ,QString("://icons/8x8/bullet_magenta.png")
-                                 // 14
+    // 14
     ,QString("://icons/8x8/bullet_cyan.png")
-                                 // 15
+    // 15
     ,QString("://icons/8x8/bullet_white.png")
     ,QString("")                 // 16
 };
@@ -110,39 +111,56 @@ const Qt::BrushStyle CGisItemOvlArea::brushStyles[OVL_N_STYLES] =
 };
 
 
-QString CGisItemOvlArea::keyUserFocus;
+IGisItem::key_t CGisItemOvlArea::keyUserFocus;
 
-CGisItemOvlArea::CGisItemOvlArea(const QPolygonF& line, const QString &name, IGisProject * project, int idx)
+CGisItemOvlArea::CGisItemOvlArea(const SGisLine &line, const QString &name, IGisProject * project, int idx)
     : IGisItem(project, eTypeOvl, idx)
     , penForeground(Qt::blue, 3, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
     , penBackground(Qt::white, 5, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
 {
     area.name = name;
-    readLine(line);
+    readAreaDataFromGisLine(line);
 
     flags |=  eFlagCreatedInQms|eFlagWriteAllowed;
 
     setColor(str2color(""));
-    setText(1, "*");
-    setText(0, area.name);
-    setToolTip(0, getInfo());
-    genKey();
-    project->setText(1,"*");
+    setupHistory();
+    updateDecoration(eMarkChanged, eMarkNone);
 }
 
-CGisItemOvlArea::CGisItemOvlArea(const CGisItemOvlArea& parentArea, IGisProject * project, int idx)
-    : IGisItem(project, eTypeTrk, idx)
+CGisItemOvlArea::CGisItemOvlArea(const CGisItemOvlArea& parentArea, IGisProject * project, int idx, bool clone)
+    : IGisItem(project, eTypeOvl, idx)
     , penForeground(Qt::blue, 3, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
     , penBackground(Qt::white, 5, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
 {
     *this = parentArea;
+    key.project = project->getKey();
+    key.device  = project->getDeviceKey();
 
-    setText(1, "*");
-    setText(0, area.name);
-    setToolTip(0, getInfo());
-    key.clear();
-    genKey();
-    project->setText(1,"*");
+    if(clone)
+    {
+        area.name += QObject::tr("_Clone");
+        key.clear();
+        history.events.clear();
+    }
+
+    if(parentArea.isOnDevice())
+    {
+        flags |= eFlagWriteAllowed;
+    }
+    else if(!parentArea.isReadOnly())
+    {
+        flags |= eFlagWriteAllowed;
+    }
+    else
+    {
+        flags &= ~eFlagWriteAllowed;
+    }
+
+
+
+    setupHistory();
+    updateDecoration(eMarkChanged, eMarkNone);
 }
 
 CGisItemOvlArea::CGisItemOvlArea(const QDomNode &xml, IGisProject *project)
@@ -154,10 +172,26 @@ CGisItemOvlArea::CGisItemOvlArea(const QDomNode &xml, IGisProject *project)
     setColor(penForeground.color());
     readArea(xml, area);
     // --- stop read and process data ----
-    setText(0, area.name);
-    setToolTip(0, getInfo());
-    genKey();
 
+    setupHistory();
+    updateDecoration(eMarkNone, eMarkNone);
+}
+
+CGisItemOvlArea::CGisItemOvlArea(const history_t& hist, IGisProject * project)
+    : IGisItem(project, eTypeOvl, project->childCount())
+    , penForeground(Qt::blue, 3, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
+    , penBackground(Qt::white, 5, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
+{
+    history = hist;
+    loadHistory(hist.histIdxCurrent);
+}
+
+CGisItemOvlArea::CGisItemOvlArea(quint64 id, QSqlDatabase& db, IGisProject * project)
+    : IGisItem(project, eTypeOvl, NOIDX)
+    , penForeground(Qt::blue, 3, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
+    , penBackground(Qt::white, 5, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
+{
+    loadFromDb(id, db);
 }
 
 CGisItemOvlArea::~CGisItemOvlArea()
@@ -167,22 +201,16 @@ CGisItemOvlArea::~CGisItemOvlArea()
     {
         keyUserFocus.clear();
     }
-
 }
 
-void CGisItemOvlArea::genKey()
+void CGisItemOvlArea::setSymbol()
 {
-    if(key.isEmpty())
-    {
-        QCryptographicHash md5(QCryptographicHash::Md5);
-        md5.addData((const char*)&area, sizeof(area));
-        key = md5.result().toHex();
-    }
+    setColor(str2color(area.color));
 }
 
 bool CGisItemOvlArea::isCloseTo(const QPointF& pos)
 {
-    foreach(const QPointF& pt, line)
+    foreach(const QPointF &pt, line)
     {
         if((pt - pos).manhattanLength() < MIN_DIST_CLOSE_TO)
         {
@@ -195,9 +223,9 @@ bool CGisItemOvlArea::isCloseTo(const QPointF& pos)
 QPointF CGisItemOvlArea::getPointCloseBy(const QPoint& screenPos)
 {
     qint32 i    = 0;
-    qint32 idx  = -1;
-    qint32  d   = NOINT;
-    foreach(const QPointF& point, line)
+    qint32 idx  = NOIDX;
+    qint32 d    = NOINT;
+    foreach(const QPointF &point, line)
     {
         int tmp = (screenPos - point).manhattanLength();
         if(tmp < d)
@@ -216,91 +244,34 @@ QPointF CGisItemOvlArea::getPointCloseBy(const QPoint& screenPos)
     return line[idx];
 }
 
-void CGisItemOvlArea::readLine(const QPolygonF &line)
+void CGisItemOvlArea::readAreaDataFromGisLine(const SGisLine &l)
 {
     area.pts.clear();
-    area.pts.resize(line.size());
 
-    for(int i = 0; i < line.size(); i++)
+    for(int i = 0; i < l.size(); i++)
     {
-        pt_t& areapt        = area.pts[i];
-        const QPointF& pt   = line[i];
+        area.pts << pt_t();
 
-        areapt.lon = pt.x() * RAD_TO_DEG;
-        areapt.lat = pt.y() * RAD_TO_DEG;
+        pt_t& areapt      = area.pts.last();
+        const point_t& pt = l[i];
+
+        areapt.lon = pt.coord.x() * RAD_TO_DEG;
+        areapt.lat = pt.coord.y() * RAD_TO_DEG;
+
+        for(int n = 0; n < pt.subpts.size(); n++)
+        {
+            area.pts << pt_t();
+            pt_t& areapt       = area.pts.last();
+            const subpt_t& sub = pt.subpts[n];
+
+            areapt.lon = sub.coord.x() * RAD_TO_DEG;
+            areapt.lat = sub.coord.y() * RAD_TO_DEG;
+        }
     }
 
     deriveSecondaryData();
 }
 
-void CGisItemOvlArea::readArea(const QDomNode& xml, area_t& area)
-{
-    readXml(xml, "ql:name", area.name);
-    readXml(xml, "ql:cmt", area.cmt);
-    readXml(xml, "ql:desc", area.desc);
-    readXml(xml, "ql:src", area.src);
-    readXml(xml, "ql:link", area.links);
-    readXml(xml, "ql:number", area.number);
-    readXml(xml, "ql:type", area.type);
-    readXml(xml, "ql:color", area.color);
-    readXml(xml, "ql:width", area.width);
-    readXml(xml, "ql:style", area.style);
-    readXml(xml, "ql:opacity", area.opacity);
-    readXml(xml, "ql:key", key);
-    readXml(xml, "ql:flags", flags);
-    readXml(xml, history);
-
-
-
-    const QDomNodeList& xmlPts = xml.toElement().elementsByTagName("ql:point");
-    int M = xmlPts.count();
-    area.pts.resize(M);
-    for(int m = 0; m < M; ++m)
-    {
-        pt_t& pt = area.pts[m];
-        const QDomNode& xmlPt = xmlPts.item(m);
-        readWpt(xmlPt, pt);
-    }
-
-
-    setColor(str2color(area.color));
-
-    deriveSecondaryData();
-}
-
-void CGisItemOvlArea::save(QDomNode& gpx)
-{
-    QDomDocument doc = gpx.ownerDocument();
-
-    QDomElement xmlArea = doc.createElement("ql:area");
-    gpx.appendChild(xmlArea);
-
-    writeXml(xmlArea, "ql:name", area.name);
-    writeXml(xmlArea, "ql:cmt", area.cmt);
-    writeXml(xmlArea, "ql:desc", area.desc);
-    writeXml(xmlArea, "ql:src", area.src);
-    writeXml(xmlArea, "ql:link", area.links);
-    writeXml(xmlArea, "ql:number", area.number);
-    writeXml(xmlArea, "ql:type", area.type);
-    writeXml(xmlArea, "ql:color", area.color);
-    writeXml(xmlArea, "ql:width", area.width);
-    writeXml(xmlArea, "ql:style", area.style);
-    writeXml(xmlArea, "ql:opacity", area.opacity);
-    writeXml(xmlArea, "ql:key", key);
-    writeXml(xmlArea, "ql:flags", flags);
-    writeXml(xmlArea, history);
-
-
-    foreach(const pt_t& pt, area.pts)
-    {
-        QDomElement xmlPt = doc.createElement("ql:point");
-        xmlArea.appendChild(xmlPt);
-        writeWpt(xmlPt, pt);
-    }
-
-    setText(1, "");
-
-}
 
 void CGisItemOvlArea::edit()
 {
@@ -317,28 +288,66 @@ void CGisItemOvlArea::deriveSecondaryData()
     qreal south =  90;
     qreal west  =  180;
 
-    foreach(const pt_t& pt, area.pts)
+    foreach(const pt_t &pt, area.pts)
     {
-        if(pt.lon < west)  west    = pt.lon;
-        if(pt.lon > east)  east    = pt.lon;
-        if(pt.lat < south) south   = pt.lat;
-        if(pt.lat > north) north   = pt.lat;
+        if(pt.lon < west)
+        {
+            west    = pt.lon;
+        }
+        if(pt.lon > east)
+        {
+            east    = pt.lon;
+        }
+        if(pt.lat < south)
+        {
+            south   = pt.lat;
+        }
+        if(pt.lat > north)
+        {
+            north   = pt.lat;
+        }
     }
 
     boundingRect = QRectF(QPointF(west * DEG_TO_RAD, north * DEG_TO_RAD), QPointF(east * DEG_TO_RAD,south * DEG_TO_RAD));
+
+    QPolygonF line(area.pts.size());
+    for(int i = 1; i < area.pts.size(); i++)
+    {
+        qreal a1, a2, d;
+        const pt_t& pt11 = area.pts[i - 1];
+        const pt_t& pt12 = area.pts[i];
+
+        QPointF& pt21    = line[i - 1];
+        QPointF& pt22    = line[i];
+
+        d = GPS_Math_Distance(pt11.lon * DEG_TO_RAD, pt11.lat * DEG_TO_RAD, pt12.lon * DEG_TO_RAD, pt12.lat * DEG_TO_RAD, a1, a2);
+
+        pt22.rx() = pt21.x() + qCos(a1 * DEG_TO_RAD) * d;
+        pt22.ry() = pt21.y() + qSin(a1 * DEG_TO_RAD) * d;
+    }
+
+    area.area = 0;
+    int j = line.size() - 1;
+    for(int i = 0; i < line.size(); i++)
+    {
+        area.area += (line[j].x() + line[i].x())*(line[j].y() - line[i].y());
+        j = i;
+    }
+
+    area.area = qAbs(area.area/2);
 }
 
-void CGisItemOvlArea::drawItem(QPainter& p, const QRectF& viewport, QList<QRectF>& blockedAreas, CGisDraw * gis)
+void CGisItemOvlArea::drawItem(QPainter& p, const QPolygonF& viewport, QList<QRectF>& blockedAreas, CGisDraw * gis)
 {
     line.clear();
-    if(!viewport.intersects(boundingRect))
+    if(!isVisible(boundingRect, viewport, gis))
     {
         return;
     }
 
     QPointF pt1;
 
-    foreach(const pt_t& pt, area.pts)
+    foreach(const pt_t &pt, area.pts)
     {
         pt1.setX(pt.lon);
         pt1.setY(pt.lat);
@@ -348,6 +357,7 @@ void CGisItemOvlArea::drawItem(QPainter& p, const QRectF& viewport, QList<QRectF
 
     gis->convertRad2Px(line);
 
+    p.save();
     p.setOpacity(area.opacity ? 0.3 : 1.0);
 
     penBackground.setWidth(area.width + 2);
@@ -360,10 +370,10 @@ void CGisItemOvlArea::drawItem(QPainter& p, const QRectF& viewport, QList<QRectF
     p.setBrush(QBrush(color, (Qt::BrushStyle)area.style));
     p.setPen(penForeground);
     p.drawPolygon(line);
-
+    p.restore();
 }
 
-void CGisItemOvlArea::drawLabel(QPainter& p, const QRectF& viewport,QList<QRectF>& blockedAreas, const QFontMetricsF& fm, CGisDraw * gis)
+void CGisItemOvlArea::drawLabel(QPainter& p, const QPolygonF &viewport, QList<QRectF>& blockedAreas, const QFontMetricsF& fm, CGisDraw * gis)
 {
     if(line.isEmpty())
     {
@@ -386,12 +396,11 @@ void CGisItemOvlArea::drawHighlight(QPainter& p)
     }
     p.setPen(QPen(QColor(255,0,0,100),11,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
     p.drawPolygon(line);
-
 }
 
 void CGisItemOvlArea::gainUserFocus(bool yes)
 {
-    keyUserFocus = yes ? key : "";
+    keyUserFocus = yes ? key : key_t();
 }
 
 QPointF CGisItemOvlArea::getPolygonCentroid(const QPolygonF& polygon)
@@ -421,19 +430,26 @@ IScrOpt * CGisItemOvlArea::getScreenOptions(const QPoint& origin, IMouse * mouse
     return scrOpt;
 }
 
-const QString& CGisItemOvlArea::getName()
+const QString& CGisItemOvlArea::getName() const
 {
     return area.name;
 }
 
-QString CGisItemOvlArea::getInfo()
+QString CGisItemOvlArea::getInfo(bool allowEdit) const
 {
-    QString str = getName();
+    QString unit, val;
+    QString str = "<div style='font-weight: bold;'>" + getName() + "</div>";
+
+    IUnit::self().meter2area(area.area, val, unit);
+    str += "<br/>\n" + QObject::tr("Area: %1%2").arg(val).arg(unit);
 
     QString desc = removeHtml(area.desc).simplified();
     if(desc.count())
     {
-        if(!str.isEmpty()) str += "<br/>\n";
+        if(!str.isEmpty())
+        {
+            str += "<br/>\n";
+        }
 
         if(desc.count() < 200)
         {
@@ -449,7 +465,10 @@ QString CGisItemOvlArea::getInfo()
         QString cmt = removeHtml(area.cmt).simplified();
         if(cmt.count())
         {
-            if(!str.isEmpty()) str += "<br/>\n";
+            if(!str.isEmpty())
+            {
+                str += "<br/>\n";
+            }
 
             if(cmt.count() < 200)
             {
@@ -465,61 +484,66 @@ QString CGisItemOvlArea::getInfo()
     return str;
 }
 
-void CGisItemOvlArea::getData(QPolygonF& line)
-{    
-    line.clear();
-    foreach(const pt_t& pt, area.pts)
+void CGisItemOvlArea::getPolylineFromData(SGisLine &l)
+{
+    l.clear();
+    foreach(const pt_t &pt, area.pts)
     {
-        line << QPointF(pt.lon * DEG_TO_RAD, pt.lat * DEG_TO_RAD);
+        l << point_t(QPointF(pt.lon * DEG_TO_RAD, pt.lat * DEG_TO_RAD));
     }
 }
 
-void CGisItemOvlArea::setData(const QPolygonF& line)
+void CGisItemOvlArea::setDataFromPolyline(const SGisLine& l)
 {
-    readLine(line);
+    readAreaDataFromGisLine(l);
 
     flags |= eFlagTainted;
-    setText(1,"*");
-    setToolTip(0, getInfo());
-    parent()->setText(1,"*");
-    changed(QObject::tr("Changed area shape."));
+
+    changed(QObject::tr("Changed area shape."), "://icons/48x48/AreaMove.png");
+    updateDecoration(eMarkChanged, eMarkNone);
 }
 
 void CGisItemOvlArea::setName(const QString& str)
 {
-    setText(0, str);
+    setText(CGisListWks::eColumnName, str);
     area.name = str;
-    changed(QObject::tr("Changed name."));
+    changed(QObject::tr("Changed name."), "://icons/48x48/EditText.png");
 }
 
 void CGisItemOvlArea::setWidth(qint32 w)
 {
     area.width = w;
-    changed(QObject::tr("Changed border width."));
+    changed(QObject::tr("Changed border width."), "://icons/48x48/TextBold.png");
 }
 
 void CGisItemOvlArea::setStyle(qint32 s)
 {
     area.style = s;
-    changed(QObject::tr("Changed fill pattern."));
+    changed(QObject::tr("Changed fill pattern."), "://icons/48x48/Pattern.png");
 }
 
 void CGisItemOvlArea::setOpacity(bool yes)
 {
     area.opacity = yes;
-    changed(QObject::tr("Changed opacity."));
+    changed(QObject::tr("Changed opacity."), "://icons/48x48/Opacity.png");
 }
 
 void CGisItemOvlArea::setComment(const QString& str)
 {
     area.cmt = str;
-    changed(QObject::tr("Changed comment."));
+    changed(QObject::tr("Changed comment."), "://icons/48x48/EditText.png");
 }
 
 void CGisItemOvlArea::setDescription(const QString& str)
 {
     area.desc = str;
-    changed(QObject::tr("Changed description."));
+    changed(QObject::tr("Changed description."), "://icons/48x48/EditText.png");
+}
+
+void CGisItemOvlArea::setLinks(const QList<link_t>& links)
+{
+    area.links = links;
+    changed(QObject::tr("Changed links"), "://icons/48x48/Link.png");
 }
 
 
@@ -531,7 +555,7 @@ void CGisItemOvlArea::setColor(int idx)
         return;
     }
     setColor(lineColors[idx]);
-    changed(QObject::tr("Changed color"));
+    changed(QObject::tr("Changed color"), "://icons/48x48/SelectColor.png");
 }
 
 void CGisItemOvlArea::setColor(const QColor& c)
@@ -570,5 +594,5 @@ void CGisItemOvlArea::setIcon(const QString& c)
     mask.setMask( icon.createMaskFromColor( Qt::transparent ) );
     icon = mask.scaled(22,22, Qt::KeepAspectRatio, Qt::SmoothTransformation);
 
-    QTreeWidgetItem::setIcon(0,icon);
+    QTreeWidgetItem::setIcon(CGisListWks::eColumnIcon,icon);
 }
