@@ -16,17 +16,19 @@
 
 **********************************************************************************************/
 
-#include "gis/rte/CScrOptRte.h"
-#include "gis/rte/CGisItemRte.h"
-#include "gis/CGisWidget.h"
-#include "mouse/IMouse.h"
-#include "canvas/CCanvas.h"
 #include "CMainWindow.h"
+#include "canvas/CCanvas.h"
+#include "gis/CGisWidget.h"
+#include "gis/rte/CGisItemRte.h"
+#include "gis/rte/CScrOptRte.h"
+#include "mouse/IMouse.h"
 
 CScrOptRte::CScrOptRte(CGisItemRte *rte, const QPoint& point, IMouse *parent)
-    : IScrOpt(parent->getCanvas())
-    , key(rte->getKey())
+    : IScrOpt(parent)
+
 {
+    key = rte->getKey();
+
     setupUi(this);
     setOrigin(point);
     label->setFont(CMainWindow::self().getMapFont());
@@ -38,11 +40,14 @@ CScrOptRte::CScrOptRte(CGisItemRte *rte, const QPoint& point, IMouse *parent)
     show();
 
     connect(toolDelete, SIGNAL(clicked()), this, SLOT(slotDelete()));
+    connect(toolCopy, SIGNAL(clicked()), this, SLOT(slotCopy()));
+    connect(toolCalc, SIGNAL(clicked()), this, SLOT(slotCalc()));
+    connect(toolReset, SIGNAL(clicked()), this, SLOT(slotReset()));
+    connect(toolEdit, SIGNAL(clicked()), this, SLOT(slotEdit()));
 }
 
 CScrOptRte::~CScrOptRte()
 {
-
 }
 
 void CScrOptRte::slotDelete()
@@ -50,6 +55,30 @@ void CScrOptRte::slotDelete()
     CGisWidget::self().delItemByKey(key);
 }
 
+void CScrOptRte::slotCopy()
+{
+    CGisWidget::self().copyItemByKey(key);
+    deleteLater();
+}
+
+void CScrOptRte::slotCalc()
+{
+    CGisWidget::self().calcRteByKey(key);
+    deleteLater();
+}
+
+void CScrOptRte::slotReset()
+{
+    CGisWidget::self().resetRteByKey(key);
+    deleteLater();
+}
+
+void CScrOptRte::slotEdit()
+{
+    CGisWidget::self().editRteByKey(key);
+    deleteLater();
+}
+
 
 void CScrOptRte::draw(QPainter& p)
 {
@@ -63,3 +92,4 @@ void CScrOptRte::draw(QPainter& p)
 
     drawBubble2(anchor, p);
 }
+
