@@ -16,11 +16,21 @@
 
 **********************************************************************************************/
 
+#include "GeoMath.h"
 #include "gis/trk/CDetailsTrk.h"
+#include "gis/trk/filter/CFilterDelete.h"
+#include "gis/trk/filter/CFilterDouglasPeuker.h"
+#include "gis/trk/filter/CFilterMedian.h"
+#include "gis/trk/filter/CFilterNewDate.h"
+#include "gis/trk/filter/CFilterObscureDate.h"
+#include "gis/trk/filter/CFilterOffsetElevation.h"
+#include "gis/trk/filter/CFilterReplaceElevation.h"
+#include "gis/trk/filter/CFilterReset.h"
+#include "gis/trk/filter/CFilterSpeed.h"
+#include "helpers/CLinksDialog.h"
 #include "helpers/CSettings.h"
 #include "helpers/CTextEditWidget.h"
 #include "units/IUnit.h"
-#include "GeoMath.h"
 
 #include <QtWidgets>
 #include <proj_api.h>
@@ -28,9 +38,9 @@
 CDetailsTrk::CDetailsTrk(CGisItemTrk& trk, QWidget *parent)
     : QWidget(parent)
     , trk(trk)
+    , originator(false)
 {
     setupUi(this);
-    setupGui();
 
     QPixmap icon(16,8);
     for(int i=0; i < TRK_N_COLORS; ++i)
@@ -39,33 +49,88 @@ CDetailsTrk::CDetailsTrk(CGisItemTrk& trk, QWidget *parent)
         comboColor->addItem(icon,"",CGisItemTrk::lineColors[i]);
     }
 
-    lineName->setText(trk.getName());
-    comboColor->setCurrentIndex(trk.getColorIdx());
+    setupGui();
 
     plotElevation->setTrack(&trk);
     plotDistance->setTrack(&trk);
     plotSpeed->setTrack(&trk);
-    plotTrack->setTrack(&trk);
+
+
+    if(trk.isOnDevice())
+    {
+        toolLock->setDisabled(true);
+    }
+
+    QTreeWidgetItem * item, * item0;
+    item0 = new QTreeWidgetItem(treeFilter);
+    item0->setIcon(0, QIcon("://icons/48x48/PointHide.png"));
+    item0->setText(0, tr("Reduce visible track points"));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterDouglasPeuker(trk, treeFilter));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterReset(trk, treeFilter));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterDelete(trk, treeFilter));
+
+    item0 = new QTreeWidgetItem(treeFilter);
+    item0->setIcon(0, QIcon("://icons/48x48/SetEle.png"));
+    item0->setText(0, tr("Change elevation of track points"));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterMedian(trk, treeFilter));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterReplaceElevation(trk, treeFilter));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterOffsetElevation(trk, treeFilter));
+
+    item0 = new QTreeWidgetItem(treeFilter);
+    item0->setIcon(0, QIcon("://icons/48x48/Time.png"));
+    item0->setText(0, tr("Change timestamp of track points"));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterNewDate(trk, treeFilter));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterObscureDate(trk, treeFilter));
+
+    item = new QTreeWidgetItem(item0);
+    treeFilter->setItemWidget(item,0, new CFilterSpeed(trk, treeFilter));
+
+    item0 = new QTreeWidgetItem(treeFilter);
+    item0->setIcon(0, QIcon("://icons/48x48/TrkCut.png"));
+    item0->setText(0, tr("Cut track into pieces"));
+
 
     SETTINGS;
     cfg.beginGroup("TrackDetails");
     checkProfile->setChecked(cfg.value("showProfile", true).toBool());
     checkSpeed->setChecked(cfg.value("showSpeed", true).toBool());
     checkProgress->setChecked(cfg.value("showProgress", true).toBool());
+    splitter->restoreState(cfg.value("splitterSizes").toByteArray());
+    treeWidget->header()->restoreState(cfg.value("trackPointListState").toByteArray());
     cfg.endGroup();
 
     connect(checkProfile, SIGNAL(clicked()), this, SLOT(slotShowPlots()));
     connect(checkSpeed, SIGNAL(clicked()), this, SLOT(slotShowPlots()));
     connect(checkProgress, SIGNAL(clicked()), this, SLOT(slotShowPlots()));
     connect(comboColor, SIGNAL(currentIndexChanged(int)), this, SLOT(slotColorChanged(int)));
-    connect(lineName, SIGNAL(returnPressed()), this, SLOT(slotNameChanged()));
-    connect(lineName, SIGNAL(textChanged(QString)), this, SLOT(slotNameChanged(QString)));
     connect(toolLock, SIGNAL(toggled(bool)), this, SLOT(slotChangeReadOnlyMode(bool)));
     connect(treeWidget, SIGNAL(itemSelectionChanged()), this, SLOT(slotItemSelectionChanged()));
     connect(textCmtDesc, SIGNAL(anchorClicked(QUrl)), this, SLOT(slotLinkActivated(QUrl)));
+    connect(labelInfo, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
+
+    connect(plotDistance, SIGNAL(sigMouseClickState(int)), this, SLOT(slotMouseClickState(int)));
+    connect(plotElevation, SIGNAL(sigMouseClickState(int)), this, SLOT(slotMouseClickState(int)));
+    connect(plotSpeed, SIGNAL(sigMouseClickState(int)), this, SLOT(slotMouseClickState(int)));
+
+    connect(listHistory, SIGNAL(sigChanged()), this, SLOT(setupGui()));
 
     slotShowPlots();
-
 }
 
 CDetailsTrk::~CDetailsTrk()
@@ -75,26 +140,27 @@ CDetailsTrk::~CDetailsTrk()
     cfg.setValue("showProfile", checkProfile->isChecked());
     cfg.setValue("showSpeed", checkSpeed->isChecked());
     cfg.setValue("showProgress", checkProgress->isChecked());
+    cfg.setValue("splitterSizes", splitter->saveState());
+    cfg.setValue("trackPointListState", treeWidget->header()->saveState());
     cfg.endGroup();
 }
 
-QString CDetailsTrk::toLink(bool isReadOnly, const QString& href, const QString& str)
-{
-    if(isReadOnly)
-    {
-        return QString("%1").arg(str);
-    }
-
-    return QString("<a href='%1'>%2</a>").arg(href).arg(str);
-}
 
 
 void CDetailsTrk::setupGui()
 {
-    QString str, val, unit;
+    if(originator)
+    {
+        return;
+    }
+    QApplication::setOverrideCursor(Qt::WaitCursor);
+    originator = true;
 
+    QString str, val, unit;
     bool isReadOnly = trk.isReadOnly();
 
+    tabWidget->widget(2)->setEnabled(!isReadOnly);
+
     if(trk.isTainted())
     {
         labelTainted->show();
@@ -104,8 +170,8 @@ void CDetailsTrk::setupGui()
         labelTainted->hide();
     }
 
-    labelInfo->setText(trk.getInfo());
-    lineName->setEnabled(!isReadOnly);
+    labelInfo->setText(trk.getInfo(true));
+    comboColor->setCurrentIndex(trk.getColorIdx());
     comboColor->setEnabled(!isReadOnly);
     toolLock->setChecked(isReadOnly);
 
@@ -115,7 +181,6 @@ void CDetailsTrk::setupGui()
     {
         foreach(const CGisItemTrk::trkpt_t& trkpt, seg.pts)
         {
-
             QTreeWidgetItem * item = new QTreeWidgetItem();
             item->setTextAlignment(eColNum,Qt::AlignLeft);
             item->setTextAlignment(eColEle,Qt::AlignRight);
@@ -200,7 +265,6 @@ void CDetailsTrk::setupGui()
             item->setText(eColPosition,str);
 
             items << item;
-
         }
     }
 
@@ -209,51 +273,23 @@ void CDetailsTrk::setupGui()
     treeWidget->header()->resizeSections(QHeaderView::ResizeToContents);
 
     textCmtDesc->document()->clear();
-
-    foreach(const IGisItem::link_t& link, trk.getLinks())
-    {
-        QString str = QString("<p><a href='%1'>%2</a></p>").arg(link.uri.toString()).arg(link.text);
-        textCmtDesc->append(str);
-    }
-
-    textCmtDesc->append(toLink(isReadOnly, "comment", tr("<h4>Comment:</h4>")));
-    if(IGisItem::removeHtml(trk.getComment()).simplified().isEmpty())
-    {
-        textCmtDesc->append(tr("<p>--- no comment ---</p>"));
-    }
-    else
-    {
-        textCmtDesc->append(trk.getComment());
-    }
-
-    textCmtDesc->append(toLink(isReadOnly, "description", tr("<h4>Description:</h4>")));
-    if(IGisItem::removeHtml(trk.getDescription()).simplified().isEmpty())
-    {
-        textCmtDesc->append(tr("<p>--- no description ---</p>"));
-    }
-    else
-    {
-        textCmtDesc->append(trk.getDescription());
-    }
-    textCmtDesc->moveCursor (QTextCursor::Start) ;
-    textCmtDesc->ensureCursorVisible() ;
+    textCmtDesc->append(IGisItem::createText(isReadOnly, trk.getComment(), trk.getDescription(), trk.getLinks()));
+    textCmtDesc->moveCursor (QTextCursor::Start);
+    textCmtDesc->ensureCursorVisible();
 
 
-    if(!trk.getHistory().isEmpty())
-    {
-        textHistory->clear();
-        foreach(const QString& entry, trk.getHistory())
-        {
-            textHistory->append(entry);
-        }
-    }
+    plotTrack->setTrack(&trk);
+    listHistory->setupHistory(trk);
+
+    originator = false;
+    QApplication::restoreOverrideCursor();
 }
 
-void CDetailsTrk::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
+void CDetailsTrk::setMouseFocus(const CGisItemTrk::trkpt_t * pt)
 {
     if(pt != 0)
     {
-        plotTrack->setMouseMoveFocus(pt->lon, pt->lat);
+        plotTrack->setMouseFocus(pt->lon, pt->lat);
         labelInfoTrkPt->setText(trk.getInfoTrkPt(*pt));
         labelInfoProgress->setText(trk.getInfoProgress(*pt));
     }
@@ -264,6 +300,18 @@ void CDetailsTrk::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
     }
 }
 
+void CDetailsTrk::setMouseRangeFocus(const CGisItemTrk::trkpt_t * pt1, const CGisItemTrk::trkpt_t * pt2)
+{
+    if(pt1 && pt2)
+    {
+        labelInfoRange->setText(trk.getInfoRange(*pt1, *pt2));
+    }
+    else
+    {
+        labelInfoRange->setText("-\n-");
+    }
+}
+
 void CDetailsTrk::setMouseClickFocus(const CGisItemTrk::trkpt_t * pt)
 {
     if(pt != 0)
@@ -274,6 +322,17 @@ void CDetailsTrk::setMouseClickFocus(const CGisItemTrk::trkpt_t * pt)
     }
 }
 
+void CDetailsTrk::slotMouseClickState(int s)
+{
+    if(s == IPlot::eMouseClickIdle)
+    {
+        labelInfoRange->setText("-\n-");
+        plotDistance->setMouseRangeFocus(0,0);
+        plotElevation->setMouseRangeFocus(0,0);
+        plotSpeed->setMouseRangeFocus(0,0);
+    }
+}
+
 void CDetailsTrk::slotShowPlots()
 {
     if(checkProfile->isChecked())
@@ -310,47 +369,14 @@ void CDetailsTrk::slotColorChanged(int idx)
     {
         trk.setColor(idx);
     }
-    setupGui();
 }
 
 void CDetailsTrk::slotChangeReadOnlyMode(bool on)
 {
     trk.setReadOnlyMode(on);
     setupGui();
-
 }
 
-void CDetailsTrk::slotNameChanged(const QString& name)
-{
-    QPalette palette = lineName->palette();
-    if(trk.getName() != name)
-    {
-        palette.setColor(QPalette::Base, QColor(255, 128, 128));
-    }
-    else
-    {
-        palette.setColor(QPalette::Base, QColor(255, 255, 255));
-    }
-    lineName->setPalette(palette);
-}
-
-
-void CDetailsTrk::slotNameChanged()
-{
-    QString  name  = lineName->text();
-    QString _name_ = trk.getName();
-
-    QPalette palette = lineName->palette();
-
-    if(_name_ != name)
-    {
-        trk.setName(name);
-        palette.setColor(QPalette::Base, QColor(128, 255, 128));
-    }
-
-    lineName->setPalette(palette);
-    setupGui();
-}
 
 void CDetailsTrk::slotItemSelectionChanged()
 {
@@ -358,7 +384,21 @@ void CDetailsTrk::slotItemSelectionChanged()
     if(item != 0)
     {
         quint32 idx = item->text(eColNum).toUInt();
-        trk.setMouseFocusByIndex(idx, CGisItemTrk::eFocusMouseMove);
+        trk.setMouseFocusByTotalIndex(idx, CGisItemTrk::eFocusMouseMove, "CDetailsTrk");
+    }
+}
+
+void CDetailsTrk::slotLinkActivated(const QString& url)
+{
+    if(url == "name")
+    {
+        QString name = QInputDialog::getText(this, tr("Edit name..."), tr("Enter new track name."), QLineEdit::Normal, trk.getName());
+        if(name.isEmpty())
+        {
+            return;
+        }
+        trk.setName(name);
+        setupGui();
     }
 }
 
@@ -366,18 +406,17 @@ void CDetailsTrk::slotLinkActivated(const QUrl& url)
 {
     if(url.toString() == "comment")
     {
-        CTextEditWidget dlg(0);
+        CTextEditWidget dlg(this);
         dlg.setHtml(trk.getComment());
         if(dlg.exec() == QDialog::Accepted)
         {
             trk.setComment(dlg.getHtml());
         }
         setupGui();
-
     }
     else if(url.toString() == "description")
     {
-        CTextEditWidget dlg(0);
+        CTextEditWidget dlg(this);
         dlg.setHtml(trk.getDescription());
         if(dlg.exec() == QDialog::Accepted)
         {
@@ -385,6 +424,16 @@ void CDetailsTrk::slotLinkActivated(const QUrl& url)
         }
         setupGui();
     }
+    else if(url.toString() == "links")
+    {
+        QList<IGisItem::link_t> links = trk.getLinks();
+        CLinksDialog dlg(links, this);
+        if(dlg.exec() == QDialog::Accepted)
+        {
+            trk.setLinks(links);
+        }
+        setupGui();
+    }
     else
     {
         QDesktopServices::openUrl(url);
