@@ -16,43 +16,53 @@
 
 **********************************************************************************************/
 
-#include "gis/trk/CScrOptTrk.h"
-#include "gis/trk/CGisItemTrk.h"
-#include "gis/CGisWidget.h"
-#include "mouse/IMouse.h"
-#include "canvas/CCanvas.h"
 #include "CMainWindow.h"
+#include "canvas/CCanvas.h"
+#include "gis/CGisWidget.h"
+#include "gis/trk/CGisItemTrk.h"
+#include "gis/trk/CScrOptTrk.h"
+#include "mouse/IMouse.h"
 
 CScrOptTrk::CScrOptTrk(CGisItemTrk * trk, const QPoint& point, IMouse *parent)
-    : IScrOpt(parent->getCanvas())
-    , key(trk->getKey())
+    : IScrOpt(parent)
 {
-    setupUi(this);    
+    key         = trk->getKey();
+
+    setupUi(this);
     setOrigin(point);
     label->setFont(CMainWindow::self().getMapFont());
     label->setText(trk->getInfo());
     adjustSize();
 
     toolProfile->setChecked(trk->hasUserFocus());
-    toolEdit->setEnabled(!trk->isReadOnly());
 
     anchor = trk->getPointCloseBy(point);
     move(anchor.toPoint() + QPoint(-width()/2,SCR_OPT_OFFSET));
-    show();        
+    show();
+
+    bool isOnDevice = trk->isOnDevice();
+    toolCut->setDisabled(isOnDevice);
+    toolEdit->setDisabled(isOnDevice);
+    toolReverse->setDisabled(isOnDevice);
+    toolRange->setDisabled(isOnDevice);
 
     connect(toolEditDetails, SIGNAL(clicked()), this, SLOT(slotEditDetails()));
     connect(toolDelete, SIGNAL(clicked()), this, SLOT(slotDelete()));
+    connect(toolCopy, SIGNAL(clicked()), this, SLOT(slotCopy()));
     connect(toolProfile, SIGNAL(toggled(bool)), this, SLOT(slotProfile(bool)));
     connect(toolCut, SIGNAL(clicked()), this, SLOT(slotCut()));
     connect(toolEdit, SIGNAL(clicked()), this, SLOT(slotEdit()));
     connect(toolReverse, SIGNAL(clicked()), this, SLOT(slotReverse()));
     connect(toolCombine, SIGNAL(clicked()), this, SLOT(slotCombine()));
     connect(toolRange, SIGNAL(clicked()), this, SLOT(slotRange()));
+
+    // reset user focus if the track has it
+    trk->setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove, "CScrOptTrk");
+    trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseClick, "CScrOptTrk");
 }
 
 CScrOptTrk::~CScrOptTrk()
 {
-
 }
 
 void CScrOptTrk::slotDelete()
@@ -61,6 +71,12 @@ void CScrOptTrk::slotDelete()
     deleteLater();
 }
 
+void CScrOptTrk::slotCopy()
+{
+    CGisWidget::self().copyItemByKey(key);
+    deleteLater();
+}
+
 void CScrOptTrk::slotEditDetails()
 {
     CGisWidget::self().editItemByKey(key);
@@ -95,7 +111,6 @@ void CScrOptTrk::slotCombine()
 {
     CGisWidget::self().combineTrkByKey(key);
     deleteLater();
-
 }
 
 void CScrOptTrk::slotRange()
