@@ -19,24 +19,32 @@
 #ifndef CDETAILSGEOCACHE_H
 #define CDETAILSGEOCACHE_H
 
-#include <QDialog>
 #include "ui_IDetailsGeoCache.h"
+#include <QDialog>
 
 class CGisItemWpt;
+class QNetworkAccessManager;
+class QTimer;
 
 class CDetailsGeoCache : public QDialog, private Ui::IDetailsGeoCache
 {
     Q_OBJECT
-    public:
-        CDetailsGeoCache(CGisItemWpt& wpt, QWidget * parent);
-        virtual ~CDetailsGeoCache();
+public:
+    CDetailsGeoCache(CGisItemWpt& wpt, QWidget * parent);
+    virtual ~CDetailsGeoCache();
 
-    private slots:
-        void slotHintChanged(bool on);
-        void slotLinkClicked(const QUrl& url);
+private slots:
+    void slotHintChanged(bool on);
+    void slotLinkClicked(const QUrl& url);
+    void slotCollectSpoiler();
+    void slotRequestFinished(QNetworkReply * reply);
+    void slotDownloadDone();
 
-    private:
-        CGisItemWpt& wpt;
+private:
+    CGisItemWpt& wpt;
+    QNetworkAccessManager * networkManager;
+    int cntSpoiler;
+    QTimer * timerDownload;
 };
 
 #endif //CDETAILSGEOCACHE_H
