@@ -16,14 +16,16 @@
 
 **********************************************************************************************/
 
+#include "GeoMath.h"
 #include "gis/wpt/CDetailsWpt.h"
 #include "gis/wpt/CGisItemWpt.h"
-#include "GeoMath.h"
-#include "units/IUnit.h"
+#include "helpers/CElevationDialog.h"
 #include "helpers/CInputDialog.h"
+#include "helpers/CLinksDialog.h"
 #include "helpers/CPositionDialog.h"
-#include "helpers/CWptIconDialog.h"
 #include "helpers/CTextEditWidget.h"
+#include "helpers/CWptIconDialog.h"
+#include "units/IUnit.h"
 
 
 #include <QtWidgets>
@@ -32,35 +34,48 @@
 CDetailsWpt::CDetailsWpt(CGisItemWpt &wpt, QWidget *parent)
     : QDialog(parent)
     , wpt(wpt)
+    , originator(false)
 {
     setupUi(this);
+    toolAddImage->hide();
+    toolDelImage->hide();
+    photoAlbum->hide();
+
     setupGui();
+
+    if(wpt.isOnDevice())
+    {
+        toolLock->setDisabled(true);
+    }
+
     connect(labelName, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
-    connect(labelPositon, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
+    connect(labelPosition, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
     connect(labelElevation, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
     connect(labelProximity, SIGNAL(linkActivated(QString)), this, SLOT(slotLinkActivated(QString)));
     connect(textCmtDesc, SIGNAL(anchorClicked(QUrl)), this, SLOT(slotLinkActivated(QUrl)));
     connect(toolIcon, SIGNAL(clicked()), this, SLOT(slotChangeIcon()));
     connect(toolLock, SIGNAL(toggled(bool)), this, SLOT(slotChangeReadOnlyMode(bool)));
+
+    connect(listHistory, SIGNAL(sigChanged()), this, SLOT(setupGui()));
+
+    connect(toolAddImage, SIGNAL(clicked()), photoAlbum, SLOT(slotAddImage()));
+    connect(toolDelImage, SIGNAL(clicked()), photoAlbum, SLOT(slotDelImage()));
+    connect(photoAlbum, SIGNAL(sigChanged(QList<CGisItemWpt::image_t>)), this, SLOT(slotChangedImages(QList<CGisItemWpt::image_t>)));
 }
 
 CDetailsWpt::~CDetailsWpt()
 {
-
 }
 
-QString CDetailsWpt::toLink(bool isReadOnly, const QString& href, const QString& str)
-{
-    if(isReadOnly)
-    {
-        return QString("%1").arg(str);
-    }
-
-    return QString("<a href='%1'>%2</a>").arg(href).arg(str);
-}
 
 void CDetailsWpt::setupGui()
 {
+    if(originator)
+    {
+        return;
+    }
+    originator = true;
+
     setWindowTitle(wpt.getName());
 
     QString val, unit;
@@ -71,9 +86,9 @@ void CDetailsWpt::setupGui()
     bool isReadOnly = wpt.isReadOnly();
 
     toolIcon->setIcon(wpt.getIcon());
-    toolIcon->setObjectName(wpt.getIconName());   
-    labelName->setText(toLink(isReadOnly, "name", wpt.getName()));
-    labelPositon->setText(toLink(isReadOnly, "position", strPos));
+    toolIcon->setObjectName(wpt.getIconName());
+    labelName->setText(IGisItem::toLink(isReadOnly, "name", wpt.getName(), ""));
+    labelPosition->setText(IGisItem::toLink(isReadOnly, "position", strPos, ""));
 
     if(wpt.isTainted())
     {
@@ -87,21 +102,21 @@ void CDetailsWpt::setupGui()
     if(wpt.getElevation() != NOINT)
     {
         IUnit::self().meter2elevation(wpt.getElevation(), val, unit);
-        labelElevation->setText(toLink(isReadOnly, "elevation", QString("%1 %2").arg(val).arg(unit)));
+        labelElevation->setText(IGisItem::toLink(isReadOnly, "elevation", QString("%1 %2").arg(val).arg(unit), ""));
     }
     else
     {
-        labelElevation->setText(toLink(isReadOnly, "elevation", "--"));
+        labelElevation->setText(IGisItem::toLink(isReadOnly, "elevation", "--", ""));
     }
 
     if(wpt.getProximity() != NOFLOAT)
     {
         IUnit::self().meter2elevation(wpt.getProximity(), val, unit);
-        labelProximity->setText(toLink(isReadOnly, "proximity", QString("%1 %2").arg(val).arg(unit)));
+        labelProximity->setText(IGisItem::toLink(isReadOnly, "proximity", QString("%1 %2").arg(val).arg(unit), ""));
     }
     else
     {
-        labelProximity->setText(toLink(isReadOnly, "proximity", "--"));
+        labelProximity->setText(IGisItem::toLink(isReadOnly, "proximity", "--", ""));
     }
 
     if(wpt.getTime().isValid())
@@ -110,60 +125,36 @@ void CDetailsWpt::setupGui()
     }
 
     textCmtDesc->document()->clear();
-
-    foreach(const IGisItem::link_t& link, wpt.getLinks())
-    {
-        QString str = QString("<p><a href='%1'>%2</a></p>").arg(link.uri.toString()).arg(link.text);
-        textCmtDesc->append(str);
-    }
-
-    textCmtDesc->append(toLink(isReadOnly, "comment", tr("<h4>Comment:</h4>")));
-    if(IGisItem::removeHtml(wpt.getComment()).simplified().isEmpty())
-    {
-        textCmtDesc->append(tr("<p>--- no comment ---</p>"));
-    }
-    else
-    {
-        textCmtDesc->append(wpt.getComment());
-    }
-
-    textCmtDesc->append(toLink(isReadOnly, "description", tr("<h4>Description:</h4>")));
-    if(IGisItem::removeHtml(wpt.getDescription()).simplified().isEmpty())
-    {
-        textCmtDesc->append(tr("<p>--- no description ---</p>"));
-    }
-    else
-    {
-        textCmtDesc->append(wpt.getDescription());
-    }
-    textCmtDesc->moveCursor (QTextCursor::Start) ;
-    textCmtDesc->ensureCursorVisible() ;
-
-
-
-    if(!wpt.getHistory().isEmpty())
-    {
-        textHistory->clear();
-        foreach(const QString& entry, wpt.getHistory())
-        {
-            textHistory->append(entry);
-        }
-        textHistory->show();
-    }
-    else
-    {
-        textHistory->hide();
-    }
-
+    textCmtDesc->append(IGisItem::createText(isReadOnly, wpt.getComment(), wpt.getDescription(), wpt.getLinks()));
+    textCmtDesc->moveCursor (QTextCursor::Start);
+    textCmtDesc->ensureCursorVisible();
 
     toolLock->setChecked(isReadOnly);
+
+    listHistory->setupHistory(wpt);
+
+    const QList<CGisItemWpt::image_t>& images = wpt.getImages();
+    photoAlbum->reload(images);
+    toolAddImage->hide();
+    toolDelImage->hide();
+
+    if(!isReadOnly)
+    {
+        toolAddImage->show();
+        if(!images.isEmpty())
+        {
+            toolDelImage->show();
+        }
+    }
+
+    originator = false;
 }
 
 void CDetailsWpt::slotLinkActivated(const QString& link)
 {
     if(link == "name")
     {
-        QString name = QInputDialog::getText(0, tr("Edit name..."), tr("Enter new waypoint name."), QLineEdit::Normal, wpt.getName());
+        QString name = QInputDialog::getText(this, tr("Edit name..."), tr("Enter new waypoint name."), QLineEdit::Normal, wpt.getName());
         if(name.isEmpty())
         {
             return;
@@ -173,7 +164,7 @@ void CDetailsWpt::slotLinkActivated(const QString& link)
     else if(link == "elevation")
     {
         QVariant var(wpt.getElevation());
-        CInputDialog dlg(0, tr("Enter new elevation."), var, QVariant(NOINT));
+        CElevationDialog dlg(this, var, QVariant(NOINT), wpt.getPosition());
         if(dlg.exec() == QDialog::Accepted)
         {
             wpt.setElevation(var.toInt());
@@ -182,7 +173,7 @@ void CDetailsWpt::slotLinkActivated(const QString& link)
     else if(link == "proximity")
     {
         QVariant var(wpt.getProximity());
-        CInputDialog dlg(0, tr("Enter new proximity range."), var, QVariant(NOFLOAT));
+        CInputDialog dlg(this, tr("Enter new proximity range."), var, QVariant(NOFLOAT));
         if(dlg.exec() == QDialog::Accepted)
         {
             wpt.setProximity(var.toDouble());
@@ -191,7 +182,7 @@ void CDetailsWpt::slotLinkActivated(const QString& link)
     else if(link == "position")
     {
         QPointF pos = wpt.getPosition();
-        CPositionDialog dlg(0, pos);
+        CPositionDialog dlg(this, pos);
         if(dlg.exec() == QDialog::Accepted)
         {
             wpt.setPosition(pos);
@@ -205,22 +196,31 @@ void CDetailsWpt::slotLinkActivated(const QUrl& url)
 {
     if(url.toString() == "comment")
     {
-        CTextEditWidget dlg(0);
+        CTextEditWidget dlg(this);
         dlg.setHtml(wpt.getComment());
         if(dlg.exec() == QDialog::Accepted)
         {
-            wpt.setComment(dlg.getHtml());            
+            wpt.setComment(dlg.getHtml());
         }
         setupGui();
-
     }
     else if(url.toString() == "description")
     {
-        CTextEditWidget dlg(0);
+        CTextEditWidget dlg(this);
         dlg.setHtml(wpt.getDescription());
         if(dlg.exec() == QDialog::Accepted)
         {
-            wpt.setDescription(dlg.getHtml());            
+            wpt.setDescription(dlg.getHtml());
+        }
+        setupGui();
+    }
+    else if(url.toString() == "links")
+    {
+        QList<IGisItem::link_t> links = wpt.getLinks();
+        CLinksDialog dlg(links, this);
+        if(dlg.exec() == QDialog::Accepted)
+        {
+            wpt.setLinks(links);
         }
         setupGui();
     }
@@ -232,7 +232,6 @@ void CDetailsWpt::slotLinkActivated(const QUrl& url)
 
 void CDetailsWpt::slotChangeIcon()
 {
-
     if(wpt.isReadOnly())
     {
         return;
@@ -252,3 +251,9 @@ void CDetailsWpt::slotChangeReadOnlyMode(bool on)
     wpt.setReadOnlyMode(on);
     setupGui();
 }
+
+void CDetailsWpt::slotChangedImages(const QList<CGisItemWpt::image_t>& images)
+{
+    wpt.setImages(images);
+    setupGui();
+}
