@@ -19,26 +19,34 @@
 #ifndef CPOSITIONDIALOG_H
 #define CPOSITIONDIALOG_H
 
-#include <QDialog>
 #include "ui_IPositionDialog.h"
+#include <QDialog>
 
 class QPointF;
 
 class CPositionDialog : public QDialog, private Ui::IPositionDialog
 {
     Q_OBJECT
-    public:
-        CPositionDialog(QWidget * parent, QPointF &pos);
-        virtual ~CPositionDialog();
+public:
+    CPositionDialog(QWidget * parent, QPointF &pos);
+    virtual ~CPositionDialog();
 
-    public slots:
-        void accept();
+    static QRegExp reCoord1;
+    static QRegExp reCoord2;
+    static QRegExp reCoord3;
+    static QRegExp reCoord4;
+    static QRegExp reCoord5;
 
-    private slots:
-        void slotEdit(const QString& str);
+    static void getPosition(QPointF& pt, const QString &str);
 
-    private:
-        QPointF& pos;
+public slots:
+    void accept();
+
+private slots:
+    void slotEdit(const QString& str);
+
+private:
+    QPointF& pos;
 };
 
 #endif //CPOSITIONDIALOG_H
