@@ -16,20 +16,21 @@
 
 **********************************************************************************************/
 
-#include "map/IMap.h"
+#include "CMainWindow.h"
+#include "canvas/CCanvas.h"
+#include "helpers/CSettings.h"
 #include "map/CMapDraw.h"
 #include "map/CMapItem.h"
 #include "map/CMapList.h"
 #include "map/CMapPathSetup.h"
-#include "canvas/CCanvas.h"
-#include "CMainWindow.h"
-#include "helpers/CSettings.h"
+#include "map/IMap.h"
 
 #include <QtGui>
 #include <QtWidgets>
 
 
 QList<CMapDraw*> CMapDraw::maps;
+QString CMapDraw::cachePath = QDir::home().absoluteFilePath(".QMapShack/");
 QStringList CMapDraw::mapPaths;
 QStringList CMapDraw::supportedFormats = QString("*.vrt|*.jnx|*.img|*.rmap|*.wmts|*.tms").split('|');
 
@@ -49,7 +50,6 @@ CMapDraw::CMapDraw(CCanvas *parent)
 
 CMapDraw::~CMapDraw()
 {
-
     maps.removeOne(this);
 }
 
@@ -68,12 +68,32 @@ void CMapDraw::setProjection(const QString& proj)
 
 void CMapDraw::setupMapPath()
 {
-    CMapPathSetup dlg(mapPaths);
+    QStringList paths = mapPaths;
+    CMapPathSetup dlg(paths, cachePath);
     if(dlg.exec() != QDialog::Accepted)
     {
         return;
     }
 
+    setupMapPath(paths);
+}
+
+void CMapDraw::setupMapPath(const QString &path)
+{
+    if(mapPaths.contains(path))
+    {
+        return;
+    }
+
+    QStringList paths(mapPaths);
+    paths << path;
+    setupMapPath(paths);
+}
+
+void CMapDraw::setupMapPath(const QStringList& paths)
+{
+    mapPaths = paths;
+
     foreach(CMapDraw * map, maps)
     {
         QStringList keys;
@@ -86,11 +106,13 @@ void CMapDraw::setupMapPath()
 void CMapDraw::saveMapPath(QSettings& cfg)
 {
     cfg.setValue("mapPath", mapPaths);
+    cfg.setValue("cachePath", cachePath);
 }
 
 void CMapDraw::loadMapPath(QSettings& cfg)
 {
-    mapPaths = cfg.value("mapPath", mapPaths).toStringList();
+    mapPaths    = cfg.value("mapPath", mapPaths).toStringList();
+    cachePath   = cfg.value("cachePath", cachePath).toString();
 }
 
 
@@ -120,7 +142,6 @@ void CMapDraw::getInfo(const QPoint& px, QString& str)
         }
     }
     CMapItem::mutexActiveMaps.unlock();
-
 }
 
 void CMapDraw::getToolTip(const QPoint& px, QString& str)
@@ -145,13 +166,12 @@ void CMapDraw::getToolTip(const QPoint& px, QString& str)
             }
 
             item->mapfile->getToolTip(px, str);
-
         }
     }
     CMapItem::mutexActiveMaps.unlock();
 }
 
-bool CMapDraw::findPolylineCloseBy(QPointF& pt1, QPointF& pt2, qint32 threshold, QPolygonF& polyline)
+bool CMapDraw::findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qint32 threshold, QPolygonF& polyline)
 {
     if(isRunning())
     {
@@ -195,7 +215,6 @@ void CMapDraw::saveConfig(QSettings& cfg)
     cfg.setValue("active", keys);
     cfg.setValue("zoomIndex", zoomIndex);
     cfg.endGroup();
-
 }
 
 void CMapDraw::loadConfig(QSettings& cfg)
@@ -204,7 +223,14 @@ void CMapDraw::loadConfig(QSettings& cfg)
     cfgGroup = cfg.group();
     // -------------------
     cfg.beginGroup("map");
-    restoreActiveMapsList(cfg.value("active", "").toStringList());
+    if(cfgGroup.isEmpty())
+    {
+        restoreActiveMapsList(cfg.value("active", "").toStringList(), cfg);
+    }
+    else
+    {
+        restoreActiveMapsList(cfg.value("active", "").toStringList());
+    }
     int idx = cfg.value("zoomIndex",zoomIndex).toInt();
     cfg.endGroup();
 
@@ -220,11 +246,11 @@ void CMapDraw::buildMapList()
     QMutexLocker lock(&CMapItem::mutexActiveMaps);
     mapList->clear();
 
-    foreach(const QString& path, mapPaths)
+    foreach(const QString &path, mapPaths)
     {
         QDir dir(path);
         // find available maps
-        foreach(const QString& filename, dir.entryList(supportedFormats, QDir::Files|QDir::Readable, QDir::Name))
+        foreach(const QString &filename, dir.entryList(supportedFormats, QDir::Files|QDir::Readable, QDir::Name))
         {
             QFileInfo fi(filename);
 
@@ -273,6 +299,11 @@ void CMapDraw::saveActiveMapsList(QStringList& keys, QSettings& cfg)
 
 void CMapDraw::loadConfigForMapItem(CMapItem * item)
 {
+    if(cfgGroup.isEmpty())
+    {
+        return;
+    }
+
     SETTINGS;
     cfg.beginGroup(cfgGroup);
     cfg.beginGroup("map");
@@ -285,7 +316,7 @@ void CMapDraw::restoreActiveMapsList(const QStringList& keys)
 {
     QMutexLocker lock(&CMapItem::mutexActiveMaps);
 
-    foreach(const QString& key, keys)
+    foreach(const QString &key, keys)
     {
         for(int i = 0; i < mapList->count(); i++)
         {
@@ -296,7 +327,7 @@ void CMapDraw::restoreActiveMapsList(const QStringList& keys)
                 /**
                     @Note   the item will load it's configuration uppon successful activation
                             by calling loadConfigForMapItem().
-                */
+                 */
                 item->activate();
                 break;
             }
@@ -306,6 +337,31 @@ void CMapDraw::restoreActiveMapsList(const QStringList& keys)
     mapList->updateHelpText();
 }
 
+void CMapDraw::restoreActiveMapsList(const QStringList& keys, QSettings& cfg)
+{
+    QMutexLocker lock(&CMapItem::mutexActiveMaps);
+
+    foreach(const QString &key, keys)
+    {
+        for(int i = 0; i < mapList->count(); i++)
+        {
+            CMapItem * item = mapList->item(i);
+
+            if(item && item->key == key)
+            {
+                if(item->activate())
+                {
+                    item->loadConfig(cfg);
+                }
+                break;
+            }
+        }
+    }
+
+    mapList->updateHelpText();
+}
+
+
 void CMapDraw::reportStatusToCanvas(const QString& key, const QString& msg)
 {
     canvas->reportStatus(key, msg);
@@ -330,7 +386,6 @@ void CMapDraw::drawt(IDrawContext::buffer_t& currentBuffer)
             }
 
             item->mapfile->draw(currentBuffer);
-
         }
     }
     CMapItem::mutexActiveMaps.unlock();
