@@ -16,9 +16,9 @@
 
 **********************************************************************************************/
 
-#include "map/IMap.h"
 #include "map/CMapDraw.h"
 #include "map/CMapPropSetup.h"
+#include "map/IMap.h"
 #include "units/IUnit.h"
 
 #include <QtWidgets>
@@ -29,6 +29,12 @@ IMap::IMap(quint32 features, CMapDraw *parent)
     , pjsrc(0)
     , isActivated(false)
     , flagsFeature(features)
+    , showPolygons(true)
+    , showPolylines(true)
+    , showPOIs(true)
+    , cacheSizeMB(100)
+    , cacheExpiration(8)
+
 {
     pjtar = pj_init_plus("+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs");
 }
@@ -40,6 +46,35 @@ IMap::~IMap()
     delete setup;
 }
 
+void IMap::saveConfig(QSettings& cfg)
+{
+    IDrawObject::saveConfig(cfg);
+
+    if(hasFeatureVectorItems())
+    {
+        cfg.setValue("showPolygons", getShowPolygons());
+        cfg.setValue("showPolylines", getShowPolylines());
+        cfg.setValue("showPOIs", getShowPOIs());
+    }
+
+    if(hasFeatureTileCache())
+    {
+        cfg.setValue("cacheSizeMB", cacheSizeMB);
+        cfg.setValue("cacheExpiration", cacheExpiration);
+    }
+}
+
+void IMap::loadConfig(QSettings& cfg)
+{
+    IDrawObject::loadConfig(cfg);
+
+    slotSetShowPolygons(cfg.value("showPolygons", getShowPolygons()).toBool());
+    slotSetShowPolylines(cfg.value("showPolylines", getShowPolylines()).toBool());
+    slotSetShowPOIs(cfg.value("showPOIs", getShowPOIs()).toBool());
+    slotSetCacheSize(cfg.value("cacheSizeMB", getCacheSize()).toInt());
+    slotSetCacheExpiration(cfg.value("cacheExpiration", getCacheExpiration()).toInt());
+}
+
 IMapProp *IMap::getSetup()
 {
     if(setup.isNull())
@@ -79,11 +114,11 @@ void IMap::drawTile(QImage& img, QPolygonF& l, QPainter& p)
     qreal dy1   = l[0].y() - l[1].y();
     qreal dx2   = l[0].x() - l[3].x();
     qreal dy2   = l[0].y() - l[3].y();
-    qreal w    = ceil( sqrt(dx1*dx1 + dy1*dy1));
-    qreal h    = ceil( sqrt(dx2*dx2 + dy2*dy2));
+    qreal w     = qCeil( qSqrt(dx1*dx1 + dy1*dy1));
+    qreal h     = qCeil( qSqrt(dx2*dx2 + dy2*dy2));
 
     // calculate rotation. This is not really a reprojection but might be good enough for close zoom levels
-    qreal a = atan(dy1/dx1) * RAD_TO_DEG;
+    qreal a = qAtan(dy1/dx1) * RAD_TO_DEG;
 
     // finally translate, scale, rotate and draw tile
     p.save();
@@ -92,10 +127,9 @@ void IMap::drawTile(QImage& img, QPolygonF& l, QPainter& p)
     p.rotate(a);
     p.drawImage(0,0,img);
     p.restore();
-
 }
 
-bool IMap::findPolylineCloseBy(QPointF& pt1, QPointF& pt2, qint32 threshold, QPolygonF& polyline)
+bool IMap::findPolylineCloseBy(const QPointF& pt1, const QPointF& pt2, qint32 threshold, QPolygonF& polyline)
 {
     Q_UNUSED(pt1);
     Q_UNUSED(pt2);
