@@ -16,48 +16,47 @@
 
 **********************************************************************************************/
 
-#include "mouse/CMouseEditArea.h"
+#include "CMainWindow.h"
+#include "canvas/CCanvas.h"
 #include "gis/CGisWidget.h"
 #include "gis/ovl/CGisItemOvlArea.h"
-#include "canvas/CCanvas.h"
-
+#include "mouse/CMouseEditArea.h"
 
 #include <QtWidgets>
 
 CMouseEditArea::CMouseEditArea(const QPointF& point, CGisDraw * gis, CCanvas * parent)
-    : IMouseEditLine(point, gis, parent)
+    : IMouseEditLine(IGisItem::key_t(), point, gis, parent)
 {
-    cursor = cursor1 = QCursor(QPixmap(":/cursors/cursorMoveArea.png"),0,0);    
+    startNewLine(point);
+    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawMouse);
 }
 
 CMouseEditArea::CMouseEditArea(CGisItemOvlArea &area, CGisDraw * gis, CCanvas * parent)
-    : IMouseEditLine(area, gis, parent)
+    : IMouseEditLine(area.getKey(), area, gis, parent)
 {
-    cursor = cursor1 = QCursor(QPixmap(":/cursors/cursorMoveArea.png"),0,0);
-    key    = area.getKey();
-    canvas->reportStatus(key, tr("<b>Edit Area</b><br/>Select a corner point for more options.<br/>"));
+    canvas->reportStatus(key.item, tr("<b>Edit Area</b><br/>Select a function and a routing mode via the tool buttons. Next select a point of the line. Only points marked with a large square can be changed. The ones with a black dot are subpoints introduced by routing.<br/>"));
+
+    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawMouse);
 }
 
 CMouseEditArea::~CMouseEditArea()
 {
-    canvas->reportStatus(key, "");
+    canvas->reportStatus(key.item, "");
 }
 
 void CMouseEditArea::mousePressEvent(QMouseEvent * e)
 {
-    canvas->reportStatus(key, "");
+    canvas->reportStatus(key.item, "");
     IMouseEditLine::mousePressEvent(e);
 }
 
-
-void CMouseEditArea::drawLine(const QPolygonF &l, QPainter& p)
+void CMouseEditArea::drawLine(const QPolygonF &l, const QColor color, int width, QPainter& p)
 {
-    p.setPen(QPen(Qt::magenta, 5, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
+    p.setPen(QPen(color, width, Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin));
     p.setBrush(QBrush(Qt::magenta, Qt::BDiagPattern));
     p.drawPolygon(l);
 }
 
-
 IGisLine * CMouseEditArea::getGisLine()
 {
     return dynamic_cast<CGisItemOvlArea*>(CGisWidget::self().getItemByKey(key));
@@ -65,21 +64,21 @@ IGisLine * CMouseEditArea::getGisLine()
 
 void CMouseEditArea::slotAbort()
 {
-    canvas->reportStatus(key,"");
+    canvas->reportStatus(key.item,"");
     IMouseEditLine::slotAbort();
 }
 
 void CMouseEditArea::slotCopyToOrig()
 {
-    canvas->reportStatus(key,"");
+    canvas->reportStatus(key.item,"");
     IMouseEditLine::slotCopyToOrig();
 }
 
 void CMouseEditArea::slotCopyToNew()
 {
-    canvas->reportStatus(key,"");
+    canvas->reportStatus(key.item,"");
 
-    if(coords1.size() < 3)
+    if(points.size() < 3)
     {
         return;
     }
@@ -97,14 +96,14 @@ void CMouseEditArea::slotCopyToNew()
         name = area->getName();
     }
 
-    name = QInputDialog::getText(0, QObject::tr("Edit name..."), QObject::tr("Enter new track name."), QLineEdit::Normal, name);
+    name = QInputDialog::getText(&CMainWindow::self(), QObject::tr("Edit name..."), QObject::tr("Enter new area name."), QLineEdit::Normal, name);
 
     if(name.isEmpty())
     {
         return;
     }
 
-    new CGisItemOvlArea(coords1,name, project, -1);
+    new CGisItemOvlArea(points, name, project, NOIDX);
 
     canvas->resetMouse();
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
