@@ -16,39 +16,38 @@
 
 **********************************************************************************************/
 
-#include "mouse/CMouseEditTrk.h"
-#include "gis/trk/CGisItemTrk.h"
-#include "gis/CGisWidget.h"
+#include "CMainWindow.h"
 #include "canvas/CCanvas.h"
+#include "gis/CGisWidget.h"
+#include "gis/trk/CGisItemTrk.h"
+#include "mouse/CMouseEditTrk.h"
 
 #include <QtWidgets>
 
 CMouseEditTrk::CMouseEditTrk(const QPointF& point, CGisDraw * gis, CCanvas * parent)
-    : IMouseEditLine(point, gis, parent)
+    : IMouseEditLine(IGisItem::key_t(), point, gis, parent)
+    , isNewLine(true)
 {
-    cursor = cursor1 = QCursor(QPixmap(":/cursors/cursorMoveLine.png"),0,0);
+    startNewLine(point);
+    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawMouse);
 }
 
 CMouseEditTrk::CMouseEditTrk(CGisItemTrk &trk, CGisDraw * gis, CCanvas * parent)
-    : IMouseEditLine(trk, gis, parent)
+    : IMouseEditLine(trk.getKey(), trk, gis, parent)
+    , isNewLine(false)
 {
-    cursor = cursor1 = QCursor(QPixmap(":/cursors/cursorMoveLine.png"),0,0);
-    key    = trk.getKey();
+    canvas->reportStatus(key.item, tr("<b>Edit Track Points</b><br/>Select a function and a routing mode via the tool buttons. Next select a point of the line. Only points marked with a large square can be changed. The ones with a black dot are subpoints introduced by routing.<br/>"));
 
     // reset any focus the track might have.
-    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove);
-    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseClick);
-    if(trk.hasUserFocus())
-    {
-        trk.gainUserFocus(false);
-    }
+    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove, "CMouseEditTrk");
+    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseClick, "CMouseEditTrk");
+    trk.looseUserFocus();
 
-    canvas->reportStatus(key, tr("<b>Edit Track Points</b><br/>Select a track point for more options.<br/>"));
     /*
         trigger complete update of GIS components to make sure all changes to
         the originating object are reflected on the canvas
-    */
-    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+     */
+    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawMouse);
 }
 
 CMouseEditTrk::~CMouseEditTrk()
@@ -58,7 +57,7 @@ CMouseEditTrk::~CMouseEditTrk()
 
 void CMouseEditTrk::mousePressEvent(QMouseEvent * e)
 {
-    canvas->reportStatus(key, "");
+    canvas->reportStatus(key.item, "");
     IMouseEditLine::mousePressEvent(e);
 }
 
@@ -69,22 +68,32 @@ IGisLine * CMouseEditTrk::getGisLine()
 
 void CMouseEditTrk::slotAbort()
 {
-    canvas->reportStatus(key,"");
+    canvas->reportStatus(key.item,"");
     IMouseEditLine::slotAbort();
 }
 
 void CMouseEditTrk::slotCopyToOrig()
 {
-    canvas->reportStatus(key,"");
+    canvas->reportStatus(key.item,"");
+
+    if(!isNewLine)
+    {
+        int res = QMessageBox::warning(canvas, tr("Warning!"), tr("This will replace all data of the orignal by a simple line of coordinates. All other data will be lost permanently."), QMessageBox::Ok|QMessageBox::Abort, QMessageBox::Ok);
+
+        if(res != QMessageBox::Ok)
+        {
+            return;
+        }
+    }
     IMouseEditLine::slotCopyToOrig();
 }
 
 
 void CMouseEditTrk::slotCopyToNew()
 {
-    canvas->reportStatus(key,"");
+    canvas->reportStatus(key.item,"");
 
-    if(coords1.size() < 2)
+    if(points.size() < 2)
     {
         return;
     }
@@ -103,15 +112,17 @@ void CMouseEditTrk::slotCopyToNew()
         name = trk->getName();
     }
 
-    name = QInputDialog::getText(0, QObject::tr("Edit name..."), QObject::tr("Enter new track name."), QLineEdit::Normal, name);
+    name = QInputDialog::getText(&CMainWindow::self(), QObject::tr("Edit name..."), QObject::tr("Enter new track name."), QLineEdit::Normal, name);
 
     if(name.isEmpty())
     {
         return;
     }
 
-    new CGisItemTrk(coords1,name, project, -1);
+    CMainWindow::self().getEelevationAt(points);
+
+    new CGisItemTrk(points,name, project, NOIDX);
 
     canvas->resetMouse();
-    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);            
+    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
