@@ -16,14 +16,14 @@
 
 **********************************************************************************************/
 
-#include "mouse/CMouseMoveWpt.h"
-#include "gis/wpt/CGisItemWpt.h"
+#include "GeoMath.h"
+#include "canvas/CCanvas.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisWidget.h"
 #include "gis/WptIcons.h"
-#include "canvas/CCanvas.h"
+#include "gis/wpt/CGisItemWpt.h"
+#include "mouse/CMouseMoveWpt.h"
 #include "units/IUnit.h"
-#include "GeoMath.h"
 
 #include <QtWidgets>
 #include <proj_api.h>
@@ -31,18 +31,17 @@
 CMouseMoveWpt::CMouseMoveWpt(CGisItemWpt &wpt, CGisDraw * gis, CCanvas *parent)
     : IMouse(gis, parent)
 {
-    cursor  = QCursor(QPixmap(":/cursors/cursorMoveWpt.png"),0,0);
-    key     = wpt.getKey();
-    icon    = getWptIconByName(wpt.getIconName(), focus);
-    origPos = wpt.getPosition() * DEG_TO_RAD;
+    cursor      = QCursor(QPixmap(":/cursors/cursorMoveWpt.png"),0,0);
+    key         = wpt.getKey();
+    icon        = getWptIconByName(wpt.getIconName(), focus);
+    origPos     = wpt.getPosition() * DEG_TO_RAD;
 }
 
 CMouseMoveWpt::~CMouseMoveWpt()
 {
-
 }
 
-void CMouseMoveWpt::draw(QPainter& p,  bool needsRedraw, const QRect &rect)
+void CMouseMoveWpt::draw(QPainter& p,  CCanvas::redraw_e needsRedraw, const QRect &rect)
 {
     QString val, unit, str;
     qreal d, a1 = 0, a2 = 0;
@@ -57,8 +56,8 @@ void CMouseMoveWpt::draw(QPainter& p,  bool needsRedraw, const QRect &rect)
     gis->convertRad2Px(p2);
 
 
-    QPointF p11 = p1 + QPoint(17 * cos((a1 - 90) * DEG_TO_RAD), 17 * sin((a1 - 90) * DEG_TO_RAD));
-    QPointF p22 = p2 + QPoint(21 * cos((a2 + 90) * DEG_TO_RAD), 21 * sin((a2 + 90) * DEG_TO_RAD));
+    QPointF p11 = p1 + QPoint(17 * qCos((a1 - 90) * DEG_TO_RAD), 17 * qSin((a1 - 90) * DEG_TO_RAD));
+    QPointF p22 = p2 + QPoint(21 * qCos((a2 + 90) * DEG_TO_RAD), 21 * qSin((a2 + 90) * DEG_TO_RAD));
 
 
     QPen pen(Qt::darkBlue, 3);
@@ -84,7 +83,6 @@ void CMouseMoveWpt::draw(QPainter& p,  bool needsRedraw, const QRect &rect)
 
     p.drawPixmap(p1 - focus, icon);
     p.drawPixmap(p2 - focus, icon);
-
 }
 
 void CMouseMoveWpt::slotPanCanvas()
