@@ -16,17 +16,17 @@
 
 **********************************************************************************************/
 
+#include "GeoMath.h"
+#include "canvas/CCanvas.h"
+#include "gis/CGisDraw.h"
+#include "gis/CGisWidget.h"
+#include "gis/prj/IGisProject.h"
+#include "gis/rte/CGisItemRte.h"
+#include "gis/trk/CGisItemTrk.h"
+#include "gis/wpt/CGisItemWpt.h"
+#include "helpers/CFadingIcon.h"
 #include "mouse/CMouseNormal.h"
 #include "mouse/CScrOptUnclutter.h"
-#include "canvas/CCanvas.h"
-#include "gis/CGisWidget.h"
-#include "gis/wpt/CGisItemWpt.h"
-#include "gis/trk/CGisItemTrk.h"
-#include "gis/rte/CGisItemRte.h"
-#include "gis/CGisWidget.h"
-#include "gis/CGisDraw.h"
-#include "gis/IGisProject.h"
-#include "GeoMath.h"
 
 #include <QtWidgets>
 
@@ -36,24 +36,39 @@ CMouseNormal::CMouseNormal(CGisDraw *gis, CCanvas *canvas)
     , stateItemSel(eStateIdle)
 {
     cursor = QCursor(QPixmap(":/cursors/cursorMoveMap.png"),0,0);
-    screenUnclutter = new CScrOptUnclutter(canvas);
+    screenUnclutter = new CScrOptUnclutter(this);
 
     menu = new QMenu(canvas);
     menu->addAction(QIcon("://icons/32x32/AddWpt.png"), tr("Add Waypoint"), this, SLOT(slotAddWpt()));
     menu->addAction(QIcon("://icons/32x32/AddTrk.png"), tr("Add Track"), this, SLOT(slotAddTrk()));
+    menu->addAction(QIcon("://icons/32x32/AddRte.png"), tr("Add Route"), this, SLOT(slotAddRte()));
     menu->addAction(QIcon("://icons/32x32/AddArea.png"), tr("Add Area"), this, SLOT(slotAddArea()));
+
     menu->addSeparator();
     menu->addAction(QIcon("://icons/32x32/Copy.png"), tr("Copy position"), this, SLOT(slotCopyPosition()));
 }
 
 CMouseNormal::~CMouseNormal()
 {
+}
 
+void CMouseNormal::stopTracking()
+{
+    const IGisItem::key_t& key = CGisItemTrk::getKeyUserFocus();
+    if(!key.item.isEmpty())
+    {
+        CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
+        if(trk != 0)
+        {
+            trk->setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove, "CMouseNormal");
+        }
+    }
 }
 
 void CMouseNormal::mousePressEvent(QMouseEvent * e)
 {
     point = e->pos();
+
     if(e->button() == Qt::LeftButton)
     {
         lastPos     = e->pos();
@@ -66,7 +81,6 @@ void CMouseNormal::mousePressEvent(QMouseEvent * e)
     {
         QPoint p = canvas->mapToGlobal(point);
         menu->exec(p);
-
     }
 }
 
@@ -78,7 +92,7 @@ void CMouseNormal::mouseMoveEvent(QMouseEvent * e)
         screenItemOption->mouseMoveEvent(e);
     }
 
-    point = e->pos();    
+    point = e->pos();
 
     if(mapMove)
     {
@@ -94,25 +108,28 @@ void CMouseNormal::mouseMoveEvent(QMouseEvent * e)
     {
         switch(stateItemSel)
         {
-            case eStateIdle:
-            case eStateHooverSingle:
-            case eStateHooverMultiple:
+        case eStateIdle:
+            CGisWidget::self().mouseMove(point);
+
+        case eStateHooverSingle:
+        case eStateHooverMultiple:
+        {
+            const IGisItem::key_t& key = CGisItemTrk::getKeyUserFocus();
+            if(!key.item.isEmpty())
             {
-                const QString& key = CGisItemTrk::getKeyUserFocus();
-                if(!key.isEmpty())
+                CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
+                if(trk != 0)
                 {
-                    CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(CGisItemTrk::getKeyUserFocus()));
-                    if(trk != 0)
-                    {
-                        trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseMove);
-                    }
+                    trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseMove, "CMouseNormal");
                 }
-                break;
             }
-            default:;
+            break;
+        }
+
+        default:;
         }
         canvas->displayInfo(point);
-        canvas->update();                
+        canvas->update();
     }
 }
 
@@ -120,75 +137,73 @@ void CMouseNormal::mouseReleaseEvent(QMouseEvent *e)
 {
     point = e->pos();
     if(e->button() == Qt::LeftButton)
-    {      
+    {
         if(!mapDidMove)
         {
             switch(stateItemSel)
             {
-                case eStateHooverSingle:
-                {
-                    stateItemSel = eStateIdle;
+            case eStateHooverSingle:
+            {
+                stateItemSel = eStateIdle;
 
-                    IGisItem * item = CGisWidget::self().getItemByKey(screenUnclutter->getItemKey());
+                IGisItem * item = CGisWidget::self().getItemByKey(screenUnclutter->getItemKey());
+                if(item)
+                {
+                    item->treeWidget()->collapseAll();
+                    item->treeWidget()->setCurrentItem(item);
+                    item->treeWidget()->scrollToItem(item, QAbstractItemView::PositionAtCenter);
+
+                    if(setScreenOption(point, item))
+                    {
+                        stateItemSel = eStateShowItemOptions;
+                    }
+                    stopTracking();
+                }
+                break;
+            }
+
+            case eStateHooverMultiple:
+            {
+                screenUnclutter->setOrigin(e->pos());
+                stateItemSel = eStateUnclutterMultiple;
+                stopTracking();
+                break;
+            }
+
+            case eStateUnclutterMultiple:
+            {
+                const CScrOptUnclutter::item_t * scrOpt = screenUnclutter->selectItem(point);
+                if(scrOpt != 0)
+                {
+                    IGisItem * item = CGisWidget::self().getItemByKey(scrOpt->key);
+                    screenUnclutter->clear(); // CAUTION!! this will delete the object scrOpt is pointing to.
+                    scrOpt = 0;
                     if(item)
                     {
                         item->treeWidget()->collapseAll();
                         item->treeWidget()->setCurrentItem(item);
                         item->treeWidget()->scrollToItem(item, QAbstractItemView::PositionAtCenter);
 
-                        delete screenItemOption;
-                        screenItemOption = item->getScreenOptions(point, this);
-
-                        if(!screenItemOption.isNull())
+                        if(setScreenOption(screenUnclutter->getOrigin(), item))
                         {
                             stateItemSel = eStateShowItemOptions;
-
-                        }
-                    }                    
-                    break;
-                }
-                case eStateHooverMultiple:
-                {
-                    screenUnclutter->setOrigin(e->pos());
-                    stateItemSel = eStateUnclutterMultiple;
-                    break;
-                }
-                case eStateUnclutterMultiple:
-                {
-                    const CScrOptUnclutter::item_t * scrOpt = screenUnclutter->selectItem(point);
-                    if(scrOpt != 0)
-                    {
-                        QString key = scrOpt->key;
-                        screenUnclutter->clear();
-
-                        IGisItem * item = CGisWidget::self().getItemByKey(key);
-                        if(item)
-                        {
-                            item->treeWidget()->collapseAll();
-                            item->treeWidget()->setCurrentItem(item);
-                            item->treeWidget()->scrollToItem(item, QAbstractItemView::PositionAtCenter);
-
-                            delete screenItemOption;
-                            screenItemOption = item->getScreenOptions(screenUnclutter->getOrigin(), this);
-
-                            if(!screenItemOption.isNull())
-                            {
-                                stateItemSel = eStateShowItemOptions;
-                                break;
-                            }
+                            break;
                         }
                     }
-                    screenUnclutter->clear();
-                    stateItemSel = eStateIdle;
-                    break;
                 }
-                case eStateShowItemOptions:
-                {
-                    delete screenItemOption;
-                    stateItemSel = eStateIdle;
-                    break;
-                }
-                default:;
+                screenUnclutter->clear();
+                stateItemSel = eStateIdle;
+                break;
+            }
+
+            case eStateShowItemOptions:
+            {
+                delete screenItemOption;
+                stateItemSel = eStateIdle;
+                break;
+            }
+
+            default:;
             }
             canvas->update();
         }
@@ -197,6 +212,15 @@ void CMouseNormal::mouseReleaseEvent(QMouseEvent *e)
     }
 }
 
+void CMouseNormal::mouseDoubleClickEvent(QMouseEvent *e)
+{
+    if(stateItemSel == eStateIdle)
+    {
+        const IGisItem::key_t& key = CGisItemTrk::getKeyUserFocus();
+        CGisWidget::self().focusTrkByKey(false, key);
+    }
+}
+
 void CMouseNormal::wheelEvent(QWheelEvent * e)
 {
     screenUnclutter->clear();
@@ -204,8 +228,22 @@ void CMouseNormal::wheelEvent(QWheelEvent * e)
     stateItemSel = eStateIdle;
 }
 
+bool CMouseNormal::setScreenOption(const QPoint& pt, IGisItem * item)
+{
+    CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(item);
+    if(trk && trk->setMouseFocusByPoint(pt, CGisItemTrk::eFocusMouseClick, "CMouseNormal") == NOPOINTF)
+    {
+        new CFadingIcon(pt, "://icons/48x48/NoGo.png", canvas);
+        return false;
+    }
 
-void CMouseNormal::draw(QPainter& p, bool needsRedraw, const QRect &rect)
+    delete screenItemOption;
+    screenItemOption = item->getScreenOptions(pt, this);
+
+    return !screenItemOption.isNull();
+}
+
+void CMouseNormal::draw(QPainter& p, CCanvas::redraw_e needsRedraw, const QRect &rect)
 {
     // no mouse interaction while gis thread is running
     if(gis->isRunning())
@@ -215,87 +253,77 @@ void CMouseNormal::draw(QPainter& p, bool needsRedraw, const QRect &rect)
 
     switch(stateItemSel)
     {
-        case eStateIdle:
-        case eStateHooverSingle:
-        case eStateHooverMultiple:
+    case eStateIdle:
+    case eStateHooverSingle:
+    case eStateHooverMultiple:
+    {
+        /*
+            Collect and draw items close to the last mouse position in the draw method.
+
+            This might be a bit odd but there are two reasons:
+
+            1) Multiple update events are combined by the event loop. Thus multiple mouse move
+               events are reduced a single paint event. As getItemsByPos() is quite cycle
+               intense this seems like a good idea.
+
+            2) The list of items passed back by getItemsByPos() must not be stored. That is why
+               the list has to be generated within the draw handler to access the item's drawHighlight()
+               method.
+
+         */
+        screenUnclutter->clear();
+
+        QList<IGisItem*> items;
+        CGisWidget::self().getItemsByPos(point, items);
+
+        if(items.empty() || items.size() > 8)
         {
-
-            /*
-                Collect and draw items close to the last mouse position in the draw method.
-
-                This might be a bit odd but there are two reasons:
-
-                1) Multiple update events are combined by the event loop. Thus multiple mouse move
-                   events are reduced a single paint event. As getItemsByPos() is quite cycle
-                   intense this seems like a good idea.
-
-                2) The list of items passed back by getItemsByPos() must not be stored. That is why
-                   the list has to be generated within the draw handler to access the item's drawHighlight()
-                   method.
-
-            */
-            screenUnclutter->clear();
-
-            QList<IGisItem*> items;
-            CGisWidget::self().getItemsByPos(point, items);
-
-            if(items.empty() || items.size() > 8)
-            {
-                stateItemSel = eStateIdle;
-                break;
-            }
-
-            foreach(IGisItem * item, items)
-            {
-                item->drawHighlight(p);
-                screenUnclutter->addItem(item);
-            }
-
-            stateItemSel = (screenUnclutter->size() == 1) ? eStateHooverSingle : eStateHooverMultiple;
+            stateItemSel = eStateIdle;
             break;
         }
-        case eStateUnclutterMultiple:
-        {
-            screenUnclutter->draw(p);
-            break;
-        }
-        case eStateShowItemOptions:
-        {
-            if(screenItemOption.isNull())
-            {
-                stateItemSel = eStateIdle;
-                break;
-            }
 
-            screenItemOption->draw(p);
+        foreach(IGisItem * item, items)
+        {
+            item->drawHighlight(p);
+            screenUnclutter->addItem(item);
+        }
+
+        stateItemSel = (screenUnclutter->size() == 1) ? eStateHooverSingle : eStateHooverMultiple;
+        break;
+    }
+
+    case eStateUnclutterMultiple:
+    {
+        screenUnclutter->draw(p);
+        break;
+    }
+
+    case eStateShowItemOptions:
+    {
+        if(screenItemOption.isNull())
+        {
+            stateItemSel = eStateIdle;
             break;
         }
-        default:;
+
+        screenItemOption->draw(p);
+        break;
+    }
+
+    default:;
     }
 }
 
 
 void CMouseNormal::slotAddWpt()
 {
-
+    QString name;
+    QString icon;
     QPointF pt = point;
     gis->convertPx2Rad(pt);
     pt *= RAD_TO_DEG;
 
-    CGisItemWpt::getNewPosition(pt);
-    if(pt == NOPOINTF)
-    {
-        return;
-    }
-
-    QString name = CGisItemWpt::getNewName();
-    if(name.isEmpty())
-    {
-        return;
-    }
-
-    QString icon = CGisItemWpt::getNewIcon();
-    if(icon.isEmpty())
+    if(!CGisItemWpt::getNewWptData(pt, icon, name))
     {
         return;
     }
@@ -306,9 +334,11 @@ void CMouseNormal::slotAddWpt()
         return;
     }
 
-    new CGisItemWpt(pt, name, icon, project);
-    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+    CGisItemWpt * wpt = new CGisItemWpt(pt, name, icon, project);
+    wpt->edit();
 
+
+    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
 void CMouseNormal::slotAddTrk()
@@ -320,6 +350,15 @@ void CMouseNormal::slotAddTrk()
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
+void CMouseNormal::slotAddRte()
+{
+    QPointF pt = point;
+    gis->convertPx2Rad(pt);
+
+    canvas->setMouseEditRte(pt);
+    canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+}
+
 void CMouseNormal::slotAddArea()
 {
     QPointF pt = point;
