@@ -16,12 +16,12 @@
 
 **********************************************************************************************/
 
-#include "mouse/CMouseRangeTrk.h"
-#include "mouse/CScrOptRangeTrk.h"
-#include "gis/trk/CGisItemTrk.h"
+#include "canvas/CCanvas.h"
 #include "gis/CGisDraw.h"
 #include "gis/CGisWidget.h"
-#include "canvas/CCanvas.h"
+#include "gis/trk/CGisItemTrk.h"
+#include "mouse/CMouseRangeTrk.h"
+#include "mouse/CScrOptRangeTrk.h"
 
 #include <QtWidgets>
 
@@ -31,76 +31,52 @@ CMouseRangeTrk::CMouseRangeTrk(CGisItemTrk &trk, CGisDraw *gis, CCanvas *parent)
     , state(eStateIdle)
     , anchor(NOPOINTF)
 {
-    cursor  = QCursor(QPixmap("://cursors/cursorSelectRange.png"),0,0);
-    key = trk.getKey();
-
-    // reset user focus if the track has it
-    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove);
-    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseClick);
-    if(trk.hasUserFocus())
-    {
-        trk.gainUserFocus(false);
-    }
+    cursor      = QCursor(QPixmap("://cursors/cursorSelectRange.png"),0,0);
+    key         = trk.getKey();
 
     // switch to full mode to show deleted (hidden) track points, too
-    trk.setDrawMode(CGisItemTrk::eDrawRange);
+    trk.setMode(CGisItemTrk::eModeRange, "CMouseRangeTrk");
 
-    canvas->reportStatus(key, tr("<b>Select Range</b><br/>Select first track point. And then a second one.<br/>"));
+    // reset user focus if the track has it
+    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove, "CMouseRangeTrk");
+    trk.setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseClick, "CMouseRangeTrk");
+
+    canvas->reportStatus(key.item, tr("<b>Select Range</b><br/>Select first track point. And then a second one.<br/>"));
     /*
         trigger complete update of GIS components to make sure all changes to
         the originating object are reflected on the canvas
-    */
+     */
     canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
 }
 
 CMouseRangeTrk::~CMouseRangeTrk()
 {
-    canvas->reportStatus(key, "");
+    canvas->reportStatus(key.item, "");
 
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
     if(trk)
     {
-        trk->setDrawMode(CGisItemTrk::eDrawNormal);
+        trk->setMode(CGisItemTrk::eModeNormal, "CMouseRangeTrk");
         canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
     }
 
     delete scrOptRange;
 }
 
-void CMouseRangeTrk::draw(QPainter& p,  bool needsRedraw, const QRect &rect)
+void CMouseRangeTrk::draw(QPainter& p,  CCanvas::redraw_e, const QRect &)
 {
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
     if(trk)
     {
-        switch(state)
+        trk->drawRange(p);
+        if(anchor != NOPOINTF)
         {
-            case eStateIdle:
-            case eStateMoveMap:
-                if(anchor != NOPOINTF)
-                {
-                    p.setPen(Qt::darkBlue);
-                    p.setBrush(QColor(255,255,255,200));
-                    p.drawEllipse(anchor, 6, 6);
+            p.setPen(Qt::darkBlue);
+            p.setBrush(QColor(255,255,255,200));
+            p.drawEllipse(anchor, 6, 6);
 
-                    QPixmap bullet("://icons/8x8/bullet_magenta.png");
-                    p.drawPixmap(anchor.x() - 3, anchor.y() - 3, bullet);
-                }
-                break;
-
-            case eStateSelectRange:
-            case eStateRangeSelected:
-                trk->drawRange(p);
-                if(anchor != NOPOINTF)
-                {
-                    p.setPen(Qt::darkBlue);
-                    p.setBrush(QColor(255,255,255,200));
-                    p.drawEllipse(anchor, 6, 6);
-
-                    QPixmap bullet("://icons/8x8/bullet_magenta.png");
-                    p.drawPixmap(anchor.x() - 3, anchor.y() - 3, bullet);
-                }
-
-                break;
+            QPixmap bullet("://icons/8x8/bullet_magenta.png");
+            p.drawPixmap(anchor.x() - 3, anchor.y() - 3, bullet);
         }
     }
 
@@ -112,7 +88,7 @@ void CMouseRangeTrk::draw(QPainter& p,  bool needsRedraw, const QRect &rect)
 
 void CMouseRangeTrk::mousePressEvent(QMouseEvent * e)
 {
-    canvas->reportStatus(key, "");
+    canvas->reportStatus(key.item, "");
     point  = e->pos();
     if(e->button() == Qt::RightButton)
     {
@@ -123,55 +99,56 @@ void CMouseRangeTrk::mousePressEvent(QMouseEvent * e)
     {
         switch(state)
         {
-            case eStateIdle:
+        case eStateIdle:
+        {
+            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
+            if(trk != 0 && anchor != NOPOINTF)
             {
-                CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-                if(trk != 0 && anchor != NOPOINTF)
-                {
-                    anchor = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseClick);
-                    state = eStateSelectRange;
-                    canvas->update();
-                }
-                else
-                {
-                    state = eStateMoveMap;
-                }
-                break;
-            }
-
-            case eStateSelectRange:
-            {
-                CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-                if(trk != 0 && anchor != NOPOINTF)
-                {
-                    QPointF pt = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseMove);
-                    scrOptRange = new CScrOptRangeTrk(pt, trk, canvas);
-                    connect(scrOptRange->toolHidePoints, SIGNAL(clicked()), this, SLOT(slotHidePoints()));
-                    connect(scrOptRange->toolShowPoints, SIGNAL(clicked()), this, SLOT(slotShowPoints()));
-                    connect(scrOptRange->toolCopy, SIGNAL(clicked()), this, SLOT(slotCopy()));
-
-                    state = eStateRangeSelected;
-                    canvas->update();
-                }
-
-                break;
-            }
-            case eStateRangeSelected:
-            {
-                scrOptRange->deleteLater();
-                CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-                if(trk != 0)
-                {
-                    trk->setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove);
-                    trk->setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseClick);
-                }
-                state = eStateIdle;
+                anchor = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseClick, "CMouseRangeTrk");
+                state = eStateSelectRange;
                 canvas->update();
-                break;
             }
-            default:;
+            else
+            {
+                state = eStateMoveMap;
+            }
+            break;
         }
 
+        case eStateSelectRange:
+        {
+            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
+            if(trk != 0 && anchor != NOPOINTF)
+            {
+                QPointF pt = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseClick, "CMouseRangeTrk");
+                scrOptRange = new CScrOptRangeTrk(pt, trk, this);
+                connect(scrOptRange->toolHidePoints, SIGNAL(clicked()), this, SLOT(slotHidePoints()));
+                connect(scrOptRange->toolShowPoints, SIGNAL(clicked()), this, SLOT(slotShowPoints()));
+                connect(scrOptRange->toolCopy, SIGNAL(clicked()), this, SLOT(slotCopy()));
+
+                state = eStateRangeSelected;
+                canvas->update();
+            }
+
+            break;
+        }
+
+        case eStateRangeSelected:
+        {
+            scrOptRange->deleteLater();
+            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
+            if(trk != 0)
+            {
+                trk->setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseMove, "CMouseRangeTrk");
+                trk->setMouseFocusByPoint(NOPOINT, CGisItemTrk::eFocusMouseClick, "CMouseRangeTrk");
+            }
+            state = eStateIdle;
+            canvas->update();
+            break;
+        }
+
+        default:;
+        }
     }
 }
 
@@ -181,40 +158,40 @@ void CMouseRangeTrk::mouseMoveEvent(QMouseEvent * e)
 
     switch(state)
     {
-        case eStateIdle:
+    case eStateIdle:
+    {
+        CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
+        if(trk != 0)
         {
-            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-            if(trk != 0)
-            {
-                anchor = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseClick);
-                canvas->update();
-            }
-            break;
+            anchor = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseMove, "CMouseRangeTrk");
+            canvas->update();
         }
+        break;
+    }
 
-        case eStateMoveMap:
+    case eStateMoveMap:
+    {
+        if(point != lastPoint)
         {
-            if(point != lastPoint)
-            {
-                QPoint delta = point - lastPoint;
-                canvas->moveMap(delta);
-            }
-            break;
+            QPoint delta = point - lastPoint;
+            canvas->moveMap(delta);
         }
+        break;
+    }
 
-        case eStateSelectRange:
+    case eStateSelectRange:
+    {
+        CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
+        if(trk != 0)
         {
-
-            CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
-            if(trk != 0)
-            {
-                anchor = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseMove);
-                canvas->update();
-            }
-            panCanvas(point);
-            break;
+            anchor = trk->setMouseFocusByPoint(point, CGisItemTrk::eFocusMouseMove, "CMouseRangeTrk");
+            canvas->update();
         }
-        default:;
+        panCanvas(point);
+        break;
+    }
+
+    default:;
     }
 
     lastPoint = point;
@@ -233,7 +210,6 @@ void CMouseRangeTrk::mouseReleaseEvent(QMouseEvent *e)
 
 void CMouseRangeTrk::wheelEvent(QWheelEvent * e)
 {
-
 }
 
 void CMouseRangeTrk::slotHidePoints()
@@ -241,6 +217,10 @@ void CMouseRangeTrk::slotHidePoints()
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
     if(trk != 0)
     {
+        if(!trk->setReadOnlyMode(false))
+        {
+            return;
+        }
         trk->hideSelectedPoints();
         canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
     }
@@ -254,6 +234,11 @@ void CMouseRangeTrk::slotShowPoints()
     CGisItemTrk * trk = dynamic_cast<CGisItemTrk*>(CGisWidget::self().getItemByKey(key));
     if(trk != 0)
     {
+        if(!trk->setReadOnlyMode(false))
+        {
+            return;
+        }
+
         trk->showSelectedPoints();
         canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
     }
