@@ -19,6 +19,7 @@
 #ifndef CMOUSERANGETRK_H
 #define CMOUSERANGETRK_H
 
+#include "gis/IGisItem.h"
 #include "mouse/IMouse.h"
 #include <QPointer>
 
@@ -29,42 +30,40 @@ class CScrOptRangeTrk;
 
 class CMouseRangeTrk : public IMouse
 {
-    Q_OBJECT;
-    public:
-        CMouseRangeTrk(CGisItemTrk& trk, CGisDraw * gis, CCanvas * parent);
-        virtual ~CMouseRangeTrk();
+    Q_OBJECT
+public:
+    CMouseRangeTrk(CGisItemTrk& trk, CGisDraw * gis, CCanvas * parent);
+    virtual ~CMouseRangeTrk();
 
-        void draw(QPainter& p,  bool needsRedraw, const QRect &rect);
-        void mousePressEvent(QMouseEvent * e);
-        void mouseMoveEvent(QMouseEvent * e);
-        void mouseReleaseEvent(QMouseEvent *e);
-        void wheelEvent(QWheelEvent * e);
+    void draw(QPainter& p, CCanvas::redraw_e, const QRect &);
+    void mousePressEvent(QMouseEvent * e);
+    void mouseMoveEvent(QMouseEvent * e);
+    void mouseReleaseEvent(QMouseEvent *e);
+    void wheelEvent(QWheelEvent * e);
 
-    private slots:
-        void slotHidePoints();
-        void slotShowPoints();
-        void slotCopy();
+private slots:
+    void slotHidePoints();
+    void slotShowPoints();
+    void slotCopy();
 
-    private:
-        QString key;
+private:
+    IGisItem::key_t key;
 
-        enum state_e
-        {
-             eStateIdle
-            ,eStateMoveMap
-            ,eStateSelectRange
-            ,eStateRangeSelected
-        };
+    enum state_e
+    {
+        eStateIdle
+        ,eStateMoveMap
+        ,eStateSelectRange
+        ,eStateRangeSelected
+    };
 
-        state_e state;
+    state_e state;
 
-        QPointF anchor;
-
-        QPoint lastPoint;
-
-        QPointer<CScrOptRangeTrk> scrOptRange;
+    QPointF anchor;
 
+    QPoint lastPoint;
 
+    QPointer<CScrOptRangeTrk> scrOptRange;
 };
 
 #endif //CMOUSERANGETRK_H
