@@ -16,13 +16,13 @@
 
 **********************************************************************************************/
 
-#include "mouse/CScrOptPoint.h"
 #include "canvas/CCanvas.h"
+#include "mouse/CScrOptPoint.h"
 
 #include <QtWidgets>
 
-CScrOptPoint::CScrOptPoint(const QPointF &point, QWidget *parent)
-    : IScrOpt(parent)
+CScrOptPoint::CScrOptPoint(const QPointF &point, IMouse * mouse)
+    : IScrOpt(mouse)
 {
     setupUi(this);
     adjustSize();
@@ -31,12 +31,10 @@ CScrOptPoint::CScrOptPoint(const QPointF &point, QWidget *parent)
 
     move(point.toPoint() + QPoint(-width()/2,SCR_OPT_OFFSET));
     show();
-
 }
 
 CScrOptPoint::~CScrOptPoint()
 {
-
 }
 
 void CScrOptPoint::draw(QPainter& p)
