@@ -24,11 +24,11 @@
 
 class CScrOptRange : public IScrOpt, public Ui::IScrOptRange
 {
-    public:
-        CScrOptRange(const QPointF& point, QWidget * parent);
-        virtual ~CScrOptRange();
+public:
+    CScrOptRange(const QPointF& point, IMouse *mouse);
+    virtual ~CScrOptRange();
 
-        void draw(QPainter& p);
+    void draw(QPainter& p);
 };
 
 #endif //CSCROPTRANGE_H
