@@ -16,43 +16,43 @@
 
 **********************************************************************************************/
 
-#include "mouse/CScrOptUnclutter.h"
-#include "gis/IGisItem.h"
 #include "CMainWindow.h"
 #include "canvas/CCanvas.h"
+#include "gis/IGisItem.h"
+#include "gis/trk/CGisItemTrk.h"
+#include "mouse/CScrOptUnclutter.h"
 
 #include <QtWidgets>
 
 const QPoint CScrOptUnclutter::positions[9][8] =
 {
     {
-
     },
 
     {
-          QPoint(-50,-23)
+        QPoint(-50,-23)
     },
 
     {
-          QPoint(-30,0)
+        QPoint(-30,0)
         , QPoint( 30,0)
     },
 
     {
-          QPoint(  0,-30)
+        QPoint(  0,-30)
         , QPoint( 30, 30)
         , QPoint(-30, 30)
     },
 
     {
-          QPoint(-30,-30)
+        QPoint(-30,-30)
         , QPoint( 30,-30)
         , QPoint(-30, 30)
         , QPoint( 30, 30)
     },
 
     {
-          QPoint(-25, 40)
+        QPoint(-25, 40)
         , QPoint( 25, 40)
         , QPoint(-40, -5)
         , QPoint( 40, -5)
@@ -60,7 +60,7 @@ const QPoint CScrOptUnclutter::positions[9][8] =
     },
 
     {
-          QPoint(-40,-22)
+        QPoint(-40,-22)
         , QPoint( 40,-22)
         , QPoint(-40, 22)
         , QPoint( 40, 22)
@@ -69,7 +69,7 @@ const QPoint CScrOptUnclutter::positions[9][8] =
     },
 
     {
-          QPoint(-50,-23)
+        QPoint(-50,-23)
         , QPoint( 50,-23)
         , QPoint(-45, 21)
         , QPoint( 45, 21)
@@ -79,7 +79,7 @@ const QPoint CScrOptUnclutter::positions[9][8] =
     },
 
     {
-          QPoint(-50,-23)
+        QPoint(-50,-23)
         , QPoint( 50,-23)
         , QPoint(-50, 23)
         , QPoint( 50, 23)
@@ -88,20 +88,17 @@ const QPoint CScrOptUnclutter::positions[9][8] =
         , QPoint(-22, 55)
         , QPoint( 22, 55)
     }
-
 };
 
 
-CScrOptUnclutter::CScrOptUnclutter(QWidget *parent)
-    : IScrOpt(parent)
+CScrOptUnclutter::CScrOptUnclutter(IMouse *mouse)
+    : IScrOpt(mouse)
     , doSpecialCursor(false)
 {
-
 }
 
 CScrOptUnclutter::~CScrOptUnclutter()
 {
-
 }
 
 void CScrOptUnclutter::clear()
@@ -118,9 +115,9 @@ void CScrOptUnclutter::mouseMoveEvent(QMouseEvent * e)
 {
     IScrOpt::mouseMoveEvent(e);
 
-    foreach(const item_t& item, items)
+    foreach(const item_t &item, items)
     {
-        if(item.active.contains(mousePos))
+        if(item.active.contains(mousePos) || item.text.contains(mousePos))
         {
             if(!doSpecialCursor)
             {
@@ -142,28 +139,30 @@ void CScrOptUnclutter::mouseMoveEvent(QMouseEvent * e)
 void CScrOptUnclutter::addItem(IGisItem * gisItem)
 {
     items << item_t();
-    item_t& item    = items.last();
-    item.name       = gisItem->getName();
-    item.key        = gisItem->getKey();
-    item.icon       = gisItem->getIcon();
-    item.area       = item.icon.rect();
-    item.active     = item.area.adjusted(-10,-10,10,10);
+    item_t& item        = items.last();
+    item.hasUserFocus   = gisItem->hasUserFocus();
+    item.name           = gisItem->getNameEx();
+    item.key            = gisItem->getKey();
+    item.icon           = gisItem->getIcon();
+    item.area           = item.icon.rect();
+    item.active         = item.area.adjusted(-10,-10,10,10);
 }
 
-QString CScrOptUnclutter::getItemKey(int index)
+IGisItem::key_t CScrOptUnclutter::getItemKey(int index)
 {
     if(index < items.size())
     {
         return items[index].key;
     }
-    return QString::Null();
+    return IGisItem::key_t();
 }
 
+
 const CScrOptUnclutter::item_t * CScrOptUnclutter::selectItem(const QPoint& point)
 {
-    foreach(const item_t& item, items)
+    foreach(const item_t &item, items)
     {
-        if(item.active.contains(point))
+        if(item.active.contains(point) || item.text.contains(point))
         {
             return &item;
         }
@@ -181,7 +180,6 @@ void CScrOptUnclutter::draw(QPainter& p)
 
         if(item.text.isNull())
         {
-
             item.area.moveCenter(origin + positions[N][cnt]);
             item.active.moveCenter(item.area.center());
             item.text = fm.boundingRect(item.name);
@@ -197,16 +195,14 @@ void CScrOptUnclutter::draw(QPainter& p)
         }
     }
 
-    foreach(const item_t& item, items)
+    foreach(const item_t &item, items)
     {
-
-
         p.setPen(Qt::NoPen);
         p.setBrush(QColor(255,255,255,255));
-        p.drawEllipse(item.area.center(), 20,20);        
+        p.drawEllipse(item.area.center(), 20,20);
         p.drawRoundedRect(item.text, 3, 3);
 
-        p.setPen(QPen(Qt::lightGray,2));
+        p.setPen(QPen(item.hasUserFocus ? Qt::red : Qt::lightGray,2));
         p.setBrush(Qt::NoBrush);
         p.drawRoundedRect(item.text, 3, 3);
         p.drawEllipse(item.area.center(), 18,18);
