@@ -16,18 +16,16 @@
 
 **********************************************************************************************/
 
-#include "plot/CPlotDistance.h"
 #include "plot/CPlotAxisTime.h"
+#include "plot/CPlotDistance.h"
 
 CPlotDistance::CPlotDistance(QWidget *parent)
     : IPlot(0, CPlotData::eAxisTime, eModeNormal, parent)
 {
-
 }
 
 CPlotDistance::~CPlotDistance()
 {
-
 }
 
 void CPlotDistance::setTrack(CGisItemTrk * track)
@@ -40,7 +38,12 @@ void CPlotDistance::setTrack(CGisItemTrk * track)
 
 
 void CPlotDistance::updateData()
-{       
+{
+    if(isHidden())
+    {
+        return;
+    }
+
     CPlotData::axistype_e type = data->axisType;
 
     if(mode == eModeIcon)
@@ -56,7 +59,7 @@ void CPlotDistance::updateData()
         }
         else
         {
-            setXLabel(tr("time [h]"));
+            setXLabel(tr("time"));
         }
         setYLabel(tr("distance. [%1]").arg(IUnit::self().baseunit));
     }
@@ -64,6 +67,8 @@ void CPlotDistance::updateData()
     clear();
     if(trk->getTotalElapsedSeconds() == 0)
     {
+        resetZoom();
+        update();
         return;
     }
 
@@ -92,12 +97,11 @@ void CPlotDistance::updateData()
     newLine(lineDist, "GPS");
     setLimits();
     resetZoom();
-
 }
 
-void CPlotDistance::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
+void CPlotDistance::setMouseFocus(const CGisItemTrk::trkpt_t * ptMouseMove)
 {
-    if(pt == 0)
+    if(ptMouseMove == 0)
     {
         if(posMouse != NOPOINT)
         {
@@ -112,9 +116,10 @@ void CPlotDistance::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
             needsRedraw = true;
         }
 
-        posMouse.rx() = left  + data->x().val2pt(pt->time.toTime_t());
-        posMouse.ry() = top  +  data->y().val2pt(pt->distance);
+        posMouse.rx() = left  + data->x().val2pt(ptMouseMove->time.toTime_t());
+        posMouse.ry() = top  +  data->y().val2pt(ptMouseMove->distance);
     }
     update();
-
 }
+
+
