@@ -24,14 +24,13 @@
 class CPlotDistance : public IPlot
 {
     Q_OBJECT
-    public:
-        CPlotDistance(QWidget * parent);
-        virtual ~CPlotDistance();
-
-        void setTrack(CGisItemTrk * track);
-        void updateData();
-        void setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt);
+public:
+    CPlotDistance(QWidget * parent);
+    virtual ~CPlotDistance();
 
+    void setTrack(CGisItemTrk * track);
+    void updateData();
+    void setMouseFocus(const CGisItemTrk::trkpt_t * ptMouseMove);
 };
 
 #endif //CPLOTDISTANCE_H
