@@ -16,31 +16,29 @@
 
 **********************************************************************************************/
 
-#include "plot/CPlotProfile.h"
-#include "plot/CPlotAxis.h"
-#include "units/IUnit.h"
-#include "gis/trk/CGisItemTrk.h"
 #include "CMainWindow.h"
+#include "gis/trk/CGisItemTrk.h"
+#include "gis/wpt/CGisItemWpt.h"
+#include "plot/CPlotAxis.h"
+#include "plot/CPlotProfile.h"
+#include "units/IUnit.h"
 
 #include <proj_api.h>
 
 CPlotProfile::CPlotProfile(QWidget * parent)
     : IPlot(0, CPlotData::eAxisLinear, eModeNormal, parent)
 {
-
 }
 
 CPlotProfile::CPlotProfile(CGisItemTrk *trk, mode_e mode, QWidget *parent)
     : IPlot(trk, CPlotData::eAxisLinear, mode, parent)
 {
-
-
+    setWindowTitle(trk->getNameEx());
     updateData();
 }
 
 CPlotProfile::~CPlotProfile()
 {
-
 }
 
 void CPlotProfile::setTrack(CGisItemTrk * track)
@@ -73,10 +71,15 @@ void CPlotProfile::updateData()
         setYLabel(tr("alt. [%1]").arg(IUnit::self().baseunit));
     }
 
+
+    clear();
+
     QPolygonF lineEle;
     QPolygonF lineDem;
     QPolygonF coords;
 
+    IGisProject * project = dynamic_cast<IGisProject*>(trk->parent());
+
     qreal basefactor = IUnit::self().basefactor;
     const CGisItemTrk::trk_t& t = trk->getTrackData();
     foreach (const CGisItemTrk::trkseg_t& seg, t.segs)
@@ -88,18 +91,34 @@ void CPlotProfile::updateData()
                 continue;
             }
 
-            if(trkpt.ele != NOINT)
+            if(trkpt.ele == NOINT)
             {
-                lineEle << QPointF(type == CPlotData::eAxisLinear ? trkpt.distance : (qreal)trkpt.time.toTime_t(), trkpt.ele * basefactor);
-                coords  << QPointF(trkpt.lon * DEG_TO_RAD, trkpt.lat * DEG_TO_RAD);
-                lineDem << QPointF(type == CPlotData::eAxisLinear ? trkpt.distance : (qreal)trkpt.time.toTime_t(), NOFLOAT);
+                continue;
+            }
+
+            lineEle << QPointF(type == CPlotData::eAxisLinear ? trkpt.distance : (qreal)trkpt.time.toTime_t(), trkpt.ele * basefactor);
+            coords << QPointF(trkpt.lon * DEG_TO_RAD, trkpt.lat * DEG_TO_RAD);
+            lineDem << QPointF(type == CPlotData::eAxisLinear ? trkpt.distance : (qreal)trkpt.time.toTime_t(), NOFLOAT);
+
+            if(project == 0 || trkpt.keyWpt.item.isEmpty() || (mode == eModeIcon))
+            {
+                continue;
+            }
+
+            CGisItemWpt * wpt = dynamic_cast<CGisItemWpt*>(project->getItemByKey(trkpt.keyWpt));
+            if(wpt)
+            {
+                CPlotData::point_t tag;
+                tag.point = lineEle.last();
+                tag.icon  = wpt->getIcon();
+                tag.label = wpt->getName();
+                data->tags << tag;
             }
         }
     }
 
     CMainWindow::self().getEelevationAt(coords, lineDem);
 
-    clear();
     newLine(lineEle, "GPS");
     if(!lineDem.isEmpty())
     {
@@ -107,12 +126,11 @@ void CPlotProfile::updateData()
     }
     setLimits();
     resetZoom();
-
 }
 
-void CPlotProfile::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
+void CPlotProfile::setMouseFocus(const CGisItemTrk::trkpt_t * ptMouseMove)
 {
-    if(pt == 0)
+    if(ptMouseMove == 0)
     {
         if(posMouse != NOPOINT)
         {
@@ -127,8 +145,9 @@ void CPlotProfile::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
             needsRedraw = true;
         }
 
-        posMouse.rx() = left  + data->x().val2pt(pt->distance);
-        posMouse.ry() = top  +  data->y().val2pt(pt->ele);
+        posMouse.rx() = left  + data->x().val2pt(ptMouseMove->distance);
+        posMouse.ry() = top  +  data->y().val2pt(ptMouseMove->ele);
     }
     update();
 }
+
