@@ -16,18 +16,16 @@
 
 **********************************************************************************************/
 
-#include "plot/CPlotSpeed.h"
 #include "plot/CPlotAxis.h"
+#include "plot/CPlotSpeed.h"
 
 CPlotSpeed::CPlotSpeed(QWidget *parent)
     : IPlot(0, CPlotData::eAxisLinear, eModeNormal, parent)
 {
-
 }
 
 CPlotSpeed::~CPlotSpeed()
 {
-
 }
 
 void CPlotSpeed::setTrack(CGisItemTrk * track)
@@ -40,6 +38,11 @@ void CPlotSpeed::setTrack(CGisItemTrk * track)
 
 void CPlotSpeed::updateData()
 {
+    if(isHidden())
+    {
+        return;
+    }
+
     CPlotData::axistype_e type = data->axisType;
 
     if(mode == eModeIcon)
@@ -63,6 +66,8 @@ void CPlotSpeed::updateData()
     clear();
     if(trk->getTotalElapsedSeconds() == 0)
     {
+        resetZoom();
+        update();
         return;
     }
 
@@ -88,12 +93,14 @@ void CPlotSpeed::updateData()
 
     newLine(lineSpeed, "GPS");
     setLimits();
+    data->ymin = 0;
+    data->y().setLimits(0,data->ymax);
     resetZoom();
 }
 
-void CPlotSpeed::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
+void CPlotSpeed::setMouseFocus(const CGisItemTrk::trkpt_t * ptMouseMove)
 {
-    if(pt == 0)
+    if(ptMouseMove == 0)
     {
         if(posMouse != NOPOINT)
         {
@@ -108,8 +115,9 @@ void CPlotSpeed::setMouseMoveFocus(const CGisItemTrk::trkpt_t * pt)
             needsRedraw = true;
         }
 
-        posMouse.rx() = left  + data->x().val2pt(pt->distance);
-        posMouse.ry() = top  +  data->y().val2pt(pt->speed);
+        posMouse.rx() = left  + data->x().val2pt(ptMouseMove->distance);
+        posMouse.ry() = top  +  data->y().val2pt(ptMouseMove->speed);
     }
     update();
 }
+
