@@ -19,48 +19,27 @@
 #ifndef CPLOTTRACK_H
 #define CPLOTTRACK_H
 
+#include "plot/ITrack.h"
 #include <QWidget>
-#include <proj_api.h>
 
 class CGisItemTrk;
 
-class CPlotTrack : public QWidget
+class CPlotTrack : public QWidget, public ITrack
 {
-    public:
-        CPlotTrack(QWidget * parent);
-        virtual ~CPlotTrack();
+public:
+    CPlotTrack(QWidget * parent);
+    CPlotTrack(CGisItemTrk * trk, QWidget * parent);
+    virtual ~CPlotTrack();
 
-        void setTrack(CGisItemTrk * track, const QString &proj = "+init=epsg:3857");
-        void setTrack(const QPolygonF &track, const QString &proj = "+init=epsg:3857");
-        void updateData();
-        void setMouseMoveFocus(qreal lon, qreal lat);
+    void setMouseFocus(qreal lon, qreal lat);
 
-    protected:
-        void resizeEvent(QResizeEvent * e);
-        void paintEvent(QPaintEvent * e);
+protected:
+    void resizeEvent(QResizeEvent * e);
+    void paintEvent(QPaintEvent * e);
 
+private:
 
-    private:
-        void draw();
-
-        projPJ  pjsrc;
-        projPJ  pjtar;
-
-        bool needsRedraw;
-        CGisItemTrk * trk;
-        QPolygonF coords;
-
-        QImage buffer;
-
-        QPointF pt1;
-        QPointF pt2;
-        QPointF scale;
-        QPolygonF line;
-
-        qint32 xoff;
-        qint32 yoff;
-
-        QPointF pos;
+    QPointF pos;
 };
 
 #endif //CPLOTTRACK_H
