@@ -16,11 +16,15 @@
 
 **********************************************************************************************/
 
-#include "plot/IPlot.h"
 #include "plot/CPlotAxis.h"
+#include "plot/IPlot.h"
 
-#include "canvas/CCanvas.h"
 #include "CMainWindow.h"
+#include "canvas/CCanvas.h"
+#include "gis/CGisWidget.h"
+#include "helpers/CFadingIcon.h"
+#include "helpers/CSettings.h"
+#include "mouse/CScrOptRangeTrk.h"
 
 #include <QtWidgets>
 
@@ -30,7 +34,6 @@ QPen IPlot::pens[] =
     , QPen(QColor("#C00000"),3,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
     , QPen(Qt::yellow,3,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
     , QPen(Qt::green,3,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
-
 };
 
 QPen IPlot::pensThin[] =
@@ -39,7 +42,6 @@ QPen IPlot::pensThin[] =
     , QPen(Qt::darkRed,2,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
     , QPen(Qt::darkYellow,2,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
     , QPen(Qt::darkGreen,2,Qt::SolidLine, Qt::RoundCap, Qt::RoundJoin)
-
 };
 
 QColor IPlot::colors[] =
@@ -47,10 +49,10 @@ QColor IPlot::colors[] =
     QColor(Qt::blue)
     , QColor(0,0,0,0)
     , QColor(0,0,0,0)
-    , QColor(0,0,0,0)
-
+    , QColor(Qt::darkGreen)
 };
 
+int IPlot::cnt = 0;
 
 IPlot::IPlot(CGisItemTrk *trk, CPlotData::axistype_e type, mode_e mode, QWidget *parent)
     : QWidget(parent)
@@ -58,6 +60,7 @@ IPlot::IPlot(CGisItemTrk *trk, CPlotData::axistype_e type, mode_e mode, QWidget
     , needsRedraw(true)
     , showScale(true)
     , thinLine(false)
+    , solid(false)
     , posMouse(NOPOINT)
     , trk(trk)
     , fm(font())
@@ -71,11 +74,21 @@ IPlot::IPlot(CGisItemTrk *trk, CPlotData::axistype_e type, mode_e mode, QWidget
     , fontHeight(0)
     , scaleWidthX1(0)
     , scaleWidthY1(0)
+    , idxSel1(NOIDX)
+    , idxSel2(NOIDX)
+    , mouseClickState(eMouseClickIdle)
 
 {
+    cnt++;
+    setObjectName(QString("IPlot%1").arg(cnt));
+
+    setContextMenuPolicy(Qt::CustomContextMenu);
     setMouseTracking(true);
 
-    if(trk) trk->registerPlot(this);
+    if(trk)
+    {
+        trk->registerPlot(this);
+    }
 
     data = new CPlotData(type, this);
 
@@ -85,17 +98,51 @@ IPlot::IPlot(CGisItemTrk *trk, CPlotData::axistype_e type, mode_e mode, QWidget
         thinLine = true;
     }
 
+    if(mode == eModeWindow)
+    {
+        overrideWindowFlags(Qt::Window);
+        setAttribute(Qt::WA_DeleteOnClose, true);
+    }
+
+    menu = new QMenu(this);
+    actionStopRange = menu->addAction(QIcon("://icons/32x32/SelectRange.png"), tr("Stop Range"), this, SLOT(slotStopRange()));
+    actionPrint     = menu->addAction(QIcon("://icons/32x32/Save.png"), tr("Save..."), this, SLOT(slotSave()));
+
+    connect(this, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(slotContextMenu(QPoint)));
 }
 
 IPlot::~IPlot()
 {
-    if(trk) trk->unregisterPlot(this);
+    cnt--;
+
+    if(trk)
+    {
+        trk->unregisterPlot(this);
+        /*
+            Always set the mode to normal. If the object is not owner
+            of the current mode, the request will be ignored.
+         */
+        trk->setMode(CGisItemTrk::eModeNormal, objectName());
+
+        /*
+            As having the user focus will always display an on screen plot, closing
+            the plot has to result into the track loosing the focus.
+         */
+        if(mode == eModeWindow)
+        {
+            trk->looseUserFocus();
+            CCanvas * canvas = dynamic_cast<CCanvas*>(parent());
+            if(canvas)
+            {
+                canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+            }
+        }
+    }
 }
 
 void IPlot::clear()
 {
     data->lines.clear();
-    data->marks.points.clear();
     data->tags.clear();
     data->badData = true;
     update();
@@ -123,7 +170,7 @@ void IPlot::newLine(const QPolygonF& line, const QString& label)
     data->lines.clear();
 
     QRectF r = line.boundingRect();
-    if((r.height() < 0) || (r.width() < 0))
+    if((r.height() < 0) || (r.width() < 0) || line.isEmpty())
     {
         data->badData = true;
         return;
@@ -141,13 +188,12 @@ void IPlot::newLine(const QPolygonF& line, const QString& label)
 
     needsRedraw = true;
     update();
-
 }
 
 void IPlot::addLine(const QPolygonF& line, const QString& label)
 {
     QRectF r = line.boundingRect();
-    if(!r.isValid())
+    if(!r.isValid() || line.isEmpty())
     {
         return;
     }
@@ -177,9 +223,6 @@ void IPlot::resetZoom()
     data->y().resetZoom();
     setSizes();
 
-//    initialYMin = data->y().min();
-//    initialYMax = data->y().max();
-
     needsRedraw = true;
     update();
 }
@@ -195,9 +238,6 @@ void IPlot::resizeEvent(QResizeEvent * e)
 {
     setSizes();
 
-//    initialYMin = data->y().min();
-//    initialYMax = data->y().max();
-
     buffer = QImage(e->size(), QImage::Format_ARGB32);
 
     needsRedraw = true;
@@ -209,8 +249,6 @@ void IPlot::leaveEvent(QEvent * e)
     needsRedraw = true;
     posMouse    = NOPOINT;
 
-    if(trk) trk->setMouseFocusByDistance(NOFLOAT, CGisItemTrk::eFocusMouseMove, this);
-
     QApplication::restoreOverrideCursor();
     update();
 }
@@ -238,6 +276,11 @@ void IPlot::draw(QPainter& p)
 
 void IPlot::mouseMoveEvent(QMouseEvent * e)
 {
+    if(data->lines.isEmpty() || data->badData || !data->x().isValid() || !data->y().isValid())
+    {
+        return;
+    }
+
     posMouse = NOPOINT;
     if(rectGraphArea.contains(e->pos()))
     {
@@ -247,11 +290,17 @@ void IPlot::mouseMoveEvent(QMouseEvent * e)
         qreal x = data->x().pt2val(posMouse.x() - left);
         if(data->axisType == CPlotData::eAxisLinear)
         {
-            if(trk) trk->setMouseFocusByDistance(x, CGisItemTrk::eFocusMouseMove, this);
+            if(trk)
+            {
+                trk->setMouseFocusByDistance(x, CGisItemTrk::eFocusMouseMove, objectName());
+            }
         }
         else if(data->axisType == CPlotData::eAxisTime)
         {
-            if(trk) trk->setMouseFocusByTime(x, CGisItemTrk::eFocusMouseMove, this);
+            if(trk)
+            {
+                trk->setMouseFocusByTime(x, CGisItemTrk::eFocusMouseMove, objectName());
+            }
         }
 
         // update canvas if visible
@@ -267,6 +316,13 @@ void IPlot::mouseMoveEvent(QMouseEvent * e)
 
 void IPlot::mousePressEvent(QMouseEvent * e)
 {
+    if(data->lines.isEmpty() || data->badData || !data->x().isValid() || !data->y().isValid())
+    {
+        return;
+    }
+
+    bool wasProcessed = true;
+
     posMouse = NOPOINT;
     if((e->button() == Qt::LeftButton) && rectGraphArea.contains(e->pos()))
     {
@@ -280,20 +336,112 @@ void IPlot::mousePressEvent(QMouseEvent * e)
         {
             // set point of focus at track object
             qreal x = data->x().pt2val(posMouse.x() - left);
-            if(data->axisType == CPlotData::eAxisLinear)
+
+            switch(mouseClickState)
             {
-                if(trk) trk->setMouseFocusByDistance(x, CGisItemTrk::eFocusMouseClick, this);
-            }
-            else if(data->axisType == CPlotData::eAxisTime)
+            case eMouseClickIdle:
             {
-                if(trk) trk->setMouseFocusByTime(x, CGisItemTrk::eFocusMouseClick, this);
+                // In idle state a mouse click will select the first point of a range
+                if(trk->setMode(CGisItemTrk::eModeRange, objectName()))
+                {
+                    if(data->axisType == CPlotData::eAxisLinear)
+                    {
+                        trk->setMouseFocusByDistance(x, CGisItemTrk::eFocusMouseClick, objectName());
+                    }
+                    else if(data->axisType == CPlotData::eAxisTime)
+                    {
+                        trk->setMouseFocusByTime(x, CGisItemTrk::eFocusMouseClick, objectName());
+                    }
+
+                    mouseClickState = eMouseClick1st;
+                }
+                else
+                {
+                    /*
+                        If the object is not the owner of the range selection, no action has to be taken.
+                        However the user has to be informed, that he clicked on the wrong widget.
+                     */
+                    new CFadingIcon(posMouse, "://icons/48x48/NoGo.png", this);
+                    wasProcessed = false;
+                }
+                break;
             }
 
-            // update canvas if visible
-            CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
-            if(canvas)
+            case eMouseClick1st:
             {
-                canvas->update();
+                // In 1st click state a mouse click will select the second point of a range and display options
+                if(data->axisType == CPlotData::eAxisLinear)
+                {
+                    trk->setMouseFocusByDistance(x, CGisItemTrk::eFocusMouseClick, objectName());
+                }
+                else if(data->axisType == CPlotData::eAxisTime)
+                {
+                    trk->setMouseFocusByTime(x, CGisItemTrk::eFocusMouseClick, objectName());
+                }
+
+                /*
+                    As the screen option is created on the fly it has to be connected to all slots,too.
+                    Later, when destroyed the slots will be disconnected automatically.
+                 */
+                delete scrOptRange;
+                scrOptRange = new CScrOptRangeTrk(e->pos(), trk, &dummyMouse, this);
+                connect(scrOptRange->toolHidePoints, SIGNAL(clicked()), this, SLOT(slotHidePoints()));
+                connect(scrOptRange->toolShowPoints, SIGNAL(clicked()), this, SLOT(slotShowPoints()));
+                connect(scrOptRange->toolCopy, SIGNAL(clicked()), this, SLOT(slotCopy()));
+
+                /* Adjust posiion of screen option widget if the widget is out of the visible area*/
+                QRect r1 = scrOptRange->geometry();
+                QRect r2 = geometry();
+                r1.moveTopLeft(mapToParent(r1.topLeft()));
+                if(!r2.contains(r1))
+                {
+                    // test if screen option is out of area on the right side
+                    if(!r2.contains(r1.topRight()))
+                    {
+                        QPoint pt = QPoint(r2.width(), r2.height()) - QPoint(r1.width(), r1.height());
+                        scrOptRange->move(pt);
+                    }
+                    // test if screen option is out of area on the left side
+                    else if(!r2.contains(r1.topLeft()))
+                    {
+                        QPoint pt = QPoint(0, r2.height()) - QPoint(0, r1.height());
+                        scrOptRange->move(pt);
+                    }
+                    // test if screen option is out of area on the bottom
+                    else if(!r2.contains(r1.bottomLeft()))
+                    {
+                        QPoint pt = QPoint(r1.left(), r2.height()) - QPoint(r2.left(), r1.height());
+                        scrOptRange->move(pt);
+                    }
+                }
+
+                mouseClickState = eMouseClick2nd;
+                break;
+            }
+
+            case eMouseClick2nd:
+            {
+                // In second click state a mouse click will reset the range selection
+                delete scrOptRange;
+                trk->setMode(CGisItemTrk::eModeNormal, objectName());
+                idxSel1 = idxSel2 = NOIDX;
+                mouseClickState = eMouseClickIdle;
+                break;
+            }
+            }
+
+
+            // Update canvas only if the object ist the owener of the range selection
+            if(wasProcessed)
+            {
+                emit sigMouseClickState(mouseClickState);
+
+                // update canvas if visible
+                CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
+                if(canvas)
+                {
+                    canvas->update();
+                }
             }
         }
         e->accept();
@@ -322,7 +470,6 @@ void IPlot::setSizes()
     setSizeYLabel();
     setSizeTrackInfo();
     setSizeDrawArea();
-
 }
 
 void IPlot::setLRTB()
@@ -340,8 +487,7 @@ void IPlot::setLRTB()
     top = 0;
     if(!data->tags.isEmpty())
     {
-        top += fontHeight;
-        top += 16;
+        top += 9;
     }
     top += deadAreaY;
 
@@ -358,11 +504,6 @@ void IPlot::setLRTB()
     {
         bottom -= deadAreaY;
     }
-
-    if(!data->tags.isEmpty() /*&& CResources::self().showTrackProfileEleInfo()*/)
-    {
-        bottom -= fontHeight;
-    }
 }
 
 
@@ -383,10 +524,6 @@ void IPlot::setSizeXLabel()
         rectX1Label.setWidth( right - left );
         rectX1Label.setHeight( fontHeight );
         y = ( size().height() - rectX1Label.height()) - deadAreaY;
-        if(!data->tags.isEmpty() /*&& CResources::self().showTrackProfileEleInfo()*/)
-        {
-            y -= fontHeight;
-        }
 
         rectX1Label.moveTopLeft( QPoint( left, y ) );
     }
@@ -445,9 +582,9 @@ void IPlot::draw()
     {
         QRect r = rect();
         r.adjust(2,2,-2,-2);
-        if(underMouse() || posMouse != NOPOINT)
+        if(underMouse() || posMouse != NOPOINT || solid)
         {
-            p.setPen(CCanvas::penBorderBlue);
+            p.setPen(solid ? CCanvas::penBorderBlack : CCanvas::penBorderBlue);
             p.setOpacity(1.0);
         }
         else
@@ -460,14 +597,14 @@ void IPlot::draw()
         PAINT_ROUNDED_RECT(p,r);
     }
 
-    if(data->lines.isEmpty() || data->badData)
+    if(data->lines.isEmpty() || data->badData || !data->x().isValid() || !data->y().isValid())
     {
         p.drawText(rect(), Qt::AlignCenter, tr("No or bad data."));
         return;
     }
 
     p.setFont(CMainWindow::self().getMapFont());
-//    drawTags(p);
+    drawTags(p);
     p.setClipping(true);
     p.setClipRect(rectGraphArea);
     drawData(p);
@@ -538,11 +675,7 @@ void IPlot::drawData(QPainter& p)
         background << QPointF(right,pty);
         background << QPointF(right,bottom);
 
-//        QLinearGradient gradient(0, bottom - yaxis.val2pt(initialYMin), 0, bottom - yaxis.val2pt(initialYMax));
-//        gradient.setColorAt(0, colors[penIdx]);
-//        gradient.setColorAt(1, QColor(0,0,0,0));
         p.setPen(Qt::NoPen);
-//        p.setBrush(gradient);
         p.setBrush(colors[penIdx]);
         p.drawPolygon(background);
 
@@ -554,6 +687,7 @@ void IPlot::drawData(QPainter& p)
     }
 }
 
+
 void IPlot::drawLabels( QPainter &p )
 {
     p.setPen(Qt::darkBlue);
@@ -582,7 +716,7 @@ void IPlot::drawXScale( QPainter &p )
 
     if ( data->x().getTicType() == CPlotAxis::eNoTic )
     {
-        return ;
+        return;
     }
 
     p.setPen(Qt::darkBlue);
@@ -610,10 +744,12 @@ void IPlot::drawXScale( QPainter &p )
     qreal limMin, limMax, useMin, useMax;
     data->x().getLimits(limMin, limMax, useMin, useMax);
 
-    if((limMax - limMin) <= (useMax - useMin)) return;
+    if((limMax - limMin) <= (useMax - useMin))
+    {
+        return;
+    }
 
     qreal scale = (right - left) / (limMax - limMin);
-    //     qreal val   = data->x().pt2val(0);
 
     int x = left + (useMin - limMin) * scale;
     int y = bottom + 5;
@@ -621,7 +757,6 @@ void IPlot::drawXScale( QPainter &p )
 
     p.setPen(QPen(Qt::red,3));
     p.drawLine(x,y, x + w, y);
-
 }
 
 
@@ -630,7 +765,9 @@ void IPlot::drawYScale( QPainter &p )
     QString format_single_prec;
     QRect recText;
     if ( data->y().getTicType() == CPlotAxis::eNoTic )
-        return ;
+    {
+        return;
+    }
 
     p.setPen(Qt::darkBlue);
     recText.setHeight( fontHeight );
@@ -681,10 +818,12 @@ void IPlot::drawYScale( QPainter &p )
         t = data->y().ticmark( t );
     }
 
-    if((limMax - limMin) <= (useMax - useMin)) return;
+    if((limMax - limMin) <= (useMax - useMin))
+    {
+        return;
+    }
 
     qreal scale = (top - bottom) / (limMax - limMin);
-    //     qreal val   = data->y().pt2val(0);
 
     int x = left - 5;
     int y = bottom + (useMin - limMin) * scale;
@@ -692,7 +831,6 @@ void IPlot::drawYScale( QPainter &p )
 
     p.setPen(QPen(Qt::red,3));
     p.drawLine(x,y, x, y + h);
-
 }
 
 
@@ -799,7 +937,10 @@ void IPlot::drawYTic( QPainter &p )
 
 void IPlot::drawLegend(QPainter& p)
 {
-    if(data->lines.size() < 2) return;
+    if((data->lines.size() < 2) || (mode == eModeIcon))
+    {
+        return;
+    }
 
     int penIdx = 0;
     QFontMetrics fm(p.font());
@@ -814,23 +955,299 @@ void IPlot::drawLegend(QPainter& p)
     while(line != lines.end())
     {
         p.setPen(Qt::black);
-        p.drawText(x + 30 ,y,line->label);
+        p.drawText(x + 30,y,line->label);
         p.setPen(pens[penIdx++]);
         p.drawLine(x, y, x + 20, y);
 
         y += fm.height();
         ++line;
     }
-
 }
 
 void IPlot::drawDecoration( QPainter &p )
 {
-
     if(posMouse != NOPOINT)
     {
+        int x = posMouse.x();
         p.setPen(QPen(Qt::red,2));
-        p.drawLine(posMouse.x(), top, posMouse.x(), bottom);
+        p.drawLine(x, top, x, bottom);
+
+        foreach(const CPlotData::point_t& tag, data->tags)
+        {
+            int ptx = left + data->x().val2pt( tag.point.x() );
+
+            if(qAbs(x - ptx) < 10)
+            {
+                QFont f = CMainWindow::self().getMapFont();
+                f.setBold(true);
+                QFontMetrics fm(f);
+                QRect r = fm.boundingRect(tag.label);
+                r.moveCenter(QPoint(ptx, top - fm.height()/2 - fm.descent()));
+                r.adjust(-3,-2,3,0);
+
+                p.setPen(Qt::NoPen);
+                p.setBrush(Qt::white);
+                p.drawRoundedRect(r,3,3);
+
+                p.setFont(f);
+                p.setPen(Qt::darkBlue);
+                p.drawText(r, Qt::AlignCenter, tag.label);
+
+                break;
+            }
+        }
+    }
+
+    if((idxSel1 != NOIDX) && (idxSel2 != NOIDX) && !data->badData)
+    {
+        int penIdx = 3;
+        int ptx, pty, oldPtx, ptx1;
+
+        QPolygonF background;
+        QPolygonF foreground;
+
+        CPlotAxis& xaxis = data->x();
+        CPlotAxis& yaxis = data->y();
+
+        const QPolygonF& polyline       = data->lines.first().points.mid(idxSel1, idxSel2 - idxSel1 + 1);
+        QPolygonF::const_iterator point = polyline.begin();
+
+        ptx = left   + xaxis.val2pt( point->x() );
+        pty = bottom - yaxis.val2pt( point->y() );
+        ptx1 = oldPtx = ptx;
+
+        background << QPointF(ptx,bottom);
+        background << QPointF(ptx,pty);
+
+        foreground << QPointF(ptx,pty);
+
+        while(point != polyline.end())
+        {
+            ptx = left   + xaxis.val2pt( point->x() );
+            pty = bottom - yaxis.val2pt( point->y() );
+
+            if(oldPtx == ptx)
+            {
+                ++point;
+                continue;
+            }
+            oldPtx = ptx;
+
+            if(ptx >= left && ptx <= right)
+            {
+                background << QPointF(ptx,pty);
+                foreground << QPointF(ptx,pty);
+            }
+            ++point;
+        }
+
+        background << QPointF(ptx,pty);
+        background << QPointF(ptx,bottom);
+
+        p.setPen(Qt::NoPen);
+        p.setBrush(colors[penIdx]);
+        p.drawPolygon(background);
+
+        p.setPen(thinLine ? pensThin[penIdx] : pens[penIdx]);
+        p.setBrush(Qt::NoBrush);
+        p.drawPolyline(foreground);
+
+        p.setPen(QPen(Qt::darkBlue, 2));
+        p.drawLine(ptx1, top, ptx1, bottom);
+        p.drawLine(ptx, top, ptx, bottom);
+    }
+
+    if(!scrOptRange.isNull())
+    {
+        scrOptRange->draw(p);
     }
 }
 
+void IPlot::drawTags(QPainter& p)
+{
+    if(data->tags.isEmpty())
+    {
+        return;
+    }
+
+    int ptx, pty;
+    CPlotAxis& xaxis = data->x();
+    CPlotAxis& yaxis = data->y();
+
+    QVector<CPlotData::point_t>::const_iterator tag = data->tags.begin();
+    while(tag != data->tags.end())
+    {
+        ptx = left   + xaxis.val2pt( tag->point.x() );
+        pty = bottom - yaxis.val2pt( tag->point.y() );
+
+        if (left < ptx &&  ptx < right)
+        {
+            QPixmap icon = tag->icon.scaled(10,10, Qt::IgnoreAspectRatio, Qt::SmoothTransformation);
+            p.drawPixmap(ptx - icon.width() / 2, 2, icon);
+
+            p.setPen(QPen(Qt::white, 3));
+            if( 9 < pty)
+            {
+                if (pty > bottom)
+                {
+                    pty = bottom;
+                }
+
+                p.drawLine(ptx, top, ptx, pty);
+                p.setPen(QPen(Qt::black, 1));
+                p.drawLine(ptx, top, ptx, pty);
+            }
+        }
+        ++tag;
+    }
+}
+
+void IPlot::save(QImage& image)
+{
+    resize(image.size());
+    setSizes();
+    buffer = QImage(image.size(), QImage::Format_ARGB32);
+    draw();
+    image = buffer;
+}
+
+void IPlot::slotContextMenu(const QPoint & point)
+{
+    QPoint p = mapToGlobal(point);
+
+    actionStopRange->setEnabled((mouseClickState != eMouseClickIdle) && !(idxSel1 == NOIDX || idxSel2 == NOIDX));
+    actionPrint->setEnabled(mouseClickState != eMouseClick2nd);
+
+    menu->exec(p);
+}
+
+void IPlot::slotSave()
+{
+    SETTINGS;
+    QString path = cfg.value("Paths/lastGraphPath", QDir::homePath()).toString();
+    QString filename = QFileDialog::getSaveFileName( this, tr("Select output file"), path,"Bitmap (*.png)");
+
+    if(filename.isEmpty())
+    {
+        return;
+    }
+
+    QFileInfo fi(filename);
+    if(fi.suffix().toLower() != "png")
+    {
+        filename += ".png";
+    }
+
+    QImage img(size(), QImage::Format_ARGB32);
+    QPainter p;
+    p.begin(&img);
+    p.fillRect(rect(), QBrush(Qt::white));
+    draw(p);
+    p.end();
+
+    img.save(filename);
+
+    path = fi.absolutePath();
+    cfg.setValue("Paths/lastGraphPath", path);
+}
+
+
+void IPlot::slotHidePoints()
+{
+    trk->hideSelectedPoints();
+    scrOptRange->deleteLater();
+
+    CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
+    if(canvas)
+    {
+        canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+    }
+
+    trk->setMode(CGisItemTrk::eModeNormal, objectName());
+    mouseClickState = eMouseClickIdle;
+}
+
+void IPlot::slotShowPoints()
+{
+    trk->showSelectedPoints();
+    scrOptRange->deleteLater();
+
+    CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
+    if(canvas)
+    {
+        canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+    }
+
+    trk->setMode(CGisItemTrk::eModeNormal, objectName());
+    mouseClickState = eMouseClickIdle;
+}
+
+void IPlot::slotCopy()
+{
+    trk->copySelectedPoints();
+    scrOptRange->deleteLater();
+
+    CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
+    if(canvas)
+    {
+        canvas->slotTriggerCompleteUpdate(CCanvas::eRedrawGis);
+    }
+
+    trk->setMode(CGisItemTrk::eModeNormal, objectName());
+    mouseClickState = eMouseClickIdle;
+}
+
+void IPlot::slotStopRange()
+{
+    delete scrOptRange;
+    trk->setMode(CGisItemTrk::eModeNormal, objectName());
+    idxSel1 = idxSel2 = NOIDX;
+    mouseClickState = eMouseClickIdle;
+
+    emit sigMouseClickState(mouseClickState);
+
+    // update canvas if visible
+    CCanvas * canvas = CMainWindow::self().getVisibleCanvas();
+    if(canvas)
+    {
+        canvas->update();
+    }
+}
+
+void IPlot::setMouseRangeFocus(const CGisItemTrk::trkpt_t * ptRange1, const CGisItemTrk::trkpt_t *ptRange2)
+{
+    if(ptRange1 == 0 || ptRange2 == 0)
+    {
+        idxSel1 = idxSel2 = NOIDX;
+    }
+    else
+    {
+        if(ptRange1->idxTotal < ptRange2->idxTotal)
+        {
+            while(ptRange1->flags & CGisItemTrk::trkpt_t::eHidden)
+            {
+                ptRange1++;
+            }
+            while(ptRange2->flags & CGisItemTrk::trkpt_t::eHidden)
+            {
+                ptRange2--;
+            }
+            idxSel1 = ptRange1->idxVisible;
+            idxSel2 = ptRange2->idxVisible;
+        }
+        else
+        {
+            while(ptRange1->flags & CGisItemTrk::trkpt_t::eHidden)
+            {
+                ptRange1--;
+            }
+            while(ptRange2->flags & CGisItemTrk::trkpt_t::eHidden)
+            {
+                ptRange2++;
+            }
+            idxSel1 = ptRange2->idxVisible;
+            idxSel2 = ptRange1->idxVisible;
+        }
+    }
+    update();
+}
